@@ -99,6 +99,8 @@ LIBS=(
 	libssp-0.dll\
 	libdeflate.dll\
 	libwebp-7.dll\
+	libjbig-0.dll\
+	libLerc.dll\
 	gspawn-win64-helper.exe\
 	gspawn-win64-helper-console.exe
 )
