@@ -8,7 +8,6 @@
 #include <list>
 #include "nlohmann/json.hpp"
 #include "pool/ipool.hpp"
-#include "pool/pool_cached.hpp"
 #include "common/junction_util.hpp"
 #include "util/bbox_accumulator.hpp"
 
@@ -30,18 +29,18 @@ json Board::StackupLayer::serialize() const
     return j;
 }
 
-BoardColors::BoardColors() : solder_mask({0, .5, 0}), substrate({.2, .15, 0})
+BoardColors::BoardColors() : solder_mask({0, .5, 0}), silkscreen({1, 1, 1}), substrate({.2, .15, 0})
 {
 }
 
-static const unsigned int app_version = 2;
+static const unsigned int app_version = 4;
 
 unsigned int Board::get_app_version()
 {
     return app_version;
 }
 
-Board::Board(const UUID &uu, const json &j, Block &iblock, IPool &pool, ViaPadstackProvider &vpp)
+Board::Board(const UUID &uu, const json &j, Block &iblock, IPool &pool)
     : uuid(uu), block(&iblock), name(j.at("name").get<std::string>()), version(app_version, j),
       n_inner_layers(j.value("n_inner_layers", 0))
 {
@@ -128,7 +127,7 @@ Board::Board(const UUID &uu, const json &j, Block &iblock, IPool &pool, ViaPadst
         const json &o = j["vias"];
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID(it.key());
-            load_and_log(vias, ObjectType::VIA, std::forward_as_tuple(u, it.value(), this, &vpp),
+            load_and_log(vias, ObjectType::VIA, std::forward_as_tuple(u, it.value(), pool, this),
                          Logger::Domain::BOARD);
         }
     }
@@ -237,6 +236,9 @@ Board::Board(const UUID &uu, const json &j, Block &iblock, IPool &pool, ViaPadst
         try {
             const auto &o = j.at("colors");
             colors.solder_mask = color_from_json(o.at("solder_mask"));
+            if (o.count("silkscreen")) {
+                colors.silkscreen = color_from_json(o.at("silkscreen"));
+            }
             colors.substrate = color_from_json(o.at("substrate"));
         }
         catch (const std::exception &e) {
@@ -267,15 +269,23 @@ Board::Board(const UUID &uu, const json &j, Block &iblock, IPool &pool, ViaPadst
             Logger::log_warning("couldn't load p&p export settings", Logger::Domain::BOARD, e.what());
         }
     }
+    if (j.count("grid_settings")) {
+        try {
+            grid_settings = GridSettings(j.at("grid_settings"));
+        }
+        catch (const std::exception &e) {
+            Logger::log_warning("couldn't load grid settings", Logger::Domain::BOARD, e.what());
+        }
+    }
     fab_output_settings.update_for_board(*this);
     update_pdf_export_settings(pdf_export_settings);
     update_refs(); // fill in smashed texts
 }
 
-Board Board::new_from_file(const std::string &filename, Block &block, IPool &pool, ViaPadstackProvider &vpp)
+Board Board::new_from_file(const std::string &filename, Block &block, IPool &pool)
 {
     auto j = load_json_from_file(filename);
-    return Board(UUID(j.at("uuid").get<std::string>()), j, block, pool, vpp);
+    return Board(UUID(j.at("uuid").get<std::string>()), j, block, pool);
 }
 
 
@@ -283,12 +293,19 @@ Board::Board(const UUID &uu, Block &bl) : uuid(uu), block(&bl), version(app_vers
 {
     rules.add_rule(RuleID::CLEARANCE_COPPER);
     rules.add_rule(RuleID::CLEARANCE_COPPER_OTHER);
-    rules.add_rule(RuleID::TRACK_WIDTH);
     rules.add_rule(RuleID::PLANE);
-    auto r = dynamic_cast<RuleTrackWidth *>(rules.get_rules_nc(RuleID::TRACK_WIDTH).begin()->second);
-    r->widths.emplace(std::piecewise_construct, std::forward_as_tuple(0), std::forward_as_tuple());
-    // TBD: inner layers
-    r->widths.emplace(std::piecewise_construct, std::forward_as_tuple(-100), std::forward_as_tuple());
+    {
+        rules.add_rule(RuleID::TRACK_WIDTH);
+        auto &r = dynamic_cast<RuleTrackWidth &>(*rules.get_rules_nc(RuleID::TRACK_WIDTH).begin()->second);
+        r.widths.emplace(std::piecewise_construct, std::forward_as_tuple(0), std::forward_as_tuple());
+        // TBD: inner layers
+        r.widths.emplace(std::piecewise_construct, std::forward_as_tuple(-100), std::forward_as_tuple());
+    }
+    {
+        rules.add_rule(RuleID::LAYER_PAIR);
+        auto &r = dynamic_cast<RuleLayerPair &>(*rules.get_rules_nc(RuleID::LAYER_PAIR).begin()->second);
+        r.layers = std::make_pair(BoardLayers::TOP_COPPER, BoardLayers::BOTTOM_COPPER);
+    }
 }
 
 Junction *Board::get_junction(const UUID &uu)
@@ -311,10 +328,10 @@ Board::Board(const Board &brd, CopyMode copy_mode)
       junctions(brd.junctions), tracks(brd.tracks), texts(brd.texts), lines(brd.lines), arcs(brd.arcs),
       planes(brd.planes), keepouts(brd.keepouts), dimensions(brd.dimensions), connection_lines(brd.connection_lines),
       included_boards(brd.included_boards), board_panels(brd.board_panels), pictures(brd.pictures), decals(brd.decals),
-      warnings(brd.warnings), rules(brd.rules), fab_output_settings(brd.fab_output_settings), airwires(brd.airwires),
-      stackup(brd.stackup), colors(brd.colors), pdf_export_settings(brd.pdf_export_settings),
-      step_export_settings(brd.step_export_settings), pnp_export_settings(brd.pnp_export_settings),
-      version(brd.version), n_inner_layers(brd.n_inner_layers)
+      warnings(brd.warnings), rules(brd.rules), fab_output_settings(brd.fab_output_settings),
+      grid_settings(brd.grid_settings), airwires(brd.airwires), stackup(brd.stackup), colors(brd.colors),
+      pdf_export_settings(brd.pdf_export_settings), step_export_settings(brd.step_export_settings),
+      pnp_export_settings(brd.pnp_export_settings), version(brd.version), n_inner_layers(brd.n_inner_layers)
 {
     if (copy_mode == CopyMode::DEEP) {
         packages = brd.packages;
@@ -612,7 +629,7 @@ void Board::propagate_nets()
 
 void Board::vacuum_junctions()
 {
-    map_erase_if(junctions, [this](auto &x) {
+    map_erase_if(junctions, [](auto &x) {
         const BoardJunction &ju = x.second;
         return (ju.connected_lines.size() == 0) && (ju.connected_arcs.size() == 0) && (ju.connected_tracks.size() == 0)
                && (ju.connected_vias.size() == 0);
@@ -622,13 +639,13 @@ void Board::vacuum_junctions()
 
 static std::string replace_text_map(const std::string &s, const std::map<std::string, std::string> &m)
 {
-    auto fn = [m](const auto &k) -> std::string {
+    auto fn = [&m](const auto &k) -> std::optional<std::string> {
         if (m.count(k))
             return m.at(k);
         else
-            return "";
+            return {};
     };
-    return replace_placeholders(s, fn, true);
+    return interpolate_text(s, fn);
 }
 
 void Board::expand()
@@ -680,6 +697,11 @@ void Board::expand_some()
         if (it.second.padstack.type != Padstack::Type::HOLE && it.second.padstack.type != Padstack::Type::MECHANICAL) {
             warnings.emplace_back(it.second.placement.shift, "Illegal padstack type");
         }
+        for (auto p : it.second.pool_padstack->parameters_required) {
+            if (it.second.parameter_set.count(p) == 0) {
+                warnings.emplace_back(it.second.placement.shift, "missing parameter " + parameter_id_to_name(p));
+            }
+        }
     }
 
     for (const auto &it : junctions) {
@@ -881,7 +903,7 @@ void Board::smash_package_silkscreen_graphics(BoardPackage *pkg)
     auto tr = pkg->placement;
     if (pkg->flip)
         tr.invert_angle();
-    auto get_junction = [&junction_xlat, this, pkg, tr](Junction *ju) -> Junction * {
+    auto get_junction = [&junction_xlat, this, tr](Junction *ju) -> Junction * {
         if (junction_xlat.count(ju)) {
             return junction_xlat.at(ju);
         }
@@ -1033,6 +1055,10 @@ std::pair<Coordi, Coordi> Board::get_bbox() const
             acc.accumulate(it.second.get_bbox());
         }
     }
+    for (const auto &[_, package] : packages) {
+        acc.accumulate(package.get_bbox());
+    }
+
     return acc.get_or({{-10_mm, -10_mm}, {10_mm, 10_mm}});
 }
 
@@ -1040,7 +1066,8 @@ std::map<const BoardPackage *, PnPRow> Board::get_PnP(const PnPExportSettings &s
 {
     std::map<const BoardPackage *, PnPRow> r;
     for (const auto &it : packages) {
-        if (it.second.component->nopopulate && !settings.include_nopopulate) {
+        if ((it.second.component->nopopulate && !settings.include_nopopulate)
+            || it.second.component->part->get_flag(Part::Flag::EXCLUDE_PNP)) {
             continue;
         }
 
@@ -1074,12 +1101,14 @@ json Board::serialize() const
     {
         json o;
         o["solder_mask"] = color_to_json(colors.solder_mask);
+        o["silkscreen"] = color_to_json(colors.silkscreen);
         o["substrate"] = color_to_json(colors.substrate);
         j["colors"] = o;
     }
     j["pdf_export_settings"] = pdf_export_settings.serialize_board();
     j["step_export_settings"] = step_export_settings.serialize();
     j["pnp_export_settings"] = pnp_export_settings.serialize();
+    j["grid_settings"] = grid_settings.serialize();
     j["polygons"] = json::object();
     for (const auto &it : polygons) {
         j["polygons"][(std::string)it.first] = it.second.serialize();
@@ -1185,7 +1214,7 @@ ItemSet Board::get_pool_items_used() const
         }
     }
     for (const auto &it_via : vias) {
-        items_needed.emplace(ObjectType::PADSTACK, it_via.second.vpp_padstack->uuid);
+        items_needed.emplace(ObjectType::PADSTACK, it_via.second.pool_padstack->uuid);
     }
     for (const auto &it_hole : holes) {
         items_needed.emplace(ObjectType::PADSTACK, it_hole.second.pool_padstack->uuid);
