@@ -330,16 +330,18 @@ void Canvas::begin_group(int layer)
 void Canvas::end_group()
 {
     auto new_size = group_tris->size();
-    for (auto it : object_ref_idx) {
-        auto &layers = *it;
-        if (layers.count(group_layer)) {
-            auto &idxs = layers[group_layer];
-            idxs.second = new_size - 1;
-        }
-        else {
-            auto &idxs = layers[group_layer];
-            idxs.first = group_size;
-            idxs.second = new_size - 1;
+    if (new_size != group_size) { // something was drawn
+        for (auto it : object_ref_idx) {
+            auto &layers = *it;
+            if (layers.count(group_layer)) {
+                auto &idxs = layers[group_layer];
+                idxs.second = new_size - 1;
+            }
+            else {
+                auto &idxs = layers[group_layer];
+                idxs.first = group_size;
+                idxs.second = new_size - 1;
+            }
         }
     }
     group_tris = nullptr;
@@ -374,7 +376,7 @@ std::pair<Coordf, Coordf> Canvas::get_bbox(bool visible_only) const
                 }
                 else if (it_info.flags & TriangleInfo::FLAG_BUTT) { // line
                     const Coordf v = points.at(1) - points.at(0);
-                    const Coordf vn = Coordf(-v.y, v.x) / sqrt(v.mag_sq());
+                    const Coordf vn = Coordf(-v.y, v.x).normalize();
                     const Coordf w = vn * (it2.x2 / 2);
                     acc.accumulate(points.at(0) - w);
                     acc.accumulate(points.at(1) - w);
@@ -390,7 +392,7 @@ std::pair<Coordf, Coordf> Canvas::get_bbox(bool visible_only) const
         }
     }
     const auto [a, b] = acc.get_or_0();
-    if (sqrt((b - a).mag_sq()) < .1_mm)
+    if ((b - a).mag() < .1_mm)
         return std::make_pair(Coordf(-5_mm, -5_mm), Coordf(5_mm, 5_mm));
     else
         return std::make_pair(a, b);
