@@ -119,8 +119,8 @@ protected:
     void render(const class Shape &shape, bool interactive = true);
     void render(const class Hole &hole, bool interactive = true);
     void render(const class Package &package, bool interactive = true, bool smashed = false,
-                bool omit_silkscreen = false, bool omit_outline = false);
-    void render_pad_overlay(const class Pad &pad);
+                bool omit_silkscreen = false, bool omit_outline = false, bool on_panel = false);
+    void render_pad_overlay(const class Pad &pad, bool interactive);
     void render(const class Pad &pad);
     enum class OutlineMode { INCLUDE, OMIT };
     void render(const class Board &brd, bool interactive = true, PanelMode mode = PanelMode::INCLUDE,
@@ -158,10 +158,18 @@ protected:
                                         int layer, uint64_t width);
     void draw_arc0(const Coord<float> &center, float radius0, float a0, float a1, ColorP color, int layer,
                    uint64_t width);
-    std::pair<Coordf, Coordf> draw_text0(const Coordf &p, float size, const std::string &rtext, int angle, bool flip,
-                                         TextOrigin origin, ColorP color, int layer = 10000, uint64_t width = 0,
-                                         bool draw = true, TextData::Font font = TextData::Font::SIMPLEX,
-                                         bool center = false, bool mirror = false);
+
+    struct TextOptions {
+        uint64_t width = 0;
+        bool flip = false;
+        bool mirror = false;
+        bool draw = true;
+        TextData::Font font = TextData::Font::SIMPLEX;
+        bool center = false;
+        bool allow_upside_down = false;
+    };
+    std::pair<Coordf, Coordf> draw_text(const Coordf &p, float size, const std::string &rtext, int angle,
+                                        TextOrigin origin, ColorP color, int layer, const TextOptions &opts);
 
     virtual void draw_bitmap_text(const Coordf &p, float scale, const std::string &rtext, int angle, ColorP color,
                                   int layer)
