@@ -1,6 +1,6 @@
 #include "canvas_gl.hpp"
 #include "gl_util.hpp"
-#include "util/util.hpp"
+#include "util/geom_util.hpp"
 #include <algorithm>
 #include <epoxy/gl.h>
 #include <iostream>
@@ -111,7 +111,6 @@ void CanvasGL::zoom_gesture_update_cb(GdkEventSequence *seq)
     offset = gesture_zoom_offset_orig + Coordf(cx, cy) - gesture_zoom_pos_orig;
     update_viewmat();
     s_signal_scale_changed.emit();
-    queue_draw();
 }
 
 void CanvasGL::drag_gesture_begin_cb(GdkEventSequence *seq)
@@ -131,7 +130,6 @@ void CanvasGL::drag_gesture_update_cb(GdkEventSequence *seq)
     if (gesture_drag->get_offset(x, y)) {
         offset = gesture_drag_offset_orig + Coordf(x, y);
         update_viewmat();
-        queue_draw();
     }
 }
 
@@ -425,7 +423,6 @@ void CanvasGL::center_and_zoom(const Coordf &center, float sc)
     offset -= (c - m);
     update_viewmat();
     s_signal_scale_changed.emit();
-    queue_draw();
 }
 
 void CanvasGL::zoom_to_bbox(const Coordf &a_a, const Coordf &a_b)
@@ -446,11 +443,15 @@ void CanvasGL::zoom_to_bbox(const std::pair<Coordf, Coordf> &bb)
     zoom_to_bbox(bb.first, bb.second);
 }
 
-void CanvasGL::zoom_to_center(float factor)
+void CanvasGL::zoom_to(const Coordf &c, float inc)
 {
     if (!can_set_scale())
         return;
-    set_scale(m_width / 2, m_height / 2, scale * factor);
+
+    if (smooth_zoom)
+        start_smooth_zoom(c, inc);
+    else
+        set_scale(c.x, c.y, scale * pow(zoom_base, inc));
 }
 
 void CanvasGL::ensure_min_size(float w, float h)
@@ -472,6 +473,7 @@ void CanvasGL::update_viewmat()
     viewmat = glm::scale(glm::rotate(glm::translate(glm::mat3(1), glm::vec2(offset.x, offset.y)), -view_angle),
                          glm::vec2(scale_x, -scale));
     viewmat_noflip = glm::scale(glm::translate(glm::mat3(1), glm::vec2(offset.x, offset.y)), glm::vec2(scale, -scale));
+    queue_draw();
 }
 
 void CanvasGL::push()
@@ -497,6 +499,11 @@ void CanvasGL::update_markers()
     request_push(PF_MARKER);
 }
 
+void CanvasGL::append_target(const Target &trg)
+{
+    targets.emplace_back(trg);
+}
+
 void CanvasGL::set_flip_view(bool fl)
 {
     auto toggled = fl != flip_view;
@@ -597,10 +604,10 @@ std::set<SelectableRef> CanvasGL::get_selection_at(const Coordi &c)
     std::list<std::pair<const Selectable &, const SelectableRef &>> sel;
     unsigned int i = 0;
     for (auto &it : selectables.items) {
-        if (it.inside(c, appearance.min_selectable_size / scale)
-            && selection_filter.can_select(selectables.items_ref[i])) {
+        const auto &sr = selectables.items_ref.at(i);
+        if (it.inside(c, appearance.min_selectable_size / scale) && selection_filter.can_select(sr)) {
             // in_selection.insert(selectables.items_ref[i]);
-            sel.emplace_back(it, selectables.items_ref.at(i));
+            sel.emplace_back(it, sr);
         }
         i++;
     }
@@ -611,7 +618,7 @@ std::set<SelectableRef> CanvasGL::get_selection_at(const Coordi &c)
         assert(r != sel.end());
         return {r->second};
     }
-    else if (n_line == 1) { // only one line, select that one
+    else if (n_line == 1 && n_point == 0) { // only one line, select that one
         auto r = std::find_if(sel.begin(), sel.end(), [](const auto &x) { return x.first.is_line(); });
         assert(r != sel.end());
         return {r->second};
