@@ -125,7 +125,7 @@ public:
     void zoom_to_bbox(const Coordf &a, const Coordf &b);
     void zoom_to_bbox(const std::pair<Coordf, Coordf> &bb);
     void ensure_min_size(float w, float h);
-    void zoom_to_center(float factor);
+    void zoom_to(const Coordf &c, float inc);
 
     Glib::PropertyProxy<int> property_work_layer()
     {
@@ -190,6 +190,8 @@ public:
 
     int get_last_grid_div() const;
 
+    void append_target(const Target &target);
+
 protected:
     void push() override;
     void request_push() override;
@@ -247,6 +249,7 @@ private:
     void pan_drag_move(GdkEventMotion *motion_event);
     void pan_drag_move(GdkEventScroll *scroll_event);
     void pan_zoom(GdkEventScroll *scroll_event, bool to_cursor = true);
+    void start_smooth_zoom(const Coordf &c, float inc);
     void cursor_move(GdkEvent *motion_event);
     void hover_prelight_update(GdkEvent *motion_event);
     bool pan_dragging = false;
