@@ -24,7 +24,7 @@
 #include "selection_filter.hpp"
 #include "target.hpp"
 #include "util/placement.hpp"
-#include "util/util.hpp"
+#include "util/geom_util.hpp"
 #include "board/board_panel.hpp"
 #include "common/picture.hpp"
 #include "util/polygon_arc_removal_proxy.hpp"
@@ -77,8 +77,8 @@ void Canvas::render(const SchematicJunction &junc)
         draw_line(junc.position, junc.position + Coordi(0, 1000), c, 0, true, 0.75_mm);
         img_line(junc.position, junc.position + Coordi(0, 1000), 0.75_mm, 0, true);
     }
-    else if (junc.connected_bus_labels.size() || junc.connected_bus_rippers.size()
-             || junc.connected_net_labels.size()) {
+    else if (junc.connected_bus_labels.size() || junc.connected_bus_rippers.size() || junc.connected_net_labels.size()
+             || junc.connected_lines.size() || junc.connected_arcs.size()) {
         // nop
     }
     else {
@@ -131,8 +131,8 @@ void Canvas::render(const PowerSymbol &sym)
         }
 
         if (sym.junction->net->power_symbol_name_visible)
-            draw_text0(sym.junction->position + text_offset, 1.5_mm, sym.junction->net->name, text_angle, false,
-                       TextOrigin::CENTER, c, 0);
+            draw_text(sym.junction->position + text_offset, 1.5_mm, sym.junction->net->name, text_angle,
+                      TextOrigin::CENTER, c, 0, {});
     } break;
 
     case Net::PowerSymbolStyle::DOT:
@@ -183,8 +183,8 @@ void Canvas::render(const PowerSymbol &sym)
         }
 
         if (sym.junction->net->power_symbol_name_visible)
-            draw_text0(sym.junction->position + text_offset, 1.5_mm, sym.junction->net->name, text_angle, false,
-                       TextOrigin::CENTER, c, 0);
+            draw_text(sym.junction->position + text_offset, 1.5_mm, sym.junction->net->name, text_angle,
+                      TextOrigin::CENTER, c, 0, {});
     }
     }
 
@@ -280,8 +280,8 @@ void Canvas::render(const Track &track, bool interactive)
         && track.to.is_junc() && (!show_text_in_vias || (!track.from.junc->has_via && !track.to.junc->has_via))) {
         auto overlay_layer = get_overlay_layer(track.layer, true);
         set_lod_size(width);
-        auto vec = (track.from.get_position() - track.to.get_position());
-        auto length = sqrt(vec.mag_sq());
+        const auto vec = (track.from.get_position() - track.to.get_position());
+        const auto length = vec.magd();
         Placement p;
         p.set_angle_rad(get_view_angle());
         if (get_flip_view())
@@ -421,21 +421,23 @@ void Canvas::render(const SymbolPin &pin, bool interactive, ColorP co)
                                 && (pin_orientation == Orientation::LEFT || pin_orientation == Orientation::RIGHT));
 
         if (draw_in_line) {
-            draw_text0(p_name, 1.5_mm, pin.name, orientation_to_angle(name_orientation), false, TextOrigin::CENTER,
-                       c_name, 0);
+            draw_text(p_name, 1.5_mm, pin.name, orientation_to_angle(name_orientation), TextOrigin::CENTER, c_name, 0,
+                      {});
         }
         else { // draw perp
             Placement tr;
             tr.set_angle(orientation_to_angle(pin_orientation));
             auto shift = tr.transform(Coordi(-1_mm, 0));
-            draw_text0(p_name + shift, 1.5_mm, pin.name, orientation_to_angle(name_orientation) + 16384, false,
-                       TextOrigin::CENTER, c_name, 0, 0, true, TextData::Font::SIMPLEX, true);
+            TextOptions opts;
+            opts.center = true;
+            draw_text(p_name + shift, 1.5_mm, pin.name, orientation_to_angle(name_orientation) + 16384,
+                      TextOrigin::CENTER, c_name, 0, opts);
         }
     }
     std::pair<Coordf, Coordf> pad_extents;
     if (interactive || pin.pad_visible) {
-        pad_extents = draw_text0(p_pad, 0.75_mm, pin.pad, orientation_to_angle(pad_orientation), false,
-                                 TextOrigin::BASELINE, c_pad, 0);
+        pad_extents = draw_text(p_pad, 0.75_mm, pin.pad, orientation_to_angle(pad_orientation), TextOrigin::BASELINE,
+                                c_pad, 0, {});
     }
 
     transform_save();
@@ -573,7 +575,7 @@ void Canvas::render(const SymbolPin &pin, bool interactive, ColorP co)
 
     case SymbolPin::ConnectorStyle::NC:
         draw_cross(p0, 0.25_mm, c_main, 0, false);
-        draw_text0(p_nc, 1.5_mm, "NC", orientation_to_angle(nc_orientation), false, TextOrigin::CENTER, c_pin, 0);
+        draw_text(p_nc, 1.5_mm, "NC", orientation_to_angle(nc_orientation), TextOrigin::CENTER, c_pin, 0, {});
         break;
 
     case SymbolPin::ConnectorStyle::NONE:
@@ -589,23 +591,23 @@ void Canvas::render(const SymbolPin &pin, bool interactive, ColorP co)
     img_auto_line = false;
 }
 
-static int64_t sq(int64_t x)
-{
-    return x * x;
-}
-
 void Canvas::render(const Arc &arc, bool interactive, ColorP co)
 {
     Coordf a(arc.from->position); // ,b,c;
     Coordf b(arc.to->position);   // ,b,c;
     Coordf c = project_onto_perp_bisector(a, b, arc.center->position);
-    float radius0 = sqrt(sq(c.x - a.x) + sq(c.y - a.y));
-    float a0 = atan2f(a.y - c.y, a.x - c.x);
-    float a1 = atan2f(b.y - c.y, b.x - c.x);
-    auto bb = draw_arc2(c, radius0, a0, a1, co, arc.layer, arc.width);
+    const float radius0 = (c - a).mag();
+    const float a0 = c2pi(atan2f(a.y - c.y, a.x - c.x));
+    const float a1 = c2pi(atan2f(b.y - c.y, b.x - c.x));
+    const float dphi = c2pi(a1 - a0);
+
+    draw_arc2(c, radius0, a0, a1, co, arc.layer, arc.width);
     Coordf t(radius0, radius0);
-    if (interactive)
-        selectables.append(arc.uuid, ObjectType::ARC, c, bb.first, bb.second, 0, arc.layer);
+    if (interactive) {
+        const float ax = std::min(asin(arc.width / 2 / radius0), static_cast<float>((2 * M_PI - dphi) / 2 - .1e-4));
+        selectables.append_arc(arc.uuid, ObjectType::ARC, c, radius0 - arc.width / 2, radius0 + arc.width / 2, a0 - ax,
+                               a1 + ax, 0, arc.layer);
+    }
 }
 
 void Canvas::render(const SchematicSymbol &sym)
@@ -651,8 +653,16 @@ void Canvas::render(const Text &text, bool interactive, ColorP co)
     img_patch_type(PatchType::TEXT);
     triangle_type_current = TriangleInfo::Type::TEXT;
     object_ref_push(ObjectType::TEXT, text.uuid);
-    auto extents = draw_text0(transform.shift, text.size, text.overridden ? text.text_override : text.text, angle, rev,
-                              text.origin, co, text.layer, text.width, true, text.font, false, transform.mirror);
+
+    TextOptions opts;
+    opts.flip = rev;
+    opts.mirror = transform.mirror;
+    opts.font = text.font;
+    opts.width = text.width;
+    opts.allow_upside_down = text.allow_upside_down;
+
+    const auto extents = draw_text(transform.shift, text.size, text.overridden ? text.text_override : text.text, angle,
+                                   text.origin, co, text.layer, opts);
     triangle_type_current = TriangleInfo::Type::NONE;
     object_ref_pop();
     // img_text_extents(text, extents);
@@ -704,8 +714,8 @@ void Canvas::render(const NetLabel &label)
                            extents.second);
     }
     else {
-        auto extents = draw_text0(label.junction->position, label.size, txt, orientation_to_angle(label.orientation),
-                                  false, TextOrigin::BASELINE, c);
+        const auto extents = draw_text(label.junction->position, label.size, txt,
+                                       orientation_to_angle(label.orientation), TextOrigin::BASELINE, c, 0, {});
         selectables.append(label.uuid, ObjectType::NET_LABEL, label.junction->position + Coordi(0, 1000000),
                            extents.first, extents.second);
     }
@@ -749,14 +759,16 @@ void Canvas::render(const BusRipper &ripper)
     default:
         angle = 0;
     }
-    auto extents = draw_text0(connector_pos + Coordi(0, 0.5_mm), 1.5_mm, ripper.bus_member->name, angle, false,
-                              TextOrigin::BASELINE, c);
+    const auto extents = draw_text(connector_pos + Coordi(0, 0.5_mm), 1.5_mm, ripper.bus_member->name, angle,
+                                   TextOrigin::BASELINE, c, 0, {});
     targets.emplace_back(ripper.uuid, ObjectType::BUS_RIPPER, connector_pos);
     selectables.append(ripper.uuid, ObjectType::BUS_RIPPER, connector_pos, extents.first, extents.second);
 }
 
 void Canvas::render(const Warning &warn)
 {
+    if (img_mode)
+        return;
     draw_error(warn.position, 2e5, warn.text);
 }
 
@@ -827,12 +839,13 @@ void Canvas::render(const Polygon &ipoly, bool interactive, ColorP co)
         object_ref_pop();
     }
     else { // normal polygon
-        triangle_type_current = TriangleInfo::Type::POLYGON;
+        if (triangle_type_current == TriangleInfo::Type::NONE)
+            triangle_type_current = TriangleInfo::Type::POLYGON;
         begin_group(poly.layer);
         if (poly_is_rect(poly)) {
             const Coordf p0 = (poly.get_vertex(0).position + poly.get_vertex(1).position) / 2;
             const Coordf p1 = (poly.get_vertex(2).position + poly.get_vertex(3).position) / 2;
-            const float width = sqrt((poly.get_vertex(0).position - poly.get_vertex(1).position).mag_sq());
+            const float width = (poly.get_vertex(0).position - poly.get_vertex(1).position).magd();
             add_triangle(poly.layer, transform.transform(p0), transform.transform(p1), Coordf(width, 1), co,
                          TriangleInfo::FLAG_BUTT);
         }
@@ -850,7 +863,7 @@ void Canvas::render(const Polygon &ipoly, bool interactive, ColorP co)
             }
             std::list<TPPLPoly> outpolys;
             TPPLPartition part;
-            po.SetOrientation(TPPL_CCW);
+            po.SetOrientation(TPPL_ORIENTATION_CCW);
             part.Triangulate_EC(&po, &outpolys);
             for (auto &tri : outpolys) {
                 assert(tri.GetNumPoints() == 3);
@@ -865,44 +878,41 @@ void Canvas::render(const Polygon &ipoly, bool interactive, ColorP co)
             }
         }
         end_group();
-        triangle_type_current = TriangleInfo::Type::NONE;
+        if (triangle_type_current == TriangleInfo::Type::POLYGON)
+            triangle_type_current = TriangleInfo::Type::NONE;
     }
 
     if (interactive) {
         selectables.group_begin();
-        const Polygon::Vertex *v_last = nullptr;
-        {
-            size_t i = 0;
-            for (const auto &it : ipoly.vertices) {
-                if (v_last) {
-                    auto center = (v_last->position + it.position) / 2;
-                    if (v_last->type != Polygon::Vertex::Type::ARC) {
-                        selectables.append_line(poly.uuid, ObjectType::POLYGON_EDGE, v_last->position, it.position, 0,
-                                                i - 1, poly.layer);
+        for (int i = 0; i < static_cast<int>(ipoly.vertices.size()); i++) {
+            const auto &v = ipoly.get_vertex(i);
+            const auto &v_last = ipoly.get_vertex(i - 1);
+            int i_last = i - 1;
+            if (i_last < 0)
+                i_last += ipoly.vertices.size();
+            if (v_last.type != Polygon::Vertex::Type::ARC) {
+                auto center = (v.position + v_last.position) / 2;
+                selectables.append_line(ipoly.uuid, ObjectType::POLYGON_EDGE, v_last.position, v.position, 0, i_last,
+                                        ipoly.layer);
+                targets.emplace_back(ipoly.uuid, ObjectType::POLYGON_EDGE, center, i_last, ipoly.layer);
+            }
+            else {
+                const Coordf b = v.position;
+                const Coordf a = v_last.position;
+                const Coordf c = project_onto_perp_bisector(a, b, v_last.arc_center);
+                const auto r = (a - c).mag();
+                float a0 = atan2f(a.y - c.y, a.x - c.x);
+                float a1 = atan2f(b.y - c.y, b.x - c.x);
+                if (v_last.arc_reverse)
+                    std::swap(a0, a1);
+                selectables.append_arc(ipoly.uuid, ObjectType::POLYGON_EDGE, c, r, r, a0, a1, i_last, ipoly.layer);
+            }
 
-                        targets.emplace_back(poly.uuid, ObjectType::POLYGON_EDGE, center, i - 1, poly.layer);
-                    }
-                }
-                v_last = &it;
-                i++;
-            }
-            if (ipoly.vertices.back().type != Polygon::Vertex::Type::ARC) {
-                auto center = (ipoly.vertices.front().position + ipoly.vertices.back().position) / 2;
-                targets.emplace_back(poly.uuid, ObjectType::POLYGON_EDGE, center, i - 1, poly.layer);
-                selectables.append_line(poly.uuid, ObjectType::POLYGON_EDGE, poly.vertices.front().position,
-                                        ipoly.vertices.back().position, 0, i - 1, poly.layer);
-            }
-        }
-        {
-            size_t i = 0;
-            for (const auto &it : ipoly.vertices) {
-                selectables.append(poly.uuid, ObjectType::POLYGON_VERTEX, it.position, i, poly.layer);
-                targets.emplace_back(poly.uuid, ObjectType::POLYGON_VERTEX, it.position, i, poly.layer);
-                if (it.type == Polygon::Vertex::Type::ARC) {
-                    selectables.append(poly.uuid, ObjectType::POLYGON_ARC_CENTER, it.arc_center, i, poly.layer);
-                    targets.emplace_back(poly.uuid, ObjectType::POLYGON_ARC_CENTER, it.arc_center, i, poly.layer);
-                }
-                i++;
+            selectables.append(ipoly.uuid, ObjectType::POLYGON_VERTEX, v.position, i, ipoly.layer);
+            targets.emplace_back(ipoly.uuid, ObjectType::POLYGON_VERTEX, v.position, i, ipoly.layer);
+            if (v.type == Polygon::Vertex::Type::ARC) {
+                selectables.append(ipoly.uuid, ObjectType::POLYGON_ARC_CENTER, v.arc_center, i, ipoly.layer);
+                targets.emplace_back(ipoly.uuid, ObjectType::POLYGON_ARC_CENTER, v.arc_center, i, ipoly.layer);
             }
         }
         selectables.group_end();
@@ -962,22 +972,23 @@ void Canvas::render(const Hole &hole, bool interactive)
     transform.accumulate(hole.placement);
     const int64_t r = hole.diameter / 2;
     const int64_t l = std::max((int64_t)hole.length / 2 - r, (int64_t)0);
+    const int layer = get_overlay_layer({BoardLayers::TOP_COPPER, BoardLayers::BOTTOM_COPPER});
     if (hole.shape == Hole::Shape::ROUND) {
-        draw_circle(Coordf(), r);
+        draw_circle(Coordf(), r, co, layer);
         if (hole.plated) {
-            draw_circle(Coordf(), r * 0.9);
+            draw_circle(Coordf(), r * 0.9, co, layer);
         }
         float x = hole.diameter / 2 / M_SQRT2;
-        draw_line(Coordi(-x, -x), Coordi(x, x), co);
-        draw_line(Coordi(x, -x), Coordi(-x, x), co);
+        draw_line(Coordi(-x, -x), Coordi(x, x), co, layer);
+        draw_line(Coordi(x, -x), Coordi(-x, x), co, layer);
         if (interactive)
             selectables.append(hole.uuid, ObjectType::HOLE, Coordi(), Coordi(-r, -r), Coordi(r, r));
     }
     else if (hole.shape == Hole::Shape::SLOT) {
-        draw_circle(Coordi(-l, 0), r, co);
-        draw_circle(Coordi(l, 0), r, co);
-        draw_line(Coordi(-l, -r), Coordi(l, -r), co);
-        draw_line(Coordi(-l, r), Coordi(l, r), co);
+        draw_circle(Coordi(-l, 0), r, co, layer);
+        draw_circle(Coordi(l, 0), r, co, layer);
+        draw_line(Coordi(-l, -r), Coordi(l, -r), co, layer);
+        draw_line(Coordi(-l, r), Coordi(l, r), co, layer);
         if (interactive)
             selectables.append(hole.uuid, ObjectType::HOLE, Coordi(), Coordi(-l - r, -r), Coordi(l + r, +r));
     }
@@ -995,7 +1006,9 @@ void Canvas::render(const Pad &pad)
         img_patch_type(PatchType::PAD_TH);
     else
         img_patch_type(PatchType::PAD);
+    triangle_type_current = TriangleInfo::Type::PAD;
     render(pad.padstack, false);
+    triangle_type_current = TriangleInfo::Type::NONE;
     img_patch_type(PatchType::OTHER);
     img_net(nullptr);
     transform_restore();
@@ -1102,13 +1115,21 @@ void Canvas::render(const Padstack &padstack, bool interactive)
     img_set_padstack(false);
 }
 
-void Canvas::render_pad_overlay(const Pad &pad)
+void Canvas::render_pad_overlay(const Pad &pad, bool interactive)
 {
     if (img_mode)
         return;
+    if (pad.padstack.type == Padstack::Type::MECHANICAL && !interactive)
+        return;
     transform_save();
     transform.accumulate(pad.placement);
-    auto bb = pad.padstack.get_bbox(true); // only copper
+    auto bb = pad.padstack.get_bbox(true);  // only copper
+    if (bb.second - bb.first == Coordi()) { // empty bbox
+        bb = pad.padstack.get_bbox(false);  // everything
+    }
+    if (bb.second - bb.first == Coordi()) { // still empty??
+        return;
+    }
     auto a = bb.first;
     auto b = bb.second;
     transform.accumulate(Placement((a + b) / 2));
@@ -1174,7 +1195,8 @@ static LayerRange get_layer_for_padstack_type(Padstack::Type type)
     }
 }
 
-void Canvas::render(const Package &pkg, bool interactive, bool smashed, bool omit_silkscreen, bool omit_outline)
+void Canvas::render(const Package &pkg, bool interactive, bool smashed, bool omit_silkscreen, bool omit_outline,
+                    bool on_panel)
 {
     if (interactive) {
         for (const auto &it : pkg.junctions) {
@@ -1202,26 +1224,27 @@ void Canvas::render(const Package &pkg, bool interactive, bool smashed, bool omi
         render(it.second, interactive);
     }
     if (object_refs_current.size() && object_refs_current.back().type == ObjectType::BOARD_PACKAGE) {
+        // rendering as a board package, not on a panel
         auto pkg_uuid = object_refs_current.back().uuid;
         for (const auto &it : pkg.pads) {
             object_ref_push(ObjectType::PAD, it.second.uuid, pkg_uuid);
-            render_pad_overlay(it.second);
+            render_pad_overlay(it.second, interactive);
             render(it.second);
             object_ref_pop();
         }
     }
-    else if (interactive) {
+    else if (on_panel) {
         for (const auto &it : pkg.pads) {
-            object_ref_push(ObjectType::PAD, it.second.uuid);
-            render_pad_overlay(it.second);
+            render_pad_overlay(it.second, interactive);
             render(it.second);
-            object_ref_pop();
         }
     }
     else {
         for (const auto &it : pkg.pads) {
-            render_pad_overlay(it.second);
+            object_ref_push(ObjectType::PAD, it.second.uuid);
+            render_pad_overlay(it.second, interactive);
             render(it.second);
+            object_ref_pop();
         }
     }
     for (const auto &it : pkg.polygons) {
@@ -1276,7 +1299,7 @@ void Canvas::render(const BoardPackage &pkg, bool interactive)
     if (interactive)
         object_ref_push(ObjectType::BOARD_PACKAGE, pkg.uuid);
 
-    render(pkg.package, false, pkg.smashed, pkg.omit_silkscreen, pkg.omit_outline);
+    render(pkg.package, false, pkg.smashed, pkg.omit_silkscreen, pkg.omit_outline, !interactive);
 
     if (interactive)
         object_ref_pop();
@@ -1336,9 +1359,11 @@ void Canvas::render(const BoardHole &hole, bool interactive)
     transform_save();
     transform.accumulate(hole.placement);
     auto bb = hole.padstack.get_bbox();
-    if (interactive)
-        selectables.append(hole.uuid, ObjectType::BOARD_HOLE, {0, 0}, bb.first, bb.second, 0,
-                           LayerRange(BoardLayers::TOP_COPPER, BoardLayers::BOTTOM_COPPER));
+    if (interactive) {
+        const auto layers = LayerRange(BoardLayers::TOP_COPPER, BoardLayers::BOTTOM_COPPER);
+        selectables.append(hole.uuid, ObjectType::BOARD_HOLE, {0, 0}, bb.first, bb.second, 0, layers);
+        targets.emplace_back(hole.uuid, ObjectType::BOARD_HOLE, transform.transform(Coordi()), 0, layers);
+    }
     img_net(hole.net);
     if (hole.padstack.type == Padstack::Type::HOLE)
         img_patch_type(PatchType::HOLE_PTH);
@@ -1369,8 +1394,8 @@ void Canvas::render(const class Dimension &dim)
         p1 = Coordd(p0.x, p1.y);
     }
 
-    Coordd v = p1 - p0;
-    auto vn = v / sqrt(v.mag_sq());
+    const Coordd v = p1 - p0;
+    const auto vn = v.normalize();
     Coordd w = Coordd(-v.y, v.x);
     if (dim.mode == Dimension::Mode::HORIZONTAL) {
         w.y = std::abs(w.y);
@@ -1378,7 +1403,7 @@ void Canvas::render(const class Dimension &dim)
     else if (dim.mode == Dimension::Mode::VERTICAL) {
         w.x = std::abs(w.x);
     }
-    auto wn = w / sqrt(w.mag_sq());
+    const auto wn = w.normalize();
 
     ColorP co = ColorP::DIMENSION;
 
@@ -1393,10 +1418,12 @@ void Canvas::render(const class Dimension &dim)
         draw_line(dim.p1, p1 + wn * (dim.label_distance + sgn(dim.label_distance) * (int64_t)dim.label_size / 2), co,
                   10000, true, 0);
 
-        auto length = sqrt(v.mag_sq());
+        auto length = v.mag();
         auto s = dim_to_string(length, false);
 
-        auto text_bb = draw_text0({0, 0}, dim.label_size, s, 0, false, TextOrigin::CENTER, co, 10000, 0, false);
+        TextOptions opts;
+        opts.draw = false;
+        const auto text_bb = draw_text({0, 0}, dim.label_size, s, 0, TextOrigin::CENTER, co, 10000, opts);
         auto text_width = std::abs(text_bb.second.x - text_bb.first.x);
 
         Coordf text_pos = (q0 + v / 2) - (vn * text_width / 2);
@@ -1431,10 +1458,12 @@ void Canvas::render(const class Dimension &dim)
         draw_line({0, 0}, Coordf(-arrow_mul * dim.label_size, (int64_t)dim.label_size / -2), co, 10000, true, 0);
         transform_restore();
 
-        int angle_i = (angle / M_PI) * 32768;
-        auto real_text_bb = draw_text0(Coordi(text_pos.x, text_pos.y), dim.label_size, s,
-                                       get_flip_view() ? (32768 - angle_i) : angle_i, get_flip_view(),
-                                       TextOrigin::CENTER, co, 10000, 0, true);
+        const int angle_i = angle_from_rad(angle);
+        TextOptions opts2;
+        opts2.flip = get_flip_view();
+        const auto real_text_bb =
+                draw_text(Coordi(text_pos.x, text_pos.y), dim.label_size, s,
+                          get_flip_view() ? (32768 - angle_i) : angle_i, TextOrigin::CENTER, co, 10000, opts2);
 
         selectables.append(dim.uuid, ObjectType::DIMENSION, q0, real_text_bb.first, real_text_bb.second, 2);
         targets.emplace_back(dim.uuid, ObjectType::DIMENSION, Coordi(q0.x, q0.y), 2);
@@ -1588,9 +1617,11 @@ void Canvas::render(const Picture &pic)
                                   Coordf(1_mm, 1_mm), 0);
         return;
     }
+    if (img_mode)
+        return;
     pictures.emplace_back();
     auto &x = pictures.back();
-    x.angle = pic.placement.get_angle() / 65536.0 * 2 * M_PI;
+    x.angle = pic.placement.get_angle_rad();
     x.x = pic.placement.shift.x;
     x.y = pic.placement.shift.y;
     x.px_size = pic.px_size;
@@ -1599,8 +1630,6 @@ void Canvas::render(const Picture &pic)
     x.opacity = pic.opacity;
     float w = pic.data->width * pic.px_size;
     float h = pic.data->height * pic.px_size;
-    if (img_mode)
-        return;
     selectables.append_angled(pic.uuid, ObjectType::PICTURE, pic.placement.shift, pic.placement.shift, {w, h},
                               pic.placement.get_angle_rad());
 }
