@@ -1,8 +1,9 @@
 #include "canvas3d_base.hpp"
-#include "util/step_importer.hpp"
+#include "import_step/import.hpp"
 #include "board/board_layers.hpp"
 #include "board/board.hpp"
 #include "canvas/gl_util.hpp"
+#include "util/util.hpp"
 #include "util/min_max_accumulator.hpp"
 #include <glm/glm.hpp>
 #include <glm/gtc/matrix_transform.hpp>
@@ -11,11 +12,13 @@
 #include <glibmm/miscutils.h>
 #include "pool/pool_manager.hpp"
 #include "pool/ipool.hpp"
+#include "logger/log_util.hpp"
 
 namespace horizon {
 
 Canvas3DBase::Canvas3DBase()
-    : center(0), cover_renderer(*this), wall_renderer(*this), face_renderer(*this), background_renderer(*this)
+    : center(0), cover_renderer(*this), wall_renderer(*this), face_renderer(*this), background_renderer(*this),
+      point_renderer(*this)
 {
 }
 
@@ -35,7 +38,7 @@ Color Canvas3DBase::get_layer_color(int layer) const
         return {.7, .7, .7};
 
     if (layer == BoardLayers::TOP_SILKSCREEN || layer == BoardLayers::BOTTOM_SILKSCREEN)
-        return {1, 1, 1};
+        return silkscreen_color;
 
     if (layer == BoardLayers::L_OUTLINE || layer >= 10000)
         return substrate_color;
@@ -67,7 +70,7 @@ float Canvas3DBase::get_layer_thickness(int layer) const
 bool Canvas3DBase::layer_is_visible(int layer) const
 {
     if (layer == 20000) // pth holes
-        return true;
+        return show_copper;
 
     if (layer == BoardLayers::TOP_MASK || layer == BoardLayers::BOTTOM_MASK)
         return show_solder_mask;
@@ -90,8 +93,12 @@ bool Canvas3DBase::layer_is_visible(int layer) const
             return false;
         }
     }
+
     if (layer < BoardLayers::TOP_COPPER && layer > BoardLayers::BOTTOM_COPPER)
-        return show_substrate == false || explode > 0;
+        return (show_substrate == false || explode > 0) && show_copper;
+
+    if (BoardLayers::is_copper(layer))
+        return show_copper;
 
     return true;
 }
@@ -102,6 +109,7 @@ void Canvas3DBase::a_realize()
     wall_renderer.realize();
     face_renderer.realize();
     background_renderer.realize();
+    point_renderer.realize();
     glEnable(GL_DEPTH_TEST);
     glBlendFuncSeparate(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA, GL_ONE_MINUS_DST_ALPHA, GL_DST_ALPHA);
 
@@ -110,15 +118,32 @@ void Canvas3DBase::a_realize()
 
     glGenRenderbuffers(1, &renderbuffer);
     glGenRenderbuffers(1, &depthrenderbuffer);
+    glGenRenderbuffers(1, &pickrenderbuffer);
+    glGenRenderbuffers(1, &pickrenderbuffer_downsampled);
 
     resize_buffers();
 
     GL_CHECK_ERROR
 
+    glGenFramebuffers(1, &fbo_downsampled);
+    glBindFramebuffer(GL_FRAMEBUFFER, fbo_downsampled);
+    glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_RENDERBUFFER, pickrenderbuffer_downsampled);
+
+    if (glCheckFramebufferStatus(GL_FRAMEBUFFER) != GL_FRAMEBUFFER_COMPLETE) {
+        // Gtk::MessageDialog md("Error setting up framebuffer, will now exit", false /* use_markup */,
+        // Gtk::MESSAGE_ERROR,
+        //                      Gtk::BUTTONS_OK);
+        // md.run();
+        abort();
+    }
+
+    GL_CHECK_ERROR
+
     glGenFramebuffers(1, &fbo);
     glBindFramebuffer(GL_FRAMEBUFFER, fbo);
 
     glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT0, GL_RENDERBUFFER, renderbuffer);
+    glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_COLOR_ATTACHMENT1, GL_RENDERBUFFER, pickrenderbuffer);
     glFramebufferRenderbuffer(GL_FRAMEBUFFER, GL_DEPTH_ATTACHMENT, GL_RENDERBUFFER, depthrenderbuffer);
 
     GL_CHECK_ERROR
@@ -147,6 +172,13 @@ void Canvas3DBase::resize_buffers()
     glBindRenderbuffer(GL_RENDERBUFFER, depthrenderbuffer);
     glRenderbufferStorageMultisample(GL_RENDERBUFFER, samples, GL_DEPTH_COMPONENT, width * a_get_scale_factor(),
                                      height * a_get_scale_factor());
+    glBindRenderbuffer(GL_RENDERBUFFER, pickrenderbuffer);
+    glRenderbufferStorageMultisample(GL_RENDERBUFFER, samples, GL_R16UI, width * a_get_scale_factor(),
+                                     height * a_get_scale_factor());
+
+    glBindRenderbuffer(GL_RENDERBUFFER, pickrenderbuffer_downsampled);
+    glRenderbufferStorage(GL_RENDERBUFFER, GL_R16UI, width * a_get_scale_factor(), height * a_get_scale_factor());
+
     glBindRenderbuffer(GL_RENDERBUFFER, rb);
 }
 
@@ -155,6 +187,7 @@ void Canvas3DBase::push()
     cover_renderer.push();
     wall_renderer.push();
     face_renderer.push();
+    point_renderer.push();
 }
 
 float Canvas3DBase::get_magic_number() const
@@ -162,6 +195,37 @@ float Canvas3DBase::get_magic_number() const
     return tan(0.5 * glm::radians(cam_fov));
 }
 
+static float fix_cam_elevation(float cam_elevation)
+{
+    while (cam_elevation >= 360)
+        cam_elevation -= 360;
+    while (cam_elevation < 0)
+        cam_elevation += 360;
+    if (cam_elevation > 180)
+        cam_elevation -= 360;
+    return cam_elevation;
+}
+
+void Canvas3DBase::set_cam_elevation(const float &ele)
+{
+    cam_elevation = fix_cam_elevation(ele);
+    redraw();
+    invalidate_pick();
+    s_signal_view_changed.emit();
+}
+
+void Canvas3DBase::set_cam_azimuth(const float &az)
+{
+    cam_azimuth = az;
+    while (cam_azimuth < 0)
+        cam_azimuth += 360;
+
+    while (cam_azimuth > 360)
+        cam_azimuth -= 360;
+    redraw();
+    invalidate_pick();
+    s_signal_view_changed.emit();
+}
 
 void Canvas3DBase::render(RenderBackground mode)
 {
@@ -175,6 +239,10 @@ void Canvas3DBase::render(RenderBackground mode)
     glClearDepth(10);
     glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
     GL_CHECK_ERROR
+    {
+        const std::array<GLenum, 2> bufs = {GL_COLOR_ATTACHMENT0, GL_COLOR_ATTACHMENT1};
+        glDrawBuffers(bufs.size(), bufs.data());
+    }
 
     if (mode == RenderBackground::YES) {
         glDisable(GL_DEPTH_TEST);
@@ -223,12 +291,39 @@ void Canvas3DBase::render(RenderBackground mode)
     if (show_models)
         face_renderer.render();
 
+    if (show_points)
+        point_renderer.render();
+
     cover_renderer.render();
 
     GL_CHECK_ERROR
 
+    if (pick_state == PickState::QUEUED) {
+        glBindFramebuffer(GL_DRAW_FRAMEBUFFER, fbo_downsampled);
+        glBindFramebuffer(GL_READ_FRAMEBUFFER, fbo);
+        glDrawBuffer(GL_COLOR_ATTACHMENT0);
+        glReadBuffer(GL_COLOR_ATTACHMENT1);
+        glBlitFramebuffer(0, 0, width * a_get_scale_factor(), height * a_get_scale_factor(), 0, 0,
+                          width * a_get_scale_factor(), height * a_get_scale_factor(), GL_COLOR_BUFFER_BIT, GL_NEAREST);
+
+        glBindFramebuffer(GL_READ_FRAMEBUFFER, fbo_downsampled);
+        glReadBuffer(GL_COLOR_ATTACHMENT0);
+        pick_buf.resize(width * a_get_scale_factor() * height * a_get_scale_factor());
+        GL_CHECK_ERROR
+
+        glPixelStorei(GL_PACK_ALIGNMENT, 2);
+        glReadPixels(0, 0, width * a_get_scale_factor(), height * a_get_scale_factor(), GL_RED_INTEGER,
+                     GL_UNSIGNED_SHORT, pick_buf.data());
+
+        GL_CHECK_ERROR
+        pick_state = PickState::CURRENT;
+        s_signal_pick_ready.emit();
+    }
+
     glBindFramebuffer(GL_DRAW_FRAMEBUFFER, fb);
     glBindFramebuffer(GL_READ_FRAMEBUFFER, fbo);
+    glDrawBuffer(fb ? GL_COLOR_ATTACHMENT0 : GL_FRONT);
+    glReadBuffer(GL_COLOR_ATTACHMENT0);
     glBlitFramebuffer(0, 0, width * a_get_scale_factor(), height * a_get_scale_factor(), 0, 0,
                       width * a_get_scale_factor(), height * a_get_scale_factor(), GL_COLOR_BUFFER_BIT, GL_NEAREST);
 
@@ -261,12 +356,12 @@ void Canvas3DBase::view_all()
     if (board_height < 1 || board_width < 1)
         return;
 
-    center = {(xmin + xmax) / 2e6, (ymin + ymax) / 2e6};
+    set_center({(xmin + xmax) / 2e6, (ymin + ymax) / 2e6});
 
 
-    cam_distance = std::max(board_width / width, board_height / height) / (2 * get_magic_number() / height) * 1.1;
-    cam_azimuth = 270;
-    cam_elevation = 89.99;
+    set_cam_distance(std::max(board_width / width, board_height / height) / (2 * get_magic_number() / height) * 1.1);
+    set_cam_azimuth(270);
+    set_cam_elevation(89.99);
 }
 
 void Canvas3DBase::prepare()
@@ -293,7 +388,7 @@ void Canvas3DBase::prepare_packages()
 {
     if (!brd)
         return;
-    package_transform_idxs.clear();
+    package_infos.clear();
     package_transforms.clear();
     std::map<std::pair<std::string, bool>, std::set<const BoardPackage *>> pkg_map;
     for (const auto &it : brd->packages) {
@@ -303,9 +398,12 @@ void Canvas3DBase::prepare_packages()
                     &it.second);
     }
 
+    unsigned int pick_base = 2; // 0: background, 1: PCB, 2+: models
     for (const auto &it_pkg : pkg_map) {
         size_t size_before = package_transforms.size();
+        std::vector<UUID> pkgs;
         for (const auto &it_brd_pkg : it_pkg.second) {
+            pkgs.push_back(it_brd_pkg->uuid);
             const auto &pl = it_brd_pkg->placement;
             const auto &pkg = it_brd_pkg->package;
             package_transforms.emplace_back(pl.shift.x / 1e6, pl.shift.y / 1e6, pl.get_angle(), it_brd_pkg->flip,
@@ -322,17 +420,24 @@ void Canvas3DBase::prepare_packages()
             }
         }
         size_t size_after = package_transforms.size();
-        package_transform_idxs[it_pkg.first] = {size_before, size_after - size_before};
+        package_infos[it_pkg.first] = {size_before, size_after - size_before, pick_base, pkgs};
+        pick_base += pkgs.size();
     }
+    point_pick_base = pick_base;
 }
 
+STEPImporter::Faces Canvas3DBase::import_step(const std::string &filename_rel, const std::string &filename_abs)
+{
+    const auto result = STEPImporter::import(filename_abs);
+    return result.faces;
+}
 
 void Canvas3DBase::load_3d_model(const std::string &filename, const std::string &filename_abs)
 {
     if (models.count(filename))
         return;
 
-    auto faces = STEPImporter::import(filename_abs);
+    const auto faces = import_step(filename, filename_abs);
     {
         std::lock_guard<std::mutex> lock(models_loading_mutex);
         // canvas->face_vertex_buffer.reserve(faces.size());
@@ -361,6 +466,23 @@ void Canvas3DBase::load_3d_model(const std::string &filename, const std::string
     }
 }
 
+bool Canvas3DBase::model_is_loaded(const std::string &filename)
+{
+    std::lock_guard<std::mutex> lock(models_loading_mutex);
+    return models.count(filename);
+}
+
+void Canvas3DBase::set_point_transform(const glm::dmat4 &mat)
+{
+    point_mat = mat;
+    invalidate_pick();
+    redraw();
+}
+
+void Canvas3DBase::set_points(const std::vector<Point3D> &pts)
+{
+    points = pts;
+}
 
 std::map<std::string, std::string> Canvas3DBase::get_model_filenames(IPool &pool)
 {
@@ -371,14 +493,8 @@ std::map<std::string, std::string> Canvas3DBase::get_model_filenames(IPool &pool
             std::string model_filename;
             const Package *pool_package = nullptr;
 
-            UUID this_pool_uuid;
+            UUID this_pool_uuid = pool.get_pool_info().uuid;
             UUID pkg_pool_uuid;
-            auto base_path = pool.get_base_path();
-            const auto &pools = PoolManager::get().get_pools();
-            if (pools.count(base_path))
-                this_pool_uuid = pools.at(base_path).uuid;
-
-
             try {
                 pool_package = pool.get_package(it.second.package.uuid, &pkg_pool_uuid);
             }
@@ -420,4 +536,80 @@ void Canvas3DBase::update_max_package_height()
     }
 }
 
+void Canvas3DBase::invalidate_pick()
+{
+    pick_state = PickState::INVALID;
+}
+
+void Canvas3DBase::queue_pick()
+{
+    if (pick_state == PickState::INVALID) {
+        pick_state = PickState::QUEUED;
+        redraw();
+    }
+    else if (pick_state == PickState::CURRENT) {
+        s_signal_pick_ready.emit();
+    }
+}
+
+std::variant<UUID, glm::dvec3> Canvas3DBase::pick_package_or_point(unsigned int x, unsigned int y) const
+{
+    if (pick_state != PickState::CURRENT) {
+        Logger::log_warning("can't with non-current pick state");
+        return {};
+    }
+    x *= a_get_scale_factor();
+    y *= a_get_scale_factor();
+    const int idx = ((height * a_get_scale_factor()) - y - 1) * width * a_get_scale_factor() + x;
+    const auto pick = pick_buf.at(idx);
+    if (pick >= point_pick_base) {
+        const auto &pt = points.at(pick - point_pick_base);
+        const auto p4 = point_mat * glm::dvec4(pt.x, pt.y, pt.z, 1);
+        return glm::dvec3(p4.x, p4.y, p4.z);
+    }
+    for (const auto &[k, v] : package_infos) {
+        if (pick >= v.pick_base && pick < (v.pick_base + v.n_packages)) {
+            return v.pkg.at(pick - v.pick_base);
+        }
+    }
+    return {};
+}
+
+static void acc(float &l, float &h, float v)
+{
+    l = std::min(l, v);
+    h = std::max(h, v);
+}
+
+Canvas3DBase::BBox Canvas3DBase::get_model_bbox(const std::string &filename) const
+{
+    Canvas3DBase::BBox bb;
+    bool first = true;
+    if (models.count(filename)) {
+        const auto &m = models.at(filename);
+        for (size_t i = m.face_index_offset; i < m.face_index_offset + m.count; i++) {
+            const auto idx = face_index_buffer.at(i);
+            const auto &v = face_vertex_buffer.at(idx);
+            if (first) {
+                bb.xh = v.x;
+                bb.xl = v.x;
+                bb.yh = v.y;
+                bb.yl = v.y;
+                bb.zh = v.z;
+                bb.zl = v.z;
+            }
+            else {
+                acc(bb.xl, bb.xh, v.x);
+                acc(bb.yl, bb.yh, v.y);
+                acc(bb.zl, bb.zh, v.z);
+            }
+            first = false;
+        }
+    }
+    else {
+        bb = {0};
+    }
+    return bb;
+}
+
 } // namespace horizon
