@@ -1,5 +1,6 @@
 #include "check_util.hpp"
 #include <glibmm/regex.h>
+#include "rules/rule.hpp"
 
 namespace horizon {
 bool needs_trim(const std::string &s)
@@ -13,4 +14,10 @@ bool check_tag(const std::string &s)
     static auto re = Glib::Regex::create("^[a-z-0-9.]+$");
     return re->match(us);
 }
+
+void accumulate_level(RulesCheckErrorLevel &r, RulesCheckErrorLevel l)
+{
+    r = static_cast<RulesCheckErrorLevel>(std::max(static_cast<int>(r), static_cast<int>(l)));
+}
+
 } // namespace horizon
