@@ -80,7 +80,8 @@ const std::map<ObjectType, ObjectDescription> object_descriptions = {
                             {static_cast<int>(SchematicSymbol::PinDisplayMode::CUSTOM_ONLY), "Custom only"},
                     }}},
                   {ObjectProperty::ID::DISPLAY_ALL_PADS, {ObjectProperty::Type::BOOL, "Show all pads", 3}},
-                  {ObjectProperty::ID::EXPAND, {ObjectProperty::Type::EXPAND, "Expand", 4}},
+                  {ObjectProperty::ID::VALUE, {ObjectProperty::Type::STRING_MULTILINE, "Custom value", 4}},
+                  {ObjectProperty::ID::EXPAND, {ObjectProperty::Type::EXPAND, "Expand", 5}},
           }}},
         {ObjectType::POWER_SYMBOL, {"Power symbol", "Power symbols", {}}},
         {ObjectType::POLYGON_EDGE, {"Polygon edge", "Polygon edges", {}}},
@@ -93,6 +94,8 @@ const std::map<ObjectType, ObjectDescription> object_descriptions = {
                   {ObjectProperty::ID::FROM_RULES, {ObjectProperty::Type::BOOL, "From rules", 1}},
                   {ObjectProperty::ID::NAME, {ObjectProperty::Type::STRING_RO, "Net", 0}},
                   {ObjectProperty::ID::LOCKED, {ObjectProperty::Type::BOOL, "Locked", 2}},
+                  {ObjectProperty::ID::POSITION_X, {ObjectProperty::Type::DIM, "Position X", 3}},
+                  {ObjectProperty::ID::POSITION_Y, {ObjectProperty::Type::DIM, "Position Y", 4}},
           }}},
         {ObjectType::SHAPE,
          {"Shape",
@@ -142,10 +145,11 @@ const std::map<ObjectType, ObjectDescription> object_descriptions = {
                   {ObjectProperty::ID::POSITION_Y, {ObjectProperty::Type::DIM, "Position Y", 5}},
                   {ObjectProperty::ID::ANGLE, {ObjectProperty::Type::ANGLE, "Angle", 6}},
                   {ObjectProperty::ID::MIRROR, {ObjectProperty::Type::BOOL, "Mirror", 7}},
+                  {ObjectProperty::ID::ALLOW_UPSIDE_DOWN, {ObjectProperty::Type::BOOL, "Allow upside-down", 8}},
                   {ObjectProperty::ID::FONT,
                    {ObjectProperty::Type::ENUM,
                     "Font",
-                    8,
+                    9,
                     {
                             {static_cast<int>(TextData::Font::SIMPLEX), "Simplex"},
                             {static_cast<int>(TextData::Font::DUPLEX), "Duplex"},
@@ -268,7 +272,7 @@ const std::map<ObjectType, ObjectDescription> object_descriptions = {
                   {ObjectProperty::ID::WIDTH, {ObjectProperty::Type::LENGTH, "Min. Width", 2}},
                   {ObjectProperty::ID::NAME, {ObjectProperty::Type::STRING_RO, "Net", 0}},
                   {ObjectProperty::ID::FROM_RULES, {ObjectProperty::Type::BOOL, "From rules", 1}},
-                  {ObjectProperty::ID::PRIORITY, {ObjectProperty::Type::PRIORITY, "Priority", 3}},
+                  {ObjectProperty::ID::PRIORITY, {ObjectProperty::Type::PRIORITY, "Fill order", 3}},
           }}},
         {ObjectType::DIMENSION,
          {"Dimension",
