@@ -6,7 +6,7 @@
 #include <deque>
 #include <memory>
 #include <sigc++/sigc++.h>
-#include "tool.hpp"
+#include "tool_pub.hpp"
 #include "document/document.hpp"
 #include "util/placement.hpp"
 
@@ -51,10 +51,19 @@ public:
         return nullptr;
     }
 
+    class GridSettings *get_grid_settings() override
+    {
+        return nullptr;
+    }
+
     class IPool &get_pool() override
     {
         return m_pool;
     }
+    class IPool &get_pool_caching() override
+    {
+        return m_pool_caching;
+    }
     virtual ObjectType get_object_type() const = 0;
 
     /**
@@ -166,8 +175,9 @@ public:
     }
 
 protected:
-    Core(class IPool &pool);
+    Core(class IPool &pool, IPool *m_pool_caching);
     class IPool &m_pool;
+    class IPool &m_pool_caching;
 
     std::unique_ptr<ToolBase> tool = nullptr;
     type_signal_tool_changed s_signal_tool_changed;
