@@ -9,8 +9,8 @@
 namespace horizon {
 class CoreBoard : public Core, public DocumentBoard {
 public:
-    CoreBoard(const std::string &board_filename, const std::string &block_filename, const std::string &via_dir,
-              const std::string &pictures_dir, IPool &pool);
+    CoreBoard(const std::string &board_filename, const std::string &block_filename, const std::string &pictures_dir,
+              IPool &pool, IPool &pool_caching);
 
     class Block *get_block() override;
     class LayerProvider &get_layer_provider() override;
@@ -29,11 +29,6 @@ public:
     Board *get_board() override;
     const Board *get_board() const;
 
-    ViaPadstackProvider &get_via_padstack_provider() override
-    {
-        return via_padstack_provider;
-    }
-
     class Rules *get_rules() override;
     FabOutputSettings &get_fab_output_settings() override
     {
@@ -51,6 +46,10 @@ public:
     {
         return pnp_export_settings;
     }
+    GridSettings *get_grid_settings() override
+    {
+        return &grid_settings;
+    }
 
     BoardColors &get_colors() override
     {
@@ -77,8 +76,6 @@ public:
     void reload_pool() override;
 
 private:
-    ViaPadstackProvider via_padstack_provider;
-
     std::optional<Block> block;
     std::optional<Board> brd;
 
@@ -87,6 +84,7 @@ private:
     PDFExportSettings pdf_export_settings;
     STEPExportSettings step_export_settings;
     PnPExportSettings pnp_export_settings;
+    GridSettings grid_settings;
 
     BoardColors colors;
 
