@@ -1,4 +1,5 @@
 #include "core_frame.hpp"
+#include "core_properties.hpp"
 #include "util/util.hpp"
 #include "nlohmann/json.hpp"
 #include <giomm/file.h>
@@ -6,7 +7,7 @@
 
 namespace horizon {
 CoreFrame::CoreFrame(const std::string &frame_filename, IPool &pool)
-    : Core(pool), frame(Frame::new_from_file(frame_filename)), m_frame_filename(frame_filename)
+    : Core(pool, nullptr), frame(Frame::new_from_file(frame_filename)), m_frame_filename(frame_filename)
 {
     rebuild();
 }
@@ -52,6 +53,26 @@ std::map<UUID, Arc> *CoreFrame::get_arc_map()
     return &frame.arcs;
 }
 
+bool CoreFrame::get_property_meta(ObjectType type, const UUID &uu, ObjectProperty::ID property, PropertyMeta &meta)
+{
+    if (Core::get_property_meta(type, uu, property, meta))
+        return true;
+    switch (type) {
+    case ObjectType::TEXT:
+        switch (property) {
+        case ObjectProperty::ID::ALLOW_UPSIDE_DOWN:
+            meta.is_visible = false;
+            return true;
+        default:
+            return false;
+        }
+        break;
+    default:
+        return false;
+    }
+    return false;
+}
+
 void CoreFrame::rebuild(bool from_undo)
 {
     frame.expand();
