@@ -132,6 +132,10 @@ bool Core::get_property(ObjectType type, const UUID &uu, ObjectProperty::ID prop
             dynamic_cast<PropertyValueInt &>(value).value = static_cast<int>(text->font);
             return true;
 
+        case ObjectProperty::ID::ALLOW_UPSIDE_DOWN:
+            dynamic_cast<PropertyValueBool &>(value).value = text->allow_upside_down;
+            return true;
+
         default:
             return false;
         }
@@ -342,6 +346,10 @@ bool Core::set_property(ObjectType type, const UUID &uu, ObjectProperty::ID prop
             text->font = static_cast<TextData::Font>(dynamic_cast<const PropertyValueInt &>(value).value);
             break;
 
+        case ObjectProperty::ID::ALLOW_UPSIDE_DOWN:
+            text->allow_upside_down = dynamic_cast<const PropertyValueBool &>(value).value;
+            break;
+
         default:
             return false;
         }
@@ -486,9 +494,11 @@ void Core::layers_to_meta(PropertyMeta &meta)
 {
     PropertyMetaLayers &m = dynamic_cast<PropertyMetaLayers &>(meta);
     m.layers.clear();
-    for (const auto &it : get_layer_provider().get_layers()) {
+    const auto &prv = get_layer_provider();
+    for (const auto &it : prv.get_layers()) {
         m.layers.emplace(it.first, it.second);
     }
+    m.is_visible = prv.get_layers().size() > 1;
 }
 
 void Core::get_placement(const Placement &placement, class PropertyValue &value, ObjectProperty::ID property)
