@@ -7,13 +7,15 @@
 #include <giomm/file.h>
 #include <glibmm/fileutils.h>
 #include "pool/ipool.hpp"
+#include "util/str_util.hpp"
+#include "util/picture_load.hpp"
 
 namespace horizon {
 CoreSchematic::CoreSchematic(const std::string &schematic_filename, const std::string &block_filename,
-                             const std::string &pictures_dir, IPool &pool)
-    : Core(pool), block(Block::new_from_file(block_filename, pool)),
+                             const std::string &pictures_dir, IPool &pool, IPool &pool_caching)
+    : Core(pool, &pool_caching), block(Block::new_from_file(block_filename, pool_caching)),
       project_meta_loaded_from_block(block->project_meta.size()),
-      sch(Schematic::new_from_file(schematic_filename, *block, pool)), rules(sch->rules),
+      sch(Schematic::new_from_file(schematic_filename, *block, pool_caching)), rules(sch->rules),
       bom_export_settings(block->bom_export_settings), pdf_export_settings(sch->pdf_export_settings),
       m_schematic_filename(schematic_filename), m_block_filename(block_filename), m_pictures_dir(pictures_dir)
 {
@@ -348,6 +350,10 @@ bool CoreSchematic::get_property(ObjectType type, const UUID &uu, ObjectProperty
             dynamic_cast<PropertyValueString &>(value).value = sym->component->refdes + sym->gate->suffix;
             return true;
 
+        case ObjectProperty::ID::VALUE:
+            dynamic_cast<PropertyValueString &>(value).value = sym->custom_value;
+            return true;
+
         default:
             return false;
         }
@@ -430,6 +436,11 @@ bool CoreSchematic::set_property(ObjectType type, const UUID &uu, ObjectProperty
             sym->apply_expand();
             break;
 
+        case ObjectProperty::ID::VALUE:
+            sym->custom_value = dynamic_cast<const PropertyValueString &>(value).value;
+            trim(sym->custom_value);
+            break;
+
         default:
             return false;
         }
@@ -509,6 +520,16 @@ bool CoreSchematic::get_property_meta(ObjectType type, const UUID &uu, ObjectPro
         }
         break;
 
+    case ObjectType::TEXT:
+        switch (property) {
+        case ObjectProperty::ID::ALLOW_UPSIDE_DOWN:
+            meta.is_visible = false;
+            return true;
+        default:
+            return false;
+        }
+        break;
+
     default:
         return false;
     }
@@ -638,11 +659,20 @@ void CoreSchematic::history_load(unsigned int i)
 
 void CoreSchematic::reload_pool()
 {
+    PictureKeeper keeper;
+    for (const auto &[uu_sheet, sheet] : sch->sheets) {
+        keeper.save(sheet.pictures);
+    }
     const auto sch_j = sch->serialize();
     const auto block_j = block->serialize();
     m_pool.clear();
-    block.emplace(block->uuid, block_j, m_pool);
-    sch.emplace(sch->uuid, sch_j, *block, m_pool);
+    m_pool_caching.clear();
+    block.emplace(block->uuid, block_j, m_pool_caching);
+    sch.emplace(sch->uuid, sch_j, *block, m_pool_caching);
+    for (auto &[uu_sheet, sheet] : sch->sheets) {
+        keeper.restore(sheet.pictures);
+    }
+    bom_export_settings.update_refs(m_pool_caching);
     history_clear();
     rebuild();
 }
