@@ -83,6 +83,9 @@
 #include "tools/tool_draw_plane.hpp"
 #include "tools/tool_drag_polygon_edge.hpp"
 #include "tools/tool_measure.hpp"
+#include "tools/tool_edit_custom_value.hpp"
+#include "tools/tool_place_dot.hpp"
+#include "tools/tool_set_track_width.hpp"
 
 namespace horizon {
 
@@ -397,6 +400,15 @@ std::unique_ptr<ToolBase> Core::create_tool(ToolID tool_id)
     case ToolID::MEASURE:
         return std::make_unique<ToolMeasure>(this, tool_id);
 
+    case ToolID::EDIT_CUSTOM_VALUE:
+        return std::make_unique<ToolEditCustomValue>(this, tool_id);
+
+    case ToolID::PLACE_DOT:
+        return std::make_unique<ToolPlaceDot>(this, tool_id);
+
+    case ToolID::SET_TRACK_WIDTH:
+        return std::make_unique<ToolSetTrackWidth>(this, tool_id);
+
     default:
         return nullptr;
     }
