@@ -151,5 +151,8 @@ enum class ToolID {
     DRAW_KEEPOUT,
     DRAG_POLYGON_EDGE,
     MEASURE,
+    EDIT_CUSTOM_VALUE,
+    PLACE_DOT,
+    SET_TRACK_WIDTH,
 };
 } // namespace horizon
