@@ -97,12 +97,13 @@ void ToolHelperMove::move_do(const Coordi &delta)
     }
 }
 
-void ToolHelperMove::move_do_cursor(const Coordi &cc)
+Coordi ToolHelperMove::move_do_cursor(const Coordi &cc)
 {
     auto c = get_coord_restrict(origin, cc);
     auto delta = c - last;
     move_do(delta);
     last = c;
+    return delta;
 }
 
 
@@ -374,5 +375,7 @@ void ToolHelperMove::move_mirror_or_rotate(const Coordi &center, bool rotate)
         default:;
         }
     }
+    if (doc.b && !rotate)
+        doc.b->get_board()->update_junction_connections();
 }
 } // namespace horizon
