@@ -71,6 +71,7 @@ ToolResponse ToolMapSymbol::begin(const ToolArgs &args)
             {InToolActionID::RMB},
             {InToolActionID::ROTATE},
             {InToolActionID::MIRROR},
+            {InToolActionID::EDIT, "change symbol"},
     });
 
     return ToolResponse();
@@ -143,6 +144,10 @@ ToolResponse ToolMapSymbol::update(const ToolArgs &args)
             move_mirror_or_rotate(sym_current->placement.shift, args.action == InToolActionID::ROTATE);
             break;
 
+        case InToolActionID::EDIT:
+            change_symbol(sym_current);
+            break;
+
         default:;
         }
     }
