@@ -10,6 +10,7 @@
 #include "imp/imp_interface.hpp"
 #include "pool/entity.hpp"
 #include "util/util.hpp"
+#include "util/geom_util.hpp"
 #include "core/clipboard/clipboard.hpp"
 #include <iostream>
 #include <gtkmm.h>
@@ -111,13 +112,21 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
         }
     }
     std::map<UUID, const UUID> junction_xlat;
+    std::map<UUID, Net *> junction_nets;
     if (j.count("junctions")) {
         const json &o = j["junctions"];
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID::random();
             junction_xlat.emplace(it.key(), u);
             auto x = doc.r->insert_junction(u);
-            *x = Junction(u, it.value());
+            const auto &j_ju = it.value();
+            *x = Junction(u, j_ju);
+            if (doc.c && j_ju.count("net")) {
+                const auto net_uu = UUID(j_ju.at("net").get<std::string>());
+                auto &block = *doc.c->get_block();
+                if (block.nets.count(net_uu))
+                    junction_nets.emplace(u, &block.nets.at(net_uu));
+            }
             apply_shift(x->position, cursor_pos_canvas);
             selection.emplace(u, ObjectType::JUNCTION);
         }
@@ -226,7 +235,7 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID::random();
             component_xlat.emplace(it.key(), u);
-            Component comp(u, it.value(), doc.r->get_pool());
+            Component comp(u, it.value(), doc.r->get_pool_caching());
             for (auto &it_conn : comp.connections) {
                 it_conn.second.net = &block->nets.at(net_xlat.at(it_conn.second.net.uuid));
             }
@@ -241,7 +250,7 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID::random();
             symbol_xlat.emplace(it.key(), u);
-            SchematicSymbol sym(u, it.value(), doc.r->get_pool());
+            SchematicSymbol sym(u, it.value(), doc.r->get_pool_caching());
             sym.component = &block->components.at(component_xlat.at(sym.component.uuid));
             sym.gate = &sym.component->entity->gates.at(sym.gate.uuid);
             auto x = &sheet->symbols.emplace(u, sym).first->second;
@@ -272,12 +281,14 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
             }
         }
     }
+    std::map<UUID, Net *> net_line_nets;
     if (j.count("net_lines") && doc.c) {
         const json &o = j["net_lines"];
         auto sheet = doc.c->get_sheet();
         for (auto it = o.cbegin(); it != o.cend(); ++it) {
             auto u = UUID::random();
-            LineNet line(u, it.value());
+            const auto &j_line = it.value();
+            LineNet line(u, j_line);
             auto update_net_line_conn = [&junction_xlat, &symbol_xlat, &bus_ripper_xlat,
                                          sheet](LineNet::Connection &c) {
                 if (c.junc.uuid) {
@@ -297,6 +308,12 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
             };
             if (update_net_line_conn(line.from) && update_net_line_conn(line.to)) {
                 sheet->net_lines.emplace(u, line);
+                if (j_line.count("net")) {
+                    const auto net_uu = UUID(j_line.at("net").get<std::string>());
+                    auto &block = *doc.c->get_block();
+                    if (block.nets.count(net_uu))
+                        net_line_nets.emplace(u, &block.nets.at(net_uu));
+                }
                 selection.emplace(u, ObjectType::LINE_NET);
             }
         }
@@ -354,19 +371,18 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
             auto brd = doc.b->get_board();
             auto x = &brd->vias
                               .emplace(std::piecewise_construct, std::forward_as_tuple(u),
-                                       std::forward_as_tuple(u, it.value()))
+                                       std::forward_as_tuple(u, it.value(), doc.b->get_pool_caching(), nullptr))
                               .first->second;
-            if (auto ps = doc.b->get_via_padstack_provider().get_padstack(x->vpp_padstack.uuid)) {
-                x->vpp_padstack = ps;
-                x->expand(*brd);
-            }
+            x->expand(*brd);
             if (brd->block->nets.count(x->net_set.uuid)) {
                 x->net_set = &brd->block->nets.at(x->net_set.uuid);
+                nets.insert(x->net_set->uuid);
             }
             else {
                 x->net_set = nullptr;
             }
             x->junction = &brd->junctions.at(junction_xlat.at(x->junction.uuid));
+            x->junction->net = x->net_set;
             selection.emplace(u, ObjectType::VIA);
         }
     }
@@ -391,7 +407,7 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
             auto brd = doc.b->get_board();
             auto x = &brd->holes
                               .emplace(std::piecewise_construct, std::forward_as_tuple(u),
-                                       std::forward_as_tuple(u, it.value(), nullptr, doc.r->get_pool()))
+                                       std::forward_as_tuple(u, it.value(), nullptr, doc.r->get_pool_caching()))
                               .first->second;
             if (net_xlat.count(x->net.uuid)) {
                 x->net = &brd->block->nets.at(net_xlat.at(x->net.uuid));
@@ -428,7 +444,7 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
             auto brd = doc.b->get_board();
             auto &x = brd->decals
                               .emplace(std::piecewise_construct, std::forward_as_tuple(u),
-                                       std::forward_as_tuple(u, it.value(), doc.r->get_pool(), *brd))
+                                       std::forward_as_tuple(u, it.value(), doc.r->get_pool_caching(), *brd))
                               .first->second;
             apply_shift(x.placement.shift, cursor_pos_canvas);
             selection.emplace(u, ObjectType::BOARD_DECAL);
@@ -446,7 +462,19 @@ ToolResponse ToolPaste::begin_paste(const json &j, const Coordi &cursor_pos_canv
         for (auto &it : sch->sheets) {
             it.second.warnings.clear();
         }
+        auto &sheet = *doc.c->get_sheet();
+        for (const auto &[uu, net] : junction_nets) {
+            auto &ju = sheet.junctions.at(uu);
+            if (!ju.net)
+                ju.net = net;
+        }
+        for (const auto &[uu, net] : net_line_nets) {
+            auto &li = sheet.net_lines.at(uu);
+            if (!li.net)
+                li.net = net;
+        }
     }
+    update_airwires();
     return ToolResponse();
 }
 
@@ -534,6 +562,7 @@ ToolResponse ToolPaste::update(const ToolArgs &args)
         if (args.type == ToolEventType::MOVE) {
             move_do_cursor(args.coords);
             update_tip();
+            update_airwires();
             return ToolResponse();
         }
         else if (args.type == ToolEventType::ACTION) {
@@ -541,16 +570,7 @@ ToolResponse ToolPaste::update(const ToolArgs &args)
                 if (pic) {
                     return ToolResponse::commit();
                 }
-                merge_selected_junctions();
-                for (const auto &it : selection) {
-                    if (it.type == ObjectType::SCHEMATIC_SYMBOL) {
-                        auto sym = doc.c->get_schematic_symbol(it.uuid);
-                        doc.c->get_schematic()->autoconnect_symbol(doc.c->get_sheet(), sym);
-                        if (sym->component->connections.size() == 0) {
-                            doc.c->get_schematic()->place_bipole_on_line(doc.c->get_sheet(), sym);
-                        }
-                    }
-                }
+                merge_and_connect();
                 return ToolResponse::next(ToolResponse::Result::COMMIT, tool_id,
                                           std::make_unique<ToolDataPaste>(paste_data));
             }
@@ -569,4 +589,12 @@ ToolResponse ToolPaste::update(const ToolArgs &args)
     update_tip();
     return ToolResponse();
 }
+
+void ToolPaste::update_airwires()
+{
+    if (doc.b && nets.size()) {
+        doc.b->get_board()->update_airwires(true, nets);
+    }
+}
+
 } // namespace horizon
