@@ -12,7 +12,7 @@
 #include "pool/symbol.hpp"
 #include "imp/imp_interface.hpp"
 #include "util/accumulator.hpp"
-#include "util/util.hpp"
+#include "util/geom_util.hpp"
 #include <iostream>
 #include "core/tool_id.hpp"
 #include "dialogs/enter_datum_angle_window.hpp"
@@ -190,6 +190,11 @@ void ToolRotateArbitrary::save_placements()
             placements[it] = dec.placement;
             decal_scales[it.uuid] = dec.get_scale();
         } break;
+        case ObjectType::PICTURE: {
+            const auto pic = doc.r->get_picture(it.uuid);
+            placements[it] = pic->placement;
+            picture_px_sizes[it.uuid] = pic->px_size;
+        } break;
         case ObjectType::PAD:
             placements[it] = doc.k->get_package().pads.at(it.uuid).placement;
             break;
@@ -234,6 +239,9 @@ void ToolRotateArbitrary::apply_placements_rotation(int angle)
         case ObjectType::BOARD_DECAL:
             doc.b->get_board()->decals.at(it.first.uuid).placement = rotate_placement(it.second, origin, angle);
             break;
+        case ObjectType::PICTURE:
+            doc.r->get_picture(it.first.uuid)->placement = rotate_placement(it.second, origin, angle);
+            break;
         case ObjectType::PAD:
             doc.k->get_package().pads.at(it.first.uuid).placement = rotate_placement(it.second, origin, angle);
             break;
@@ -278,6 +286,11 @@ void ToolRotateArbitrary::apply_placements_scale(double sc)
             dec.placement = scale_placement(it.second, origin, sc);
             dec.set_scale(decal_scales.at(it.first.uuid) * sc);
         } break;
+        case ObjectType::PICTURE: {
+            auto pic = doc.r->get_picture(it.first.uuid);
+            pic->placement = scale_placement(it.second, origin, sc);
+            pic->px_size = picture_px_sizes.at(it.first.uuid) * sc;
+        } break;
         default:;
         }
     }
@@ -303,8 +316,8 @@ ToolResponse ToolRotateArbitrary::update(const ToolArgs &args)
             annotation->draw_line(origin, ref, ColorP::FROM_LAYER, 2);
         }
         else if (state == State::SCALE) {
-            const double vr = sqrt((ref - origin).mag_sq());
-            const double v = sqrt((args.coords - origin).mag_sq());
+            const double vr = (ref - origin).magd();
+            const double v = (args.coords - origin).magd();
             scale = v / vr;
             apply_placements_scale(scale);
             update_airwires(true);
@@ -315,16 +328,13 @@ ToolResponse ToolRotateArbitrary::update(const ToolArgs &args)
             const auto v0 = ref - origin;
             annotation->clear();
             annotation->draw_line(origin, ref, ColorP::FROM_LAYER, 2);
-            annotation->draw_arc(origin, sqrt(v0.mag_sq()) / 3, atan2(v0.y, v0.x), atan2(v.y, v.x), ColorP::FROM_LAYER,
-                                 2);
+            annotation->draw_arc(origin, v0.magd() / 3, atan2(v0.y, v0.x), atan2(v.y, v.x), ColorP::FROM_LAYER, 2);
             annotation->draw_line(origin, rref, ColorP::FROM_LAYER, 2);
 
 
             const double angle0 = atan2(v0.y, v0.x);
             const double angle = atan2(v.y, v.x) - angle0;
-            iangle = ((angle / (2 * M_PI)) * 65536);
-            iangle += 65536 * 2;
-            iangle %= 65536;
+            iangle = wrap_angle(angle_from_rad(angle));
             if (snap)
                 iangle = round_multiple(iangle, 8192);
             apply_placements_rotation(iangle);
