@@ -15,7 +15,6 @@
 #include "ask_datum.hpp"
 #include "ask_datum_string.hpp"
 #include "ask_datum_angle.hpp"
-#include "select_via_padstack.hpp"
 #include "annotate.hpp"
 #include "pool/part.hpp"
 #include "edit_shape.hpp"
@@ -38,6 +37,7 @@
 #include "enter_datum_angle_window.hpp"
 #include "enter_datum_scale_window.hpp"
 #include "router_settings_window.hpp"
+#include "edit_custom_value.hpp"
 #include <glibmm.h>
 #include "pool/ipool.hpp"
 #include "widgets/net_selector.hpp"
@@ -76,11 +76,13 @@ std::optional<UUIDPath<2>> Dialogs::map_symbol(const std::map<UUIDPath<2>, std::
     }
 }
 
-std::optional<UUID> Dialogs::select_symbol(IPool &pool, const UUID &unit_uuid)
+std::optional<UUID> Dialogs::select_symbol(IPool &pool, const UUID &unit_uuid, const UUID &sym_default)
 {
     PoolBrowserDialog dia(parent, ObjectType::SYMBOL, pool);
     auto &br = dynamic_cast<PoolBrowserSymbol &>(dia.get_browser());
     br.set_unit_uuid(unit_uuid);
+    if (sym_default)
+        br.go_to(sym_default);
     auto r = dia.run();
     if (r == Gtk::RESPONSE_OK) {
         return br.get_selected();
@@ -95,7 +97,8 @@ std::optional<UUID> Dialogs::select_padstack(IPool &pool, const UUID &package_uu
     PoolBrowserDialog dia(parent, ObjectType::PADSTACK, pool);
     auto &br = dynamic_cast<PoolBrowserPadstack &>(dia.get_browser());
     br.set_package_uuid(package_uuid);
-    br.set_include_padstack_type(Padstack::Type::MECHANICAL, true);
+    using PT = Padstack::Type;
+    br.set_padstacks_included({PT::TOP, PT::BOTTOM, PT::THROUGH, PT::MECHANICAL});
     if (dia.run() == Gtk::RESPONSE_OK) {
         return br.get_selected();
     }
@@ -108,11 +111,8 @@ std::optional<UUID> Dialogs::select_hole_padstack(IPool &pool)
 {
     PoolBrowserDialog dia(parent, ObjectType::PADSTACK, pool);
     auto &br = dynamic_cast<PoolBrowserPadstack &>(dia.get_browser());
-    br.set_include_padstack_type(Padstack::Type::MECHANICAL, true);
-    br.set_include_padstack_type(Padstack::Type::HOLE, true);
-    br.set_include_padstack_type(Padstack::Type::TOP, false);
-    br.set_include_padstack_type(Padstack::Type::THROUGH, false);
-    br.set_include_padstack_type(Padstack::Type::BOTTOM, false);
+    using PT = Padstack::Type;
+    br.set_padstacks_included({PT::MECHANICAL, PT::HOLE});
     if (dia.run() == Gtk::RESPONSE_OK) {
         return br.get_selected();
     }
@@ -211,9 +211,15 @@ bool Dialogs::edit_shapes(std::set<Shape *> shapes)
     return dia.run() == Gtk::RESPONSE_OK;
 }
 
-bool Dialogs::edit_via(std::set<class Via *> &vias, class ViaPadstackProvider &vpp)
+bool Dialogs::edit_via(std::set<class Via *> &vias, class IPool &pool, IPool &pool_caching)
 {
-    EditViaDialog dia(parent, vias, vpp);
+    EditViaDialog dia(parent, vias, pool, pool_caching);
+    return dia.run() == Gtk::RESPONSE_OK;
+}
+
+bool Dialogs::edit_custom_value(class SchematicSymbol &sym)
+{
+    EditCustomValueDialog dia(parent, sym);
     return dia.run() == Gtk::RESPONSE_OK;
 }
 
@@ -253,15 +259,15 @@ bool Dialogs::annotate(Schematic &s)
     return dia.run() == Gtk::RESPONSE_OK;
 }
 
-bool Dialogs::edit_board_hole(std::set<class BoardHole *> &holes, IPool &pool, Block &block)
+bool Dialogs::edit_board_hole(std::set<class BoardHole *> &holes, IPool &pool, IPool &pool_caching, Block &block)
 {
-    BoardHoleDialog dia(parent, holes, pool, block);
+    BoardHoleDialog dia(parent, holes, pool, pool_caching, block);
     return dia.run() == Gtk::RESPONSE_OK;
 }
 
-bool Dialogs::edit_schematic_properties(class Schematic &sch, class IPool &pool)
+bool Dialogs::edit_schematic_properties(class Schematic &sch, class IPool &pool, class IPool &pool_caching)
 {
-    SchematicPropertiesDialog dia(parent, sch, pool);
+    SchematicPropertiesDialog dia(parent, sch, pool, pool_caching);
     return dia.run() == Gtk::RESPONSE_OK;
 }
 
@@ -287,7 +293,7 @@ std::optional<int64_t> Dialogs::ask_datum(const std::string &label, int64_t def)
 
 std::optional<std::string> Dialogs::ask_datum_string(const std::string &label, const std::string &def)
 {
-    AskDatumStringDialog dia(parent, label, false);
+    AskDatumStringDialog dia(parent, label, TextEditor::Lines::SINGLE);
     dia.set_text(def);
     auto r = dia.run();
     if (r == Gtk::RESPONSE_OK) {
@@ -300,7 +306,7 @@ std::optional<std::string> Dialogs::ask_datum_string(const std::string &label, c
 
 std::optional<std::string> Dialogs::ask_datum_string_multiline(const std::string &label, const std::string &def)
 {
-    AskDatumStringDialog dia(parent, label, true);
+    AskDatumStringDialog dia(parent, label, TextEditor::Lines::MULTI);
     dia.set_text(def);
     auto r = dia.run();
     if (r == Gtk::RESPONSE_OK) {
@@ -389,12 +395,13 @@ std::optional<UUID> Dialogs::map_package(const std::vector<std::pair<Component *
     }
 }
 
-std::optional<UUID> Dialogs::select_via_padstack(class ViaPadstackProvider &vpp)
+std::optional<UUID> Dialogs::select_via_padstack(class IPool &pool)
 {
-    SelectViaPadstackDialog dia(parent, vpp);
-    auto r = dia.run();
-    if (r == Gtk::RESPONSE_OK && dia.selection_valid) {
-        return dia.selected_uuid;
+    PoolBrowserDialog dia(parent, ObjectType::PADSTACK, pool);
+    auto &br = dynamic_cast<PoolBrowserPadstack &>(dia.get_browser());
+    br.set_padstacks_included({Padstack::Type::VIA});
+    if (dia.run() == Gtk::RESPONSE_OK) {
+        return br.get_selected();
     }
     else {
         return {};
