@@ -4,11 +4,12 @@
 #include "export_gerber/gerber_export.hpp"
 #include "logger/logger.hpp"
 #include "pool/part.hpp"
-#include "pool/pool_cached.hpp"
+#include "pool/project_pool.hpp"
 #include "property_panels/property_panels.hpp"
 #include "rules/rules_window.hpp"
 #include "util/gtk_util.hpp"
 #include "util/util.hpp"
+#include "util/geom_util.hpp"
 #include "widgets/log_view.hpp"
 #include "widgets/log_window.hpp"
 #include "widgets/spin_button_dim.hpp"
@@ -26,11 +27,14 @@
 #include <functional>
 #include "nlohmann/json.hpp"
 #include "core/tool_id.hpp"
-#include "imp/action.hpp"
+#include "actions.hpp"
 #include "preferences/preferences_util.hpp"
 #include "widgets/action_button.hpp"
 #include "in_tool_action_catalog.hpp"
 #include "util/zmq_helper.hpp"
+#include "pool/pool_parametric.hpp"
+#include "action_icon.hpp"
+#include "grids_window.hpp"
 
 #ifdef G_OS_WIN32
 #include <winsock2.h>
@@ -39,36 +43,54 @@
 
 namespace horizon {
 
-std::unique_ptr<Pool> make_pool(const PoolParams &p)
+class PoolWithParametric : public ProjectPool {
+public:
+    PoolWithParametric(const std::string &bp, bool cache) : ProjectPool(bp, cache), parametric(bp)
+    {
+    }
+
+    PoolParametric *get_parametric() override
+    {
+        return &parametric;
+    }
+
+private:
+    PoolParametric parametric;
+};
+
+static std::unique_ptr<Pool> make_pool(const PoolParams &p)
 {
-    if (p.cache_path.size() && Glib::file_test(p.cache_path, Glib::FILE_TEST_IS_DIR)) {
-        return std::make_unique<PoolCached>(p.base_path, p.cache_path);
+    if (PoolInfo(p.base_path).uuid == PoolInfo::project_pool_uuid) {
+        return std::make_unique<PoolWithParametric>(p.base_path, false);
     }
     else {
         return std::make_unique<Pool>(p.base_path);
     }
 }
 
+
+static std::unique_ptr<Pool> make_pool_caching(const PoolParams &p)
+{
+    if (PoolInfo(p.base_path).uuid == PoolInfo::project_pool_uuid) {
+        return std::make_unique<PoolWithParametric>(p.base_path, true);
+    }
+    else {
+        return nullptr;
+    }
+}
+
+
 ImpBase::ImpBase(const PoolParams &params)
-    : pool(make_pool(params)), core(nullptr), sock_broadcast_rx(zctx, ZMQ_SUB), sock_project(zctx, ZMQ_REQ)
+    : pool(make_pool(params)), real_pool_caching(make_pool_caching(params)),
+      pool_caching(real_pool_caching ? real_pool_caching.get() : pool.get()), core(nullptr),
+      sock_broadcast_rx(zctx, ZMQ_SUB), sock_project(zctx, ZMQ_REQ), drag_tool(ToolID::NONE)
 {
     auto ep_broadcast = Glib::getenv("HORIZON_EP_BROADCAST");
     if (ep_broadcast.size()) {
         sock_broadcast_rx.connect(ep_broadcast);
-        {
-            unsigned int prefix = 0;
-            sock_broadcast_rx.setsockopt(ZMQ_SUBSCRIBE, &prefix, 4);
-            prefix = getpid();
-            sock_broadcast_rx.setsockopt(ZMQ_SUBSCRIBE, &prefix, 4);
-        }
-        Glib::RefPtr<Glib::IOChannel> chan;
-#ifdef G_OS_WIN32
-        SOCKET fd = sock_broadcast_rx.getsockopt<SOCKET>(ZMQ_FD);
-        chan = Glib::IOChannel::create_from_win32_socket(fd);
-#else
-        int fd = sock_broadcast_rx.getsockopt<int>(ZMQ_FD);
-        chan = Glib::IOChannel::create_from_fd(fd);
-#endif
+        zmq_helper::subscribe_int(sock_broadcast_rx, 0);
+        zmq_helper::subscribe_int(sock_broadcast_rx, getpid());
+        auto chan = zmq_helper::io_channel_from_socket(sock_broadcast_rx);
 
         {
             auto pid_p = Glib::getenv("HORIZON_MGR_PID");
@@ -78,7 +100,7 @@ ImpBase::ImpBase(const PoolParams &params)
 
         Glib::signal_io().connect(
                 [this](Glib::IOCondition cond) {
-                    while (sock_broadcast_rx.getsockopt<int>(ZMQ_EVENTS) & ZMQ_POLLIN) {
+                    while (zmq_helper::can_recv(sock_broadcast_rx)) {
                         zmq::message_t msg;
                         if (zmq_helper::recv(sock_broadcast_rx, msg)) {
                             int prefix;
@@ -97,8 +119,7 @@ ImpBase::ImpBase(const PoolParams &params)
     auto ep_project = Glib::getenv("HORIZON_EP_MGR");
     if (ep_project.size()) {
         sock_project.connect(ep_project);
-        sock_project.setsockopt(ZMQ_RCVTIMEO, 5000);
-        sock_project.setsockopt(ZMQ_SNDTIMEO, 5000);
+        zmq_helper::set_timeouts(sock_project, 5000);
     }
     sockets_connected = ep_project.size() && ep_broadcast.size();
 }
@@ -553,12 +574,53 @@ void ImpBase::run(int argc, char *argv[])
 
     init_search();
 
-    grid_controller.emplace(*main_window, *canvas);
+    if (auto grid_settings = core->get_grid_settings()) {
+        if (!m_meta.is_null()) {
+            if (m_meta.count("grid_settings")) {
+                const auto &s = m_meta.at("grid_settings");
+                auto &cur = grid_settings->current;
+                cur.spacing_square = s.at("spacing_square").get<uint64_t>();
+                cur.origin.x = s.at("origin_x").get<int64_t>();
+                cur.origin.y = s.at("origin_y").get<int64_t>();
+                cur.spacing_rect.x = s.at("spacing_x").get<int64_t>();
+                cur.spacing_rect.y = s.at("spacing_y").get<int64_t>();
+                if (s.at("mode").get<std::string>() == "rect") {
+                    cur.mode = GridSettings::Grid::Mode::RECTANGULAR;
+                }
+                else {
+                    cur.mode = GridSettings::Grid::Mode::SQUARE;
+                }
+            }
+            else {
+                grid_settings->current.spacing_square = m_meta.value("grid_spacing", 1.25_mm);
+            }
+        }
+    }
+
+    grid_controller.emplace(*main_window, *canvas, core->get_grid_settings());
     connect_action(ActionID::SET_GRID_ORIGIN, [this](const auto &c) {
         auto co = canvas->get_cursor_pos();
         grid_controller->set_origin(co);
     });
 
+    if (auto grid_settings = core->get_grid_settings()) {
+        grids_window = GridsWindow::create(main_window, *grid_controller, *grid_settings);
+        connect_action(ActionID::GRIDS_WINDOW, [this](const auto &c) {
+            grids_window->set_select_mode(false);
+            grids_window->present();
+        });
+        connect_action(ActionID::SELECT_GRID, [this](const auto &c) {
+            grids_window->set_select_mode(true);
+            grids_window->present();
+        });
+        main_window->grid_window_button->signal_clicked().connect([this] { trigger_action(ActionID::GRIDS_WINDOW); });
+        grids_window->signal_changed().connect([this] {
+            core->set_needs_save();
+            set_action_sensitive(make_action(ActionID::SELECT_GRID), grids_window->has_grids());
+        });
+        set_action_sensitive(make_action(ActionID::SELECT_GRID), grids_window->has_grids());
+    }
+
     auto save_button = create_action_button(make_action(ActionID::SAVE));
     save_button->show();
     main_window->header->pack_start(*save_button);
@@ -766,9 +828,7 @@ void ImpBase::run(int argc, char *argv[])
     Gtk::StyleContext::add_provider_for_screen(Gdk::Screen::get_default(), cssp, 700);
 
     canvas->signal_motion_notify_event().connect([this](GdkEventMotion *ev) {
-        if (selection_for_drag_move.size()) {
-            handle_drag(ev->state & Gdk::CONTROL_MASK);
-        }
+        handle_drag();
         return false;
     });
 
@@ -815,6 +875,8 @@ void ImpBase::run(int argc, char *argv[])
     }
     update_view_hints();
 
+    reset_tool_hint_label();
+
     check_version();
 
     {
@@ -881,26 +943,32 @@ ToolID ImpBase::get_tool_for_drag_move(bool ctrl, const std::set<SelectableRef>
     if (preferences.mouse.drag_polygon_edges && sel.size() == 1 && sel.begin()->type == ObjectType::POLYGON_EDGE)
         return ToolID::DRAG_POLYGON_EDGE;
 
-    return ToolID::MOVE;
+    if (preferences.mouse.drag_to_move)
+        return ToolID::MOVE;
+    else
+        return ToolID::NONE;
 }
 
-void ImpBase::handle_drag(bool ctrl)
+void ImpBase::handle_drag()
 {
+    if (drag_tool == ToolID::NONE)
+        return;
+    if (selection_for_drag_move.size() == 0)
+        return;
     auto pos = canvas->get_cursor_pos_win();
     auto delta = pos - cursor_pos_drag_begin;
     if (delta.mag_sq() > (50 * 50)) {
-        {
-            highlights.clear();
-            update_highlights();
-            ToolArgs args;
-            args.coords = cursor_pos_grid_drag_begin;
-            args.selection = selection_for_drag_move;
-            ToolID tool_id = get_tool_for_drag_move(ctrl, selection_for_drag_move);
+        highlights.clear();
+        update_highlights();
+        ToolArgs args;
+        args.coords = cursor_pos_grid_drag_begin;
+        args.selection = selection_for_drag_move;
+
+        ToolResponse r = core->tool_begin(drag_tool, args, imp_interface.get(), true);
+        tool_process(r);
 
-            ToolResponse r = core->tool_begin(tool_id, args, imp_interface.get(), true);
-            tool_process(r);
-        }
         selection_for_drag_move.clear();
+        drag_tool = ToolID::NONE;
     }
 }
 
@@ -1176,6 +1244,19 @@ void ImpBase::create_context_menu(Gtk::Menu *parent, const std::set<SelectableRe
         delete sep;
 }
 
+void ImpBase::reset_tool_hint_label()
+{
+    const auto act = make_action(ActionID::POPOVER);
+    if (action_connections.count(act)) {
+        if (action_connections.at(act).key_sequences.size()) {
+            const auto keys = key_sequence_to_string(action_connections.at(act).key_sequences.front());
+            main_window->tool_hint_label->set_text("> " + keys + " for menu");
+            return;
+        }
+    }
+    main_window->tool_hint_label->set_text(">");
+}
+
 bool ImpBase::handle_click(const GdkEventButton *button_event)
 {
     if (button_event->button > 3) {
@@ -1186,6 +1267,8 @@ bool ImpBase::handle_click(const GdkEventButton *button_event)
     if (button_event->button != 2)
         set_search_mode(false);
 
+    main_window->key_hint_set_visible(false);
+
     bool need_menu = false;
     if (core->tool_is_active() && button_event->button != 2 && !(button_event->state & Gdk::SHIFT_MASK)
         && button_event->type != GDK_2BUTTON_PRESS && button_event->type != GDK_3BUTTON_PRESS) {
@@ -1212,7 +1295,7 @@ bool ImpBase::handle_click(const GdkEventButton *button_event)
         }
     }
     else if (!core->tool_is_active() && button_event->button == 1 && !(button_event->state & Gdk::SHIFT_MASK)) {
-        handle_maybe_drag();
+        handle_maybe_drag(button_event->state & Gdk::CONTROL_MASK);
     }
     else if (!core->tool_is_active() && button_event->button == 3) {
         for (const auto it : context_menu->get_children()) {
@@ -1292,7 +1375,7 @@ std::string ImpBase::get_complete_display_name(const SelectableRef &sr)
 }
 
 
-void ImpBase::handle_maybe_drag()
+void ImpBase::handle_maybe_drag(bool ctrl)
 {
     auto c = canvas->screen2canvas(canvas->get_cursor_pos_win());
     auto sel_at_cursor = canvas->get_selection_at(Coordi(c.x, c.y));
@@ -1301,10 +1384,13 @@ void ImpBase::handle_maybe_drag()
     std::set_intersection(sel_from_canvas.begin(), sel_from_canvas.end(), sel_at_cursor.begin(), sel_at_cursor.end(),
                           std::inserter(isect, isect.begin()));
     if (isect.size()) {
-        canvas->inhibit_drag_selection();
-        cursor_pos_drag_begin = canvas->get_cursor_pos_win();
-        cursor_pos_grid_drag_begin = canvas->get_cursor_pos();
-        selection_for_drag_move = sel_from_canvas;
+        drag_tool = get_tool_for_drag_move(ctrl, sel_from_canvas);
+        if (drag_tool != ToolID::NONE) {
+            canvas->inhibit_drag_selection();
+            cursor_pos_drag_begin = canvas->get_cursor_pos_win();
+            cursor_pos_grid_drag_begin = canvas->get_cursor_pos();
+            selection_for_drag_move = sel_from_canvas;
+        }
     }
 }
 
@@ -1312,7 +1398,7 @@ void ImpBase::tool_process(ToolResponse &resp)
 {
     if (!core->tool_is_active()) {
         imp_interface->dialogs.close_nonmodal();
-        main_window->tool_hint_label->set_text(">");
+        reset_tool_hint_label();
         canvas->set_cursor_external(false);
         canvas->snap_filter.clear();
         no_update = false;
@@ -1359,6 +1445,13 @@ void ImpBase::handle_selection_changed(void)
         update_highlights();
         update_property_panels();
     }
+    if (sockets_connected) {
+        auto selection = canvas->get_selection();
+        if (selection != last_canvas_selection) {
+            handle_selection_cross_probe();
+        }
+        last_canvas_selection = selection;
+    }
 }
 
 void ImpBase::update_property_panels()
@@ -1406,6 +1499,7 @@ void ImpBase::handle_tool_change(ToolID id)
     main_window->tool_bar_set_visible(id != ToolID::NONE);
     tool_bar_clear_actions();
     main_window->action_bar_revealer->set_reveal_child(preferences.action_bar.show_in_tool || id == ToolID::NONE);
+    update_cursor(id);
 }
 
 void ImpBase::handle_warning_selected(const Coordi &pos)
@@ -1457,7 +1551,13 @@ void ImpBase::set_monitor_items(const ItemSet &items)
     std::set<std::string> filenames;
     std::transform(items.begin(), items.end(), std::inserter(filenames, filenames.begin()),
                    [this](const auto &it) { return pool->get_filename(it.first, it.second); });
-    set_monitor_files(filenames);
+    file_monitor_delay_connection.disconnect();
+    file_monitor_delay_connection = Glib::signal_timeout().connect_seconds(
+            [this, filenames] {
+                set_monitor_files(filenames);
+                return false;
+            },
+            1);
 }
 
 void ImpBase::handle_file_changed(const Glib::RefPtr<Gio::File> &file1, const Glib::RefPtr<Gio::File> &file2,
@@ -1625,4 +1725,69 @@ void ImpBase::view_options_menu_append_action(const std::string &label, const st
     bu->show();
 }
 
+static const char *get_cursor_icon(ToolID id)
+{
+    switch (id) {
+    case ToolID::PLACE_TEXT:
+    case ToolID::PLACE_REFDES_AND_VALUE:
+    case ToolID::PLACE_SHAPE:
+    case ToolID::PLACE_SHAPE_OBROUND:
+    case ToolID::PLACE_SHAPE_RECTANGLE:
+    case ToolID::PLACE_HOLE:
+    case ToolID::PLACE_HOLE_SLOT:
+    case ToolID::PLACE_BOARD_HOLE:
+    case ToolID::PLACE_PAD:
+    case ToolID::PLACE_POWER_SYMBOL:
+    case ToolID::PLACE_NET_LABEL:
+    case ToolID::PLACE_BUS_LABEL:
+    case ToolID::PLACE_BUS_RIPPER:
+    case ToolID::PLACE_VIA:
+    case ToolID::PLACE_DOT:
+        return nullptr;
+
+    default:
+        return get_action_icon(make_action(id));
+    }
+}
+
+void ImpBase::update_cursor(ToolID tool_id)
+{
+    auto win = canvas->get_window();
+    if (tool_id == ToolID::NONE) {
+        win->set_cursor();
+        return;
+    }
+    static const int icon_size = 16;
+    static const int border_width = 1;
+    auto surf = Cairo::ImageSurface::create(Cairo::FORMAT_ARGB32, icon_size + border_width * 2,
+                                            icon_size + border_width * 2);
+    auto cr = Cairo::Context::create(surf);
+    auto theme = Gtk::IconTheme::get_default();
+    const auto icon_name = get_cursor_icon(tool_id);
+    if (!icon_name) {
+        win->set_cursor();
+        return;
+    }
+    auto icon_info = theme->lookup_icon(icon_name, icon_size);
+    if (!icon_info) {
+        win->set_cursor();
+        return;
+    }
+    bool was_symbolic = false;
+    auto pb_black = icon_info.load_symbolic(Gdk::RGBA("#000000"), Gdk::RGBA(), Gdk::RGBA(), Gdk::RGBA(), was_symbolic);
+    Gdk::Cairo::set_source_pixbuf(cr, pb_black, border_width, border_width);
+    auto pat = cr->get_source();
+    for (int x : {-1, 1, 0}) {
+        for (int y : {-1, 1, 0}) {
+            cr->save();
+            cr->translate(x * border_width, y * border_width);
+            cr->set_source_rgb(1, 1, 1);
+            cr->mask(pat);
+            cr->restore();
+        }
+    }
+    Gdk::Cairo::set_source_pixbuf(cr, pb_black, border_width, border_width);
+    cr->paint();
+    win->set_cursor(Gdk::Cursor::create(win->get_display(), surf, 0, 0));
+}
 } // namespace horizon
