@@ -26,15 +26,12 @@ namespace horizon {
 
 class PoolParams {
 public:
-    PoolParams(const std::string &bp, const std::string &cp = "") : base_path(bp), cache_path(cp)
+    PoolParams(const std::string &bp) : base_path(bp)
     {
     }
     std::string base_path;
-    std::string cache_path;
 };
 
-std::unique_ptr<Pool> make_pool(const PoolParams &params);
-
 class ImpBase {
     friend class ImpInterface;
 
@@ -84,8 +81,11 @@ protected:
     Gtk::Menu *context_menu = nullptr;
     std::unique_ptr<SelectionFilterDialog> selection_filter_dialog;
     std::optional<GridController> grid_controller;
+    class GridsWindow *grids_window = nullptr;
 
     std::unique_ptr<Pool> pool;
+    std::unique_ptr<Pool> real_pool_caching;
+    Pool *pool_caching;
     class Core *core = nullptr;
     std::unique_ptr<ClipboardBase> clipboard = nullptr;
     std::unique_ptr<ClipboardHandler> clipboard_handler = nullptr;
@@ -112,6 +112,9 @@ protected:
     virtual void expand_selection_for_property_panel(std::set<SelectableRef> &sel_extra,
                                                      const std::set<SelectableRef> &sel);
     void handle_selection_changed(void);
+    virtual void handle_selection_cross_probe()
+    {
+    }
     bool handle_key_press(const GdkEventKey *key_event);
     void handle_cursor_move(const Coordi &pos);
     bool handle_click(const GdkEventButton *button_event);
@@ -147,7 +150,7 @@ protected:
 
     std::unique_ptr<WindowStateStore> state_store = nullptr;
 
-    virtual void handle_maybe_drag();
+    virtual void handle_maybe_drag(bool ctrl);
 
     virtual ActionCatalogItem::Availability get_editor_type_for_action() const = 0;
     ObjectType get_editor_type() const;
@@ -231,17 +234,19 @@ protected:
 
     virtual ToolID get_tool_for_drag_move(bool ctrl, const std::set<SelectableRef> &sel) const;
     void force_end_tool();
+    void reset_tool_hint_label();
 
 private:
     void fix_cursor_pos();
     Glib::RefPtr<Gio::FileMonitor> preferences_monitor;
-    void handle_drag(bool ctrl);
+    void handle_drag();
     void update_selection_label();
     std::string get_tool_settings_filename(ToolID id);
 
     void hud_update();
 
     std::map<std::string, Glib::RefPtr<Gio::FileMonitor>> file_monitors;
+    sigc::connection file_monitor_delay_connection;
 
     void handle_file_changed(const Glib::RefPtr<Gio::File> &file1, const Glib::RefPtr<Gio::File> &file2,
                              Gio::FileMonitorEvent ev);
@@ -253,8 +258,7 @@ private:
     Gtk::MenuItem *create_context_menu_item(ActionToolID act);
 
     KeySequence keys_current;
-    enum class MatchResult { NONE, PREFIX, COMPLETE };
-    MatchResult keys_match(const KeySequence &keys) const;
+    KeyMatchResult keys_match(const KeySequence &keys) const;
     bool handle_action_key(const GdkEventKey *ev);
     void handle_tool_action(const ActionConnection &conn);
     void handle_select_polygon(const ActionConnection &a);
@@ -270,6 +274,7 @@ private:
 
     class LogWindow *log_window = nullptr;
     std::set<SelectableRef> selection_for_drag_move;
+    ToolID drag_tool;
     Coordf cursor_pos_drag_begin;
     Coordi cursor_pos_grid_drag_begin;
 
@@ -314,5 +319,9 @@ private:
     void set_flip_view(bool flip);
     void apply_arrow_keys();
     void check_version();
+
+    void update_cursor(ToolID tool_id);
+
+    std::set<SelectableRef> last_canvas_selection;
 };
 } // namespace horizon
