@@ -1,5 +1,5 @@
 #include "imp_board.hpp"
-#include "3d_view.hpp"
+#include "3d/3d_view.hpp"
 #include "canvas/canvas_gl.hpp"
 #include "canvas/canvas_pads.hpp"
 #include "fab_output_window.hpp"
@@ -11,6 +11,7 @@
 #include "tuning_window.hpp"
 #include "util/selection_util.hpp"
 #include "util/util.hpp"
+#include "util/geom_util.hpp"
 #include "util/str_util.hpp"
 #include "canvas/annotation.hpp"
 #include "export_pdf/export_pdf_board.hpp"
@@ -22,12 +23,13 @@
 #include "core/tool_id.hpp"
 #include "widgets/action_button.hpp"
 #include "parts_window.hpp"
-#include "pool/pool_cached.hpp"
+#include "actions.hpp"
+#include "in_tool_action.hpp"
 
 namespace horizon {
-ImpBoard::ImpBoard(const std::string &board_filename, const std::string &block_filename, const std::string &via_dir,
+ImpBoard::ImpBoard(const std::string &board_filename, const std::string &block_filename,
                    const std::string &pictures_dir, const PoolParams &pool_params)
-    : ImpLayer(pool_params), core_board(board_filename, block_filename, via_dir, pictures_dir, *pool),
+    : ImpLayer(pool_params), core_board(board_filename, block_filename, pictures_dir, *pool, *pool_caching),
       project_dir(Glib::path_get_dirname(board_filename)), searcher(core_board)
 {
     core = &core_board;
@@ -197,8 +199,6 @@ bool ImpBoard::handle_broadcast(const json &j)
 
 void ImpBoard::handle_selection_cross_probe()
 {
-    if (core_board.tool_is_active())
-        return;
     json j;
     j["op"] = "board-select";
     j["selection"] = nullptr;
@@ -295,8 +295,7 @@ void ImpBoard::update_action_sensitivity()
 void ImpBoard::apply_preferences()
 {
     if (view_3d_window) {
-        view_3d_window->set_smooth_zoom(preferences.zoom.smooth_zoom_3d);
-        view_3d_window->set_appearance(preferences.canvas_layer.appearance);
+        view_3d_window->apply_preferences(preferences);
     }
     canvas->set_highlight_on_top(preferences.board.highlight_on_top);
     canvas->show_text_in_tracks = preferences.board.show_text_in_tracks;
@@ -472,8 +471,6 @@ void ImpBoard::construct()
             }
         });
 
-        canvas->signal_selection_changed().connect(sigc::mem_fun(*this, &ImpBoard::handle_selection_cross_probe));
-
         connect_action(ActionID::GO_TO_SCHEMATIC, [this](const auto &conn) {
             json j;
             j["time"] = gtk_get_current_event_time();
@@ -576,20 +573,24 @@ void ImpBoard::construct()
         pdf_export_window->generate();
     });
 
-    view_3d_window = View3DWindow::create(*core_board.get_board(), *pool.get(), View3DWindow::Mode::BOARD);
+    view_3d_window = View3DWindow::create(*core_board.get_board(), *pool_caching, View3DWindow::Mode::BOARD);
     view_3d_window->set_solder_mask_color(rgba_from_color(core_board.get_colors().solder_mask));
+    view_3d_window->set_silkscreen_color(rgba_from_color(core_board.get_colors().silkscreen));
     view_3d_window->set_substrate_color(rgba_from_color(core_board.get_colors().substrate));
     view_3d_window->signal_changed().connect([this] {
         core_board.get_colors().solder_mask = color_from_rgba(view_3d_window->get_solder_mask_color());
+        core_board.get_colors().silkscreen = color_from_rgba(view_3d_window->get_silkscreen_color());
         core_board.get_colors().substrate = color_from_rgba(view_3d_window->get_substrate_color());
         core_board.set_needs_save();
     });
-    view_3d_window->signal_key_press_event().connect([this](GdkEventKey *ev) {
-        if (ev->keyval == GDK_KEY_Escape) {
-            main_window->present();
-            return true;
+    view_3d_window->signal_present_imp().connect([this] { main_window->present(); });
+    view_3d_window->signal_package_select().connect([this](const auto &uu) {
+        if (!core->tool_is_active() && canvas->get_selection_mode() == CanvasGL::SelectionMode::NORMAL) {
+            if (uu)
+                canvas->set_selection({{uu, ObjectType::BOARD_PACKAGE}});
+            else
+                canvas->set_selection({});
         }
-        return false;
     });
 
     step_export_window = StepExportWindow::create(main_window, core_board, project_dir);
@@ -912,14 +913,16 @@ std::string ImpBoard::get_hud_text(std::set<SelectableRef> &sel)
              + "</b>\n";
         std::set<int> layers;
         std::set<const Net *> nets;
+        std::vector<const Track *> tracks;
         int64_t length = 0;
         const Track *the_track = nullptr;
         for (const auto &it : sel) {
             if (it.type == ObjectType::TRACK) {
                 const auto &tr = core_board.get_board()->tracks.at(it.uuid);
                 the_track = &tr;
+                tracks.emplace_back(the_track);
                 layers.insert(tr.layer);
-                length += sqrt((tr.from.get_position() - tr.to.get_position()).mag_sq());
+                length += (tr.from.get_position() - tr.to.get_position()).magd();
                 if (tr.net)
                     nets.insert(tr.net);
             }
@@ -929,7 +932,21 @@ std::string ImpBoard::get_hud_text(std::set<SelectableRef> &sel)
             s += core->get_layer_provider().get_layers().at(layer).name + " ";
         }
         s += "\nTotal length: " + dim_to_string(length, false);
-        if (sel_count_type(sel, ObjectType::TRACK) == 1) {
+
+        if (n == 2) {
+            // Show spacing between 2 parallel tracks
+            const auto t1 = *tracks.at(0);
+            const auto t2 = *tracks.at(1);
+            if (t1.is_parallel_to(t2)) {
+                const Coordd u = t1.to.get_position() - t1.from.get_position();
+                const Coordd v = u.normalize();
+                const Coordd w = t2.from.get_position() - t1.from.get_position();
+                const int64_t offset = v.cross(w);
+                s += "\nSpacing: " + dim_to_string(offset, false);
+            }
+        }
+
+        if (n == 1) {
             s += "\n" + get_hud_text_for_net(the_track->net);
         }
         else {
@@ -939,8 +956,8 @@ std::string ImpBoard::get_hud_text(std::set<SelectableRef> &sel)
         sel_erase_type(sel, ObjectType::TRACK);
     }
     trim(s);
-    if (sel_count_type(sel, ObjectType::BOARD_PACKAGE) == 1) {
-        const auto &pkg = core_board.get_board()->packages.at(sel_find_one(sel, ObjectType::BOARD_PACKAGE).uuid);
+    if (auto it_sel = sel_find_exactly_one(sel, ObjectType::BOARD_PACKAGE)) {
+        const auto &pkg = core_board.get_board()->packages.at(it_sel->uuid);
         s += "\n\n<b>Package " + pkg.component->refdes + "</b>";
         if (pkg.fixed) {
             s += " (not movable)";
@@ -961,7 +978,11 @@ std::string ImpBoard::get_hud_text(std::set<SelectableRef> &sel)
             }
         }
         s += "Total pads: " + std::to_string(n_pads);
-        sel_erase_type(sel, ObjectType::BOARD_PACKAGE);
+
+        // When n == 2 selection is removed when showing delta below
+        if (n != 2) {
+            sel_erase_type(sel, ObjectType::BOARD_PACKAGE);
+        }
     }
     trim(s);
     if (sel_count_type(sel, ObjectType::POLYGON_VERTEX) == 1 || sel_count_type(sel, ObjectType::POLYGON_EDGE) == 1) {
@@ -974,13 +995,50 @@ std::string ImpBoard::get_hud_text(std::set<SelectableRef> &sel)
         if (poly) {
             if (auto plane = dynamic_cast<const Plane *>(poly->usage.ptr)) {
                 s += "\n\n<b>Plane " + plane->net->name + "</b>\n";
-                s += "Priority: " + std::to_string(plane->priority) + "\n";
+                s += "Fill order: " + std::to_string(plane->priority) + "\n";
                 s += "Layer: ";
                 s += core_board.get_layer_provider().get_layers().at(poly->layer).name + "\n";
             }
         }
     }
     trim(s);
+
+    // Display the delta if two items of these types are selected
+    for (const ObjectType type : {ObjectType::BOARD_PACKAGE, ObjectType::BOARD_HOLE, ObjectType::VIA}) {
+        if (sel_count_type(sel, type) == 2) {
+            // Already added for packages
+            if (type != ObjectType::BOARD_PACKAGE) {
+                s += "\n\n<b>2 " + object_descriptions.at(type).name_pl + "</b>";
+            }
+            std::vector<Coordi> positions;
+            const auto &brd = *core_board.get_board();
+            for (const auto &it : sel) {
+                if (it.type == type) {
+                    if (type == ObjectType::BOARD_HOLE) {
+                        const auto &hole = &brd.holes.at(it.uuid);
+                        positions.push_back(hole->placement.shift);
+                    }
+                    else if (type == ObjectType::VIA) {
+                        const auto &via = &brd.vias.at(it.uuid);
+                        positions.push_back(via->junction->position);
+                    }
+                    else if (type == ObjectType::BOARD_PACKAGE) {
+                        const auto &package = &brd.packages.at(it.uuid);
+                        positions.push_back(package->placement.shift);
+                    }
+                    else {
+                        assert(false); // unreachable
+                    }
+                }
+            }
+            assert(positions.size() == 2);
+            const auto delta = positions.at(1) - positions.at(0);
+            s += "\n" + coord_to_string(delta, true);
+            sel_erase_type(sel, type);
+        }
+    }
+    trim(s);
+
     return s;
 }
 
@@ -1005,10 +1063,10 @@ ToolID ImpBoard::get_tool_for_drag_move(bool ctrl, const std::set<SelectableRef>
         return ImpBase::get_tool_for_drag_move(ctrl, sel);
 }
 
-void ImpBoard::handle_maybe_drag()
+void ImpBoard::handle_maybe_drag(bool ctrl)
 {
     if (!preferences.board.drag_start_track) {
-        ImpBase::handle_maybe_drag();
+        ImpBase::handle_maybe_drag(ctrl);
         return;
     }
     auto target = canvas->get_current_target();
@@ -1017,19 +1075,19 @@ void ImpBoard::handle_maybe_drag()
         auto &pkg = brd->packages.at(target.path.at(0));
         auto &pad = pkg.package.pads.at(target.path.at(1));
         if (pad.padstack.type == Padstack::Type::MECHANICAL || pad.is_nc) {
-            ImpBase::handle_maybe_drag();
+            ImpBase::handle_maybe_drag(ctrl);
             return;
         }
     }
     else if (target.type == ObjectType::JUNCTION) {
         const auto &ju = brd->junctions.at(target.path.at(0));
         if (!ju.net) {
-            ImpBase::handle_maybe_drag();
+            ImpBase::handle_maybe_drag(ctrl);
             return;
         }
     }
     else {
-        ImpBase::handle_maybe_drag();
+        ImpBase::handle_maybe_drag(ctrl);
         return;
     }
     canvas->inhibit_drag_selection();
