@@ -6,8 +6,8 @@
 namespace horizon {
 class ImpBoard : public ImpLayer {
 public:
-    ImpBoard(const std::string &board_filename, const std::string &block_filename, const std::string &via_dir,
-             const std::string &pictures_dir, const PoolParams &params);
+    ImpBoard(const std::string &board_filename, const std::string &block_filename, const std::string &pictures_dir,
+             const PoolParams &params);
 
     const std::map<int, Layer> &get_layers();
     void update_highlights() override;
@@ -19,7 +19,7 @@ public:
 protected:
     void construct() override;
     bool handle_broadcast(const json &j) override;
-    void handle_maybe_drag() override;
+    void handle_maybe_drag(bool ctrl) override;
     void update_action_sensitivity() override;
     void apply_preferences() override;
 
@@ -45,7 +45,7 @@ protected:
 
 private:
     void canvas_update() override;
-    void handle_selection_cross_probe();
+    void handle_selection_cross_probe() override;
 
     CoreBoard core_board;
     const std::string project_dir;
