@@ -1,6 +1,7 @@
 #include "imp.hpp"
 #include "util/str_util.hpp"
 #include "util/util.hpp"
+#include "util/geom_util.hpp"
 #include "util/selection_util.hpp"
 #include "pool/entity.hpp"
 #include "pool/part.hpp"
@@ -42,7 +43,7 @@ std::string ImpBase::get_hud_text(std::set<SelectableRef> &sel)
             if (it.type == ObjectType::LINE) {
                 const auto li = core->get_line(it.uuid);
                 layers.insert(li->layer);
-                length += sqrt((li->from->position - li->to->position).mag_sq());
+                length += (li->from->position - li->to->position).magd();
             }
         }
         s += "Layers: ";
@@ -54,52 +55,43 @@ std::string ImpBase::get_hud_text(std::set<SelectableRef> &sel)
     }
 
     // Display the length if a single edge of a polygon is given
-    if (sel_count_type(sel, ObjectType::POLYGON_EDGE) == 1) {
-        auto n = sel_count_type(sel, ObjectType::POLYGON_EDGE);
-        s += "\n\n<b>" + std::to_string(n) + " " + object_descriptions.at(ObjectType::POLYGON_EDGE).get_name_for_n(n)
-             + "</b>\n";
-        int64_t length = 0;
-        for (const auto &it : sel) {
-            if (it.type == ObjectType::POLYGON_EDGE) {
-                const auto li = core->get_polygon(it.uuid);
-                const auto pair = li->get_vertices_for_edge(it.vertex);
-                length += sqrt((li->vertices[pair.first].position - li->vertices[pair.second].position).mag_sq());
-                s += "Layer: ";
-                s += core->get_layer_provider().get_layers().at(li->layer).name + " ";
-                s += "\nLength: " + dim_to_string(length, false);
-                if (preferences.hud_debug)
-                    s += "\nVertex: " + std::to_string(it.vertex);
-                sel_erase_type(sel, ObjectType::POLYGON_EDGE);
-                break;
-            }
-        }
+    if (auto it = sel_find_exactly_one(sel, ObjectType::POLYGON_EDGE)) {
+        s += "\n\n<b>" + object_descriptions.at(ObjectType::POLYGON_EDGE).name + "</b>\n";
+        const auto li = core->get_polygon(it->uuid);
+        const auto pair = li->get_vertices_for_edge(it->vertex);
+        const int64_t length = (li->vertices.at(pair.first).position - li->vertices.at(pair.second).position).magd();
+        s += "Layer: ";
+        s += core->get_layer_provider().get_layers().at(li->layer).name + " ";
+        s += "\nLength: " + dim_to_string(length, false);
+        if (preferences.hud_debug)
+            s += "\nVertex: " + std::to_string(it->vertex);
+        sel_erase_type(sel, ObjectType::POLYGON_EDGE);
     }
-    if (preferences.hud_debug && (sel_count_type(sel, ObjectType::POLYGON_VERTEX) == 1)) {
-        auto n = sel_count_type(sel, ObjectType::POLYGON_VERTEX);
-        s += "\n\n<b>" + std::to_string(n) + " " + object_descriptions.at(ObjectType::POLYGON_VERTEX).get_name_for_n(n)
-             + "</b>\n";
-        for (const auto &it : sel) {
-            if (it.type == ObjectType::POLYGON_VERTEX) {
-                s += "Vertex: " + std::to_string(it.vertex);
-                sel_erase_type(sel, ObjectType::POLYGON_VERTEX);
-                break;
+
+    if (auto it = sel_find_exactly_one(sel, ObjectType::POLYGON_VERTEX)) {
+        s += "\n\n<b>" + object_descriptions.at(ObjectType::POLYGON_VERTEX).name + "</b>\n";
+        const auto poly = core->get_polygon(it->uuid);
+        s += coord_to_string(poly->vertices.at(it->vertex).position);
+        if (preferences.hud_debug)
+            s += "\nVertex: " + std::to_string(it->vertex);
+        sel_erase_type(sel, ObjectType::POLYGON_VERTEX);
+    }
+
+    if (preferences.hud_debug) {
+        if (auto it = sel_find_exactly_one(sel, ObjectType::TEXT)) {
+            const auto txt = core->get_text(it->uuid);
+            s += "\n\n<b>Text:</b>\n";
+            if (txt->overridden) {
+                s += "Overriden: " + txt->text_override;
+            }
+            else {
+                s += "Not overridden";
             }
         }
     }
 
-    if (preferences.hud_debug && (sel_count_type(sel, ObjectType::TEXT) == 1)) {
-        const auto txt = core->get_text(sel_find_one(sel, ObjectType::TEXT).uuid);
-        s += "\n\n<b>Text:</b>\n";
-        if (txt->overridden) {
-            s += "Overriden: " + txt->text_override;
-        }
-        else {
-            s += "Not overridden";
-        }
-    }
-
-    if (sel_count_type(sel, ObjectType::TEXT) == 1) {
-        const auto text = core->get_text(sel_find_one(sel, ObjectType::TEXT).uuid);
+    if (auto it = sel_find_exactly_one(sel, ObjectType::TEXT)) {
+        const auto text = core->get_text(it->uuid);
         const auto txt = Glib::ustring(text->text);
         auto regex = Glib::Regex::create(R"((https?:\/\/|file:\/\/\/?)([\w\.-]+)(\/\S+)?)");
         Glib::MatchInfo ma;
@@ -123,18 +115,52 @@ std::string ImpBase::get_hud_text(std::set<SelectableRef> &sel)
         }
     }
 
-    if (preferences.hud_debug && (sel_count_type(sel, ObjectType::JUNCTION) == 1)) {
-        const auto ju = core->get_junction(sel_find_one(sel, ObjectType::JUNCTION).uuid);
-        s += "\n\n<b>Junction:</b>\n";
-        s += "Layers " + std::to_string(ju->layer.start()) + " — " + std::to_string(ju->layer.end()) + "\n";
-        const Net *net = nullptr;
-        if (auto ju_b = dynamic_cast<BoardJunction *>(ju))
-            net = ju_b->net;
-        if (net)
-            s += "Net: " + core->get_block()->get_net_name(net->uuid);
-        else
-            s += "No net";
-        sel_erase_type(sel, ObjectType::JUNCTION);
+    if (preferences.hud_debug) {
+        if (auto it = sel_find_exactly_one(sel, ObjectType::JUNCTION)) {
+            const auto ju = core->get_junction(it->uuid);
+            s += "\n\n<b>Junction:</b>\n";
+            s += "Layers " + std::to_string(ju->layer.start()) + " — " + std::to_string(ju->layer.end()) + "\n";
+            const Net *net = nullptr;
+            if (auto ju_b = dynamic_cast<BoardJunction *>(ju))
+                net = ju_b->net;
+            if (net)
+                s += "Net: " + core->get_block()->get_net_name(net->uuid);
+            else
+                s += "No net";
+            sel_erase_type(sel, ObjectType::JUNCTION);
+        }
+    }
+
+    // Display the delta if two items of these types are selected
+    for (const ObjectType type : {ObjectType::HOLE, ObjectType::POLYGON_VERTEX, ObjectType::JUNCTION}) {
+        if (sel_count_type(sel, type) == 2) {
+            s += "\n\n<b>2 " + object_descriptions.at(type).name_pl + "</b>";
+            std::vector<Coordi> positions;
+            for (const auto &iter : sel) {
+                if (iter.type == type) {
+                    if (type == ObjectType::POLYGON_VERTEX) {
+                        const auto poly = core->get_polygon(iter.uuid);
+                        const auto vertex = &poly->vertices.at(iter.vertex);
+                        positions.push_back(vertex->position);
+                    }
+                    else if (type == ObjectType::HOLE) {
+                        const auto hole = core->get_hole(iter.uuid);
+                        positions.push_back(hole->placement.shift);
+                    }
+                    else if (type == ObjectType::JUNCTION) {
+                        const auto junction = core->get_junction(iter.uuid);
+                        positions.push_back(junction->position);
+                    }
+                    else {
+                        assert(false); // unreachable
+                    }
+                }
+            }
+            assert(positions.size() == 2);
+            const auto delta = positions.at(1) - positions.at(0);
+            s += "\n" + coord_to_string(delta, true);
+            sel_erase_type(sel, type);
+        }
     }
 
     trim(s);
@@ -156,7 +182,7 @@ std::string ImpBase::get_hud_text_for_net(const Net *net)
     if (!net)
         return "No net";
 
-    std::string s = "Net: " + net->name + "\n";
+    std::string s = "Net: " + core->get_block()->get_net_name(net->uuid) + "\n";
     s += "Net class " + net->net_class->name + "\n";
     if (net->is_power)
         s += "is power net";
