@@ -2,14 +2,50 @@
 #include "core/tool_id.hpp"
 #include "logger/logger.hpp"
 #include "in_tool_action_catalog.hpp"
+#include "actions.hpp"
+#include "util/str_util.hpp"
 
 namespace horizon {
 
+
+class KeyLabel : public Gtk::Box {
+public:
+    KeyLabel(Glib::RefPtr<Gtk::SizeGroup> sg, const std::string &key_markup, ActionToolID act)
+        : Gtk::Box(Gtk::ORIENTATION_HORIZONTAL, 8), action(act)
+    {
+        {
+            auto la = Gtk::manage(new Gtk::Label);
+            la->set_xalign(0);
+            la->set_markup(key_markup);
+            la->show();
+            pack_start(*la, false, false, 0);
+            sg->add_widget(*la);
+        }
+        {
+            auto la = Gtk::manage(new Gtk::Label);
+            la->set_xalign(0);
+            la->set_text(action_catalog.at(action).name);
+            la->show();
+            pack_start(*la, true, true, 0);
+        }
+        property_margin() = 2;
+    }
+
+    const ActionToolID action;
+};
+
 void ImpBase::init_key()
 {
     canvas->signal_key_press_event().connect(sigc::mem_fun(*this, &ImpBase::handle_key_press));
     key_sequence_dialog = std::make_unique<KeySequenceDialog>(this->main_window);
     connect_action(ActionID::HELP, [this](const auto &a) { key_sequence_dialog->show(); });
+    main_window->key_hint_box->signal_row_activated().connect([this](auto row) {
+        if (auto la = dynamic_cast<KeyLabel *>(row->get_child())) {
+            trigger_action(la->action);
+            keys_current.clear();
+            main_window->key_hint_set_visible(false);
+        }
+    });
 }
 
 bool ImpBase::handle_key_press(const GdkEventKey *key_event)
@@ -18,16 +54,9 @@ bool ImpBase::handle_key_press(const GdkEventKey *key_event)
     return false;
 }
 
-ImpBase::MatchResult ImpBase::keys_match(const KeySequence &keys) const
+KeyMatchResult ImpBase::keys_match(const KeySequence &keys) const
 {
-    const auto minl = std::min(keys_current.size(), keys.size());
-    const bool match = minl && std::equal(keys_current.begin(), keys_current.begin() + minl, keys.begin());
-    if (!match)
-        return MatchResult::NONE;
-    else if (keys_current.size() == keys.size())
-        return MatchResult::COMPLETE;
-    else
-        return MatchResult::PREFIX;
+    return key_sequence_match(keys_current, keys);
 }
 
 class KeyConflictDialog : public Gtk::MessageDialog {
@@ -96,6 +125,16 @@ bool ImpBase::handle_action_key(const GdkEventKey *ev)
             canvas->set_selection_mode(CanvasGL::SelectionMode::HOVER);
             canvas->set_selection({});
             set_search_mode(false);
+
+            reset_tool_hint_label();
+            if (keys_current.size() == 0) {
+                return false;
+            }
+            else {
+                main_window->key_hint_set_visible(false);
+                keys_current.clear();
+                return true;
+            }
         }
         else {
             ToolArgs args;
@@ -107,15 +146,6 @@ bool ImpBase::handle_action_key(const GdkEventKey *ev)
             tool_process(r);
             return true;
         }
-
-        if (keys_current.size() == 0) {
-            return false;
-        }
-        else {
-            keys_current.clear();
-            main_window->tool_hint_label->set_text(key_sequence_to_string(keys_current));
-            return true;
-        }
     }
     else {
         auto display = main_window->get_display()->gobj();
@@ -131,8 +161,8 @@ bool ImpBase::handle_action_key(const GdkEventKey *ev)
             keys_current.emplace_back(keyval, mod);
         }
         auto in_tool_actions = core->get_tool_actions();
-        std::map<InToolActionID, std::pair<MatchResult, KeySequence>> in_tool_actions_matched;
-        std::map<ActionConnection *, std::pair<MatchResult, KeySequence>> connections_matched;
+        std::map<InToolActionID, std::pair<KeyMatchResult, KeySequence>> in_tool_actions_matched;
+        std::map<ActionConnection *, std::pair<KeyMatchResult, KeySequence>> connections_matched;
         auto selection = canvas->get_selection();
         update_action_sensitivity();
         for (auto &it : action_connections) {
@@ -147,7 +177,7 @@ bool ImpBase::handle_action_key(const GdkEventKey *ev)
                 }
                 if (can_begin) {
                     for (const auto &it2 : it.second.key_sequences) {
-                        if (const auto m = keys_match(it2); m != MatchResult::NONE) {
+                        if (const auto m = keys_match(it2); m != KeyMatchResult::NONE) {
                             connections_matched.emplace(std::piecewise_construct, std::forward_as_tuple(&it.second),
                                                         std::forward_as_tuple(m, it2));
                         }
@@ -159,7 +189,7 @@ bool ImpBase::handle_action_key(const GdkEventKey *ev)
             for (const auto &[action, seqs] : in_tool_key_sequeces_preferences.keys) {
                 if (in_tool_actions.count(action)) {
                     for (const auto &seq : seqs) {
-                        if (const auto m = keys_match(seq); m != MatchResult::NONE)
+                        if (const auto m = keys_match(seq); m != KeyMatchResult::NONE)
                             in_tool_actions_matched.emplace(std::piecewise_construct, std::forward_as_tuple(action),
                                                             std::forward_as_tuple(m, seq));
                     }
@@ -174,9 +204,10 @@ bool ImpBase::handle_action_key(const GdkEventKey *ev)
         else if (connections_matched.size() == 1) {
             main_window->tool_hint_label->set_text(key_sequence_to_string(keys_current));
             keys_current.clear();
+            main_window->key_hint_set_visible(false);
             auto conn = connections_matched.begin()->first;
             if (!trigger_action({conn->action_id, conn->tool_id})) {
-                main_window->tool_hint_label->set_text(">");
+                reset_tool_hint_label();
                 return false;
             }
             return true;
@@ -200,14 +231,14 @@ bool ImpBase::handle_action_key(const GdkEventKey *ev)
             bool have_conflict = false;
             for (const auto &[conn, it] : connections_matched) {
                 const auto &[res, seq] = it;
-                if (res == MatchResult::COMPLETE) {
+                if (res == KeyMatchResult::COMPLETE) {
                     have_conflict = true;
                 }
                 conflicts.emplace_back(action_catalog.at(std::make_pair(conn->action_id, conn->tool_id)).name, seq);
             }
             for (const auto &[act, it] : in_tool_actions_matched) {
                 const auto &[res, seq] = it;
-                if (res == MatchResult::COMPLETE) {
+                if (res == KeyMatchResult::COMPLETE) {
                     have_conflict = true;
                 }
                 conflicts.emplace_back(in_tool_action_catalog.at(act).name + " (in-tool action)", seq);
@@ -222,12 +253,36 @@ bool ImpBase::handle_action_key(const GdkEventKey *ev)
                 return false;
             }
 
+            for (auto ch : main_window->key_hint_box->get_children()) {
+                delete ch;
+            }
+
+            for (const auto &[conn, it] : connections_matched) {
+                const auto &[res, seq] = it;
+                std::string seq_label;
+                for (size_t i = 0; i < seq.size(); i++) {
+                    seq_label += Glib::Markup::escape_text(key_sequence_item_to_string(seq.at(i))) + " ";
+                    if (i + 1 == keys_current.size()) {
+                        seq_label += "<b>";
+                    }
+                }
+                rtrim(seq_label);
+                seq_label += "</b>";
+
+                auto la = Gtk::manage(
+                        new KeyLabel(main_window->key_hint_size_group, seq_label, {conn->action_id, conn->tool_id}));
+                main_window->key_hint_box->append(*la);
+                la->show();
+            }
+            main_window->key_hint_set_visible(true);
+
             main_window->tool_hint_label->set_text(key_sequence_to_string(keys_current) + "?");
             return true;
         }
         else if (connections_matched.size() == 0 || in_tool_actions_matched.size() == 0) {
             main_window->tool_hint_label->set_text("Unknown key sequence");
             keys_current.clear();
+            main_window->key_hint_set_visible(false);
             return false;
         }
         else {
