@@ -1,6 +1,3 @@
-#include "canvas/canvas.hpp"
-#include "common/common.hpp"
-#include "common/lut.hpp"
 #include "imp.hpp"
 #include "imp_board.hpp"
 #include "imp_package.hpp"
@@ -17,9 +14,9 @@
 #include "util/uuid.hpp"
 #include "pool/pool_manager.hpp"
 #include "util/exception_util.hpp"
+#include "util/automatic_prefs.hpp"
 #include <curl/curl.h>
 #include <fstream>
-#include <gtkmm.h>
 #include <iostream>
 #include <memory>
 #ifdef G_OS_WIN32
@@ -42,6 +39,7 @@ int main(int argc, char *argv[])
     gtk_disable_setlocale();
     Gio::init();
     horizon::PoolManager::init();
+    horizon::AutomaticPreferences::get();
 
     Glib::OptionContext options;
     options.set_summary("horizon interactive manipulator");
@@ -116,15 +114,13 @@ int main(int argc, char *argv[])
     options.parse(argc, argv);
 
     auto pool_base_path = Glib::getenv("HORIZON_POOL");
-    auto pool_cache_path = Glib::getenv("HORIZON_POOL_CACHE");
     horizon::setup_locale();
 
     horizon::create_config_dir();
 
     std::unique_ptr<horizon::ImpBase> imp = nullptr;
     if (mode_sch) {
-        imp.reset(new horizon::ImpSchematic(filenames.at(0), filenames.at(1), filenames.at(2),
-                                            {pool_base_path, pool_cache_path}));
+        imp.reset(new horizon::ImpSchematic(filenames.at(0), filenames.at(1), filenames.at(2), {pool_base_path}));
     }
     else if (mode_symbol) {
         imp.reset(new horizon::ImpSymbol(filenames.at(0), pool_base_path));
@@ -136,8 +132,7 @@ int main(int argc, char *argv[])
         imp.reset(new horizon::ImpPackage(filenames.at(0), pool_base_path));
     }
     else if (mode_board) {
-        imp.reset(new horizon::ImpBoard(filenames.at(0), filenames.at(1), filenames.at(2), filenames.at(3),
-                                        {pool_base_path, pool_cache_path}));
+        imp.reset(new horizon::ImpBoard(filenames.at(0), filenames.at(1), filenames.at(2), {pool_base_path}));
     }
     else if (mode_frame) {
         imp.reset(new horizon::ImpFrame(filenames.at(0), pool_base_path));
