@@ -16,12 +16,13 @@
 #include "core/tools/tool_map_symbol.hpp"
 #include "widgets/unplaced_box.hpp"
 #include "core/tool_id.hpp"
+#include "actions.hpp"
 #include "widgets/action_button.hpp"
 
 namespace horizon {
 ImpSchematic::ImpSchematic(const std::string &schematic_filename, const std::string &block_filename,
                            const std::string &pictures_dir, const PoolParams &pool_params)
-    : ImpBase(pool_params), core_schematic(schematic_filename, block_filename, pictures_dir, *pool),
+    : ImpBase(pool_params), core_schematic(schematic_filename, block_filename, pictures_dir, *pool, *pool_caching),
       project_dir(Glib::path_get_dirname(schematic_filename)), searcher(core_schematic)
 {
     core = &core_schematic;
@@ -67,6 +68,19 @@ void ImpSchematic::handle_remove_sheet(Sheet *sh)
     canvas_update();
 }
 
+void ImpSchematic::handle_next_prev_sheet(const ActionConnection &conn)
+{
+    const auto &sch = *core_schematic.get_schematic();
+    const int sheet_idx_current = core_schematic.get_sheet()->index;
+    const int inc = conn.action_id == ActionID::PREV_SHEET ? -1 : 1;
+    const int sheet_idx_next = sheet_idx_current + inc;
+    auto next_sheet = std::find_if(sch.sheets.begin(), sch.sheets.end(),
+                                   [sheet_idx_next](const auto &x) { return (int)x.second.index == sheet_idx_next; });
+    if (next_sheet != sch.sheets.end()) {
+        sheet_box->select_sheet(next_sheet->second.uuid);
+    }
+}
+
 void ImpSchematic::update_highlights()
 {
     std::map<UUID, std::set<ObjectRef>> highlights_for_sheet;
@@ -194,8 +208,6 @@ bool ImpSchematic::handle_broadcast(const json &j)
 
 void ImpSchematic::handle_selection_cross_probe()
 {
-    if (core_schematic.tool_is_active())
-        return;
     json j;
     j["op"] = "schematic-select";
     j["selection"] = nullptr;
@@ -294,7 +306,6 @@ void ImpSchematic::construct()
     main_window->add_action("export_pdf", [this] { trigger_action(ActionID::PDF_EXPORT_WINDOW); });
 
     if (sockets_connected) {
-        canvas->signal_selection_changed().connect(sigc::mem_fun(*this, &ImpSchematic::handle_selection_cross_probe));
         hamburger_menu->append("Cross probing", "win.cross_probing");
         auto cp_action = main_window->add_action_bool("cross_probing", true);
         cross_probing_enabled = true;
@@ -323,6 +334,9 @@ void ImpSchematic::construct()
         }
     });
 
+    connect_action(ActionID::PREV_SHEET, sigc::mem_fun(*this, &ImpSchematic::handle_next_prev_sheet));
+    connect_action(ActionID::NEXT_SHEET, sigc::mem_fun(*this, &ImpSchematic::handle_next_prev_sheet));
+
     if (sockets_connected) {
         connect_action(ActionID::TO_BOARD, [this](const auto &conn) {
             json j;
@@ -633,15 +647,15 @@ UUID ImpSchematic::net_from_selectable(const SelectableRef &sr)
 std::string ImpSchematic::get_hud_text(std::set<SelectableRef> &sel)
 {
     std::string s;
-    if (sel_count_type(sel, ObjectType::SCHEMATIC_SYMBOL) == 1) {
-        const auto &sym = core_schematic.get_sheet()->symbols.at(sel_find_one(sel, ObjectType::SCHEMATIC_SYMBOL).uuid);
+    if (auto it = sel_find_exactly_one(sel, ObjectType::SCHEMATIC_SYMBOL)) {
+        const auto &sym = core_schematic.get_sheet()->symbols.at(it->uuid);
         s += "<b>Symbol " + sym.component->refdes + "</b>\n";
         s += get_hud_text_for_component(sym.component);
         sel_erase_type(sel, ObjectType::SCHEMATIC_SYMBOL);
     }
 
-    if (sel_count_type(sel, ObjectType::TEXT) == 1) {
-        const auto text = core->get_text(sel_find_one(sel, ObjectType::TEXT).uuid);
+    if (auto it = sel_find_exactly_one(sel, ObjectType::TEXT)) {
+        const auto text = core->get_text(it->uuid);
         const auto txt = Glib::ustring(text->text);
         Glib::MatchInfo ma;
         if (core_schematic.get_schematic()->get_sheetref_regex()->match(txt, ma)) {
@@ -697,10 +711,10 @@ void ImpSchematic::handle_highlight_group_tag(const ActionConnection &a)
     }
 }
 
-void ImpSchematic::handle_maybe_drag()
+void ImpSchematic::handle_maybe_drag(bool ctrl)
 {
     if (!preferences.schematic.drag_start_net_line) {
-        ImpBase::handle_maybe_drag();
+        ImpBase::handle_maybe_drag(ctrl);
         return;
     }
 
@@ -712,7 +726,7 @@ void ImpSchematic::handle_maybe_drag()
         cursor_pos_drag_begin = canvas->get_cursor_pos_win();
     }
     else {
-        ImpBase::handle_maybe_drag();
+        ImpBase::handle_maybe_drag(ctrl);
     }
 }
 
@@ -1078,6 +1092,26 @@ ToolID ImpSchematic::get_tool_for_drag_move(bool ctrl, const std::set<Selectable
     return ImpBase::get_tool_for_drag_move(ctrl, sel);
 }
 
+void ImpSchematic::handle_extra_button(const GdkEventButton *button_event)
+{
+    if (!preferences.mouse.switch_sheets)
+        return;
+
+    switch (button_event->button) {
+    case 6:
+    case 8:
+        trigger_action(ActionID::NEXT_SHEET);
+        break;
+
+    case 7:
+    case 9:
+        trigger_action(ActionID::PREV_SHEET);
+        break;
+
+    default:;
+    }
+}
+
 void ImpSchematic::expand_selection_for_property_panel(std::set<SelectableRef> &sel_extra,
                                                        const std::set<SelectableRef> &sel)
 {
