@@ -40,7 +40,7 @@ private:
     class UnplacedBox *unplaced_box = nullptr;
     void update_unplaced();
     void update_header();
-    void handle_selection_cross_probe();
+    void handle_selection_cross_probe() override;
 
     class CanvasAnnotation *bbox_annotation = nullptr;
     void update_bbox_annotation();
