@@ -8,8 +8,6 @@ namespace horizon {
 MainWindow::MainWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder> &x)
     : Gtk::ApplicationWindow(cobject), builder(x)
 {
-
-    GET_WIDGET(gl_container);
     GET_WIDGET(tool_hint_label);
     GET_WIDGET(left_panel);
     GET_WIDGET(grid_box_square);
@@ -55,6 +53,7 @@ MainWindow::MainWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder>
     GET_WIDGET(grid_square_button);
     GET_WIDGET(grid_rect_button);
     GET_WIDGET(grid_grid);
+    GET_WIDGET(grid_window_button);
     GET_WIDGET(grid_reset_origin_button);
     GET_WIDGET(tool_bar_action_tip_label);
     GET_WIDGET(tool_bar_actions_box);
@@ -62,6 +61,10 @@ MainWindow::MainWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder>
     GET_WIDGET(version_info_bar);
     GET_WIDGET(version_label);
 
+    GET_WIDGET(key_hint_box);
+    GET_WIDGET(key_hint_revealer);
+    key_hint_size_group = Gtk::SizeGroup::create(Gtk::SIZE_GROUP_HORIZONTAL);
+
     set_version_info("");
 
     grid_options_button->signal_clicked().connect([this] {
@@ -99,14 +102,20 @@ MainWindow::MainWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder>
     label_set_tnum(search_status_label);
     label_set_tnum(view_hints_label);
     {
-        auto attributes_list = gtk_label_get_attributes(tool_bar_action_tip_label->gobj());
+        auto attributes_list = pango_attr_list_new();
         auto attribute_font_features = pango_attr_font_features_new("tnum 1");
         pango_attr_list_insert(attributes_list, attribute_font_features);
+        auto attribute_italic = pango_attr_style_new(PANGO_STYLE_ITALIC);
+        pango_attr_list_insert(attributes_list, attribute_italic);
+        gtk_label_set_attributes(tool_bar_action_tip_label->gobj(), attributes_list);
+        pango_attr_list_unref(attributes_list);
     }
 
+    GET_WIDGET(gl_container);
     canvas = Gtk::manage(new CanvasGL());
-    gl_container->pack_start(*canvas, true, true, 0);
+    gl_container->add(*canvas);
     canvas->show();
+
     tool_bar_set_visible(false);
     hud->set_reveal_child(false);
     set_use_action_bar(false);
@@ -263,6 +272,33 @@ void MainWindow::set_version_info(const std::string &s)
     }
 }
 
+void MainWindow::update_key_hint_position()
+{
+    int dest_x, dest_y;
+    if (tool_hint_label->translate_coordinates(*gl_container, 0, 0, dest_x, dest_y)) {
+        key_hint_revealer->set_margin_start(dest_x - 5); // compensate for various borders and margins
+    }
+}
+
+void MainWindow::key_hint_set_visible(bool show)
+{
+    key_hint_connection.disconnect();
+    if (show) {
+        if (!key_hint_revealer->get_reveal_child()) {
+            update_key_hint_position();
+            key_hint_connection = Glib::signal_timeout().connect(
+                    [this] {
+                        key_hint_revealer->set_reveal_child(true);
+                        return false;
+                    },
+                    500);
+        }
+    }
+    else {
+        key_hint_revealer->set_reveal_child(false);
+    }
+}
+
 MainWindow *MainWindow::create()
 {
     MainWindow *w;
