@@ -73,6 +73,12 @@ std::string Logger::domain_to_string(Logger::Domain dom)
         return "Import";
     case Logger::Domain::VERSION:
         return "Version";
+    case Logger::Domain::POOL_UPDATE:
+        return "Pool update";
+    case Logger::Domain::PICTURE:
+        return "Picture";
+    case Logger::Domain::PART:
+        return "Part";
     default:
         return "Unspecified";
     }
