@@ -2,6 +2,7 @@
 #include <gtkmm.h>
 #include <mutex>
 #include <list>
+#include <thread>
 
 namespace horizon {
 enum class PoolUpdateStatus;
@@ -9,6 +10,7 @@ enum class PoolUpdateStatus;
 class ForcedPoolUpdateDialog : public Gtk::Dialog {
 public:
     ForcedPoolUpdateDialog(const std::string &bp, Gtk::Window *parent);
+    ~ForcedPoolUpdateDialog();
 
 private:
     std::string base_path;
@@ -19,5 +21,6 @@ private:
     Gtk::Spinner *spinner = nullptr;
     void pool_update_thread();
     std::string pool_update_last_info;
+    std::thread thread;
 };
 } // namespace horizon
