@@ -10,6 +10,8 @@
 #include "pool-prj-mgr/pool-prj-mgr-app_win.hpp"
 #include "preferences/preferences_provider.hpp"
 #include "preferences/preferences.hpp"
+#include "check_column.hpp"
+#include "checks/check_item.hpp"
 #include <iomanip>
 
 namespace horizon {
@@ -34,7 +36,12 @@ void PoolGitBox::make_treeview(Gtk::TreeView *treeview)
         tvc->set_min_width(200);
         tvc->set_resizable(true);
     }
-    tree_view_append_column_ellipsis(treeview, "Path", list_columns.path, Pango::ELLIPSIZE_START);
+    {
+        auto tvc = tree_view_append_column_ellipsis(treeview, "Path", list_columns.path, Pango::ELLIPSIZE_START);
+        tvc->set_min_width(200);
+        tvc->set_resizable(true);
+        tvc->set_expand(true);
+    }
 
     treeview->signal_row_activated().connect([this, treeview](const Gtk::TreeModel::Path &path,
                                                               Gtk::TreeViewColumn *column) {
@@ -42,7 +49,7 @@ void PoolGitBox::make_treeview(Gtk::TreeView *treeview)
         if (it) {
             Gtk::TreeModel::Row row = *it;
             if ((row[list_columns.type] != ObjectType::INVALID) && (row[list_columns.type] != ObjectType::MODEL_3D)) {
-                notebook->go_to(row[list_columns.type], row[list_columns.uuid]);
+                notebook.go_to(row[list_columns.type], row[list_columns.uuid]);
             }
         }
     });
@@ -63,7 +70,7 @@ static const std::map<unsigned int, std::string> status_names = {{GIT_STATUS_CUR
                                                                  {GIT_STATUS_IGNORED, "Ignored"},
                                                                  {GIT_STATUS_CONFLICTED, "Conflicted"}};
 
-PoolGitBox::PoolGitBox(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder> &x, PoolNotebook *nb)
+PoolGitBox::PoolGitBox(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder> &x, PoolNotebook &nb)
     : Gtk::Box(cobject), notebook(nb)
 {
     x->get_widget("button_git_refresh", refresh_button);
@@ -150,6 +157,8 @@ PoolGitBox::PoolGitBox(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder>
     sort_controller_status->set_sort(0, SortController::Sort::ASC);
     sort_controller_status->signal_changed().connect(sigc::mem_fun(*this, &PoolGitBox::update_status));
 
+    add_check_column(*status_treeview, list_columns.check_result);
+
 
     x->get_widget("button_git_pr", pr_button);
     x->get_widget("button_git_back_to_master", back_to_master_button);
@@ -164,15 +173,15 @@ PoolGitBox::PoolGitBox(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder>
         back_to_master_delete_button->hide();
     }
 
-    notebook->pool.db.execute(
+    notebook.pool.db.execute(
             "CREATE TEMP TABLE 'git_files_status' ('git_filename' TEXT NOT NULL, status INTEGER NOT NULL);");
-    notebook->pool.db.execute(
+    notebook.pool.db.execute(
             "CREATE TEMP TABLE 'git_files_diff' ('git_filename' TEXT NOT NULL, status INTEGER NOT NULL);");
-    notebook->pool.db.execute("CREATE INDEX git_file_status ON git_files_status (git_filename)");
-    notebook->pool.db.execute("CREATE INDEX git_file_diff ON git_files_diff (git_filename)");
+    notebook.pool.db.execute("CREATE INDEX git_file_status ON git_files_status (git_filename)");
+    notebook.pool.db.execute("CREATE INDEX git_file_diff ON git_files_diff (git_filename)");
 
-    q_diff.emplace(notebook->pool.db, "INSERT INTO 'git_files_diff' VALUES (?, ?)");
-    q_status.emplace(notebook->pool.db, "INSERT INTO 'git_files_status' VALUES (?, ?)");
+    q_diff.emplace(notebook.pool.db, "INSERT INTO 'git_files_diff' VALUES (?, ?)");
+    q_status.emplace(notebook.pool.db, "INSERT INTO 'git_files_status' VALUES (?, ?)");
 }
 
 void PoolGitBox::diff_file_cb(const git_diff_delta *delta)
@@ -211,7 +220,7 @@ void PoolGitBox::refresh()
 {
     try {
         autofree_ptr<git_repository> repo(git_repository_free);
-        if (git_repository_open(&repo.ptr, notebook->base_path.c_str()) != 0) {
+        if (git_repository_open(&repo.ptr, notebook.base_path.c_str()) != 0) {
             throw std::runtime_error("error opening repo");
         }
         autofree_ptr<git_reference> head(git_reference_free);
@@ -254,19 +263,19 @@ void PoolGitBox::refresh()
         {
             autofree_ptr<git_diff> diff(git_diff_free);
             git_diff_tree_to_workdir_with_index(&diff.ptr, repo, tree_master, nullptr);
-            notebook->pool.db.execute("BEGIN");
-            notebook->pool.db.execute("DELETE FROM git_files_diff");
+            notebook.pool.db.execute("BEGIN");
+            notebook.pool.db.execute("DELETE FROM git_files_diff");
             git_diff_foreach(diff, &PoolGitBox::diff_file_cb_c, nullptr, nullptr, nullptr, this);
-            notebook->pool.db.execute("COMMIT");
+            notebook.pool.db.execute("COMMIT");
             diff_box->set_visible(tree_master != tree_head);
             update_diff();
         }
 
         {
-            notebook->pool.db.execute("BEGIN");
-            notebook->pool.db.execute("DELETE FROM git_files_status");
+            notebook.pool.db.execute("BEGIN");
+            notebook.pool.db.execute("DELETE FROM git_files_status");
             git_status_foreach(repo, &PoolGitBox::status_cb_c, this);
-            notebook->pool.db.execute("COMMIT");
+            notebook.pool.db.execute("COMMIT");
             update_status();
         }
 
@@ -299,7 +308,7 @@ void PoolGitBox::store_from_db(View view, const std::string &extra_q)
            "(SELECT type, uuid, name, filename FROM all_items_view UNION ALL SELECT DISTINCT 'model_3d' AS type, "
            "'00000000-0000-0000-0000-000000000000' as uuid, '' as name, model_filename as filename FROM models) "
            "ON filename=git_filename " + extra_q  + " " + order;
-    SQLite::Query q(notebook->pool.db, qs);
+    SQLite::Query q(notebook.pool.db, qs);
     while (q.step()) {
         Gtk::TreeModel::Row row = *store->append();
         const auto stype = q.get<std::string>(0);
@@ -312,6 +321,10 @@ void PoolGitBox::store_from_db(View view, const std::string &extra_q)
             row[list_columns.type] = type;
             row[list_columns.uuid] = uu;
             row[list_columns.name] = name;
+            if (view == View::STATUS) {
+                auto result = check_item(notebook.pool, type, uu);
+                row[list_columns.check_result] = result;
+            }
         }
         else {
             row[list_columns.type] = ObjectType::INVALID;
@@ -348,10 +361,17 @@ void PoolGitBox::update_diff()
 
 void PoolGitBox::update_status()
 {
+    clock_t begin = clock();
+
     status_treeview->unset_model();
     status_store->clear();
     store_from_db(View::STATUS);
     status_treeview->set_model(status_store);
+
+
+    clock_t end = clock();
+    double elapsed_secs = double(end - begin) / CLOCKS_PER_SEC;
+    std::cout << "update took " << elapsed_secs << std::endl;
 }
 
 void PoolGitBox::handle_add_with_deps()
@@ -361,22 +381,22 @@ void PoolGitBox::handle_add_with_deps()
         Gtk::TreeModel::Row row = *it;
         UUID uu = row[list_columns.uuid];
         ObjectType type = row[list_columns.type];
-        SQLite::Query q(notebook->pool.db,
+        SQLite::Query q(notebook.pool.db,
                         "WITH RECURSIVE deps(typex, uuidx) AS "
                         "( SELECT ?, ? UNION "
                         "SELECT dep_type, dep_uuid FROM dependencies, deps "
                         "WHERE dependencies.type = deps.typex AND dependencies.uuid = deps.uuidx) "
                         ", deps_sym(typey, uuidy) AS (SELECT * FROM deps UNION SELECT 'symbol', symbols.uuid FROM "
                         "symbols INNER JOIN deps ON (symbols.unit = uuidx AND typex = 'unit')) "
-                        "SELECT filename FROM deps_sym INNER JOIN all_items_view "
+                        "SELECT filename FROM deps_sym LEFT JOIN all_items_view "
                         "ON(all_items_view.type =deps_sym.typey AND all_items_view.uuid = deps_sym.uuidy) "
                         "UNION SELECT model_filename FROM models INNER JOIN deps "
                         "ON (deps.typex = 'package' AND deps.uuidx = models.package_uuid)");
-        q.bind(1, object_type_lut.lookup_reverse(type));
+        q.bind(1, type);
         q.bind(2, uu);
 
         autofree_ptr<git_repository> repo(git_repository_free);
-        if (git_repository_open(&repo.ptr, notebook->base_path.c_str()) != 0) {
+        if (git_repository_open(&repo.ptr, notebook.base_path.c_str()) != 0) {
             throw std::runtime_error("error opening repo");
         }
 
@@ -387,6 +407,11 @@ void PoolGitBox::handle_add_with_deps()
 
         while (q.step()) {
             std::string filename = q.get<std::string>(0);
+            int ignored = 0;
+            if (git_ignore_path_is_ignored(&ignored, repo, filename.c_str()) == 0) {
+                if (ignored)
+                    continue;
+            }
             if (git_index_add_bypath(index, filename.c_str()) != 0) {
                 auto last_error = giterr_last();
                 std::string err_str = last_error->message;
@@ -424,6 +449,7 @@ SelectPRDialog::SelectPRDialog(Gtk::Window &parent)
 
     int top = 0;
     pr_id_sp = Gtk::manage(new Gtk::SpinButton);
+    pr_id_sp->set_increments(1, 1);
     pr_id_sp->set_range(0, 10000);
     pr_id_sp->signal_activate().connect([this] { response(Gtk::RESPONSE_OK); });
     grid_attach_label_and_widget(grid, "PR #", pr_id_sp, top);
@@ -437,7 +463,7 @@ void PoolGitBox::handle_pr()
 {
     unsigned int pr_id = 0;
     {
-        SelectPRDialog dia(*notebook->appwin);
+        SelectPRDialog dia(notebook.appwin);
         if (dia.run() == Gtk::RESPONSE_OK) {
             pr_id = dia.pr_id_sp->get_value_as_int();
         }
@@ -447,7 +473,7 @@ void PoolGitBox::handle_pr()
     }
 
     autofree_ptr<git_repository> repo(git_repository_free);
-    if (git_repository_open(&repo.ptr, notebook->base_path.c_str()) != 0) {
+    if (git_repository_open(&repo.ptr, notebook.base_path.c_str()) != 0) {
         throw std::runtime_error("error opening repo");
     }
 
@@ -457,7 +483,8 @@ void PoolGitBox::handle_pr()
     }
 
     Glib::ustring remote_url(git_remote_url(remote));
-    const auto regex_url = Glib::Regex::create(R"(^git@github\.com:([\w-]+)\/([\w-]+).git$)");
+    const auto regex_url =
+            Glib::Regex::create(R"(^(?:git@github\.com:|https://github.com/)([\w-]+)\/([\w-]+)(?:.git)?$)");
     Glib::MatchInfo ma;
     if (regex_url->match(remote_url, ma)) {
         auto gh_owner = ma.fetch(1);
@@ -618,7 +645,7 @@ void PoolGitBox::handle_pr()
             git_repository_state_cleanup(repo);
         }
         refresh();
-        notebook->pool_update();
+        notebook.pool_update();
     }
 }
 
@@ -638,7 +665,7 @@ static void remove_branch(git_repository *repo, const char *branch_name)
 void PoolGitBox::handle_back_to_master(bool delete_pr)
 {
     autofree_ptr<git_repository> repo(git_repository_free);
-    if (git_repository_open(&repo.ptr, notebook->base_path.c_str()) != 0) {
+    if (git_repository_open(&repo.ptr, notebook.base_path.c_str()) != 0) {
         throw std::runtime_error("error opening repo");
     }
 
@@ -691,11 +718,11 @@ void PoolGitBox::handle_back_to_master(bool delete_pr)
     }
 
     refresh();
-    notebook->pool_update();
+    notebook.pool_update();
 }
 
 
-PoolGitBox *PoolGitBox::create(PoolNotebook *nb)
+PoolGitBox *PoolGitBox::create(PoolNotebook &nb)
 {
     PoolGitBox *w;
     Glib::RefPtr<Gtk::Builder> x = Gtk::Builder::create();
