@@ -1,6 +1,4 @@
 #include "pool_notebook.hpp"
-#include "editors/editor_window.hpp"
-#include "duplicate/duplicate_unit.hpp"
 #include "duplicate/duplicate_part.hpp"
 #include "util/util.hpp"
 #include "nlohmann/json.hpp"
@@ -10,15 +8,12 @@
 #include "widgets/package_info_box.hpp"
 #include "util/win32_undef.hpp"
 #include "widgets/preview_canvas.hpp"
+#include "import_kicad_package_window.hpp"
 
 namespace horizon {
 void PoolNotebook::handle_edit_package(const UUID &uu)
 {
-    if (!uu)
-        return;
-    UUID item_pool_uuid;
-    auto path = pool.get_filename(ObjectType::PACKAGE, uu, &item_pool_uuid);
-    appwin->spawn(PoolProjectManagerProcess::Type::IMP_PACKAGE, {path}, {}, pool_uuid && (item_pool_uuid != pool_uuid));
+    handle_edit_item(ObjectType::PACKAGE, uu);
 }
 
 void PoolNotebook::handle_create_package()
@@ -56,7 +51,7 @@ void PoolNotebook::handle_create_package()
                 Package pkg(horizon::UUID::random());
                 auto pkg_filename = Glib::build_filename(fn, "package.json");
                 save_json_to_file(pkg_filename, pkg.serialize());
-                appwin->spawn(PoolProjectManagerProcess::Type::IMP_PACKAGE, {pkg_filename});
+                appwin.spawn(PoolProjectManagerProcess::Type::IMP_PACKAGE, {pkg_filename});
             }
         }
         break;
@@ -79,11 +74,11 @@ void PoolNotebook::handle_create_padstack_for_package(const UUID &uu)
     chooser->set_current_folder(Glib::build_filename(pkg_dir, "padstacks"));
 
     if (gtk_native_dialog_run(GTK_NATIVE_DIALOG(native)) == GTK_RESPONSE_ACCEPT) {
-        std::string fn = EditorWindow::fix_filename(chooser->get_filename());
+        std::string fn = append_dot_json(chooser->get_filename());
         Padstack ps(horizon::UUID::random());
         ps.name = "Pad";
         save_json_to_file(fn, ps.serialize());
-        appwin->spawn(PoolProjectManagerProcess::Type::IMP_PADSTACK, {fn});
+        appwin.spawn(PoolProjectManagerProcess::Type::IMP_PADSTACK, {fn});
     }
 }
 
@@ -97,7 +92,8 @@ void PoolNotebook::handle_duplicate_package(const UUID &uu)
             "Save Package", top->gobj(), GTK_FILE_CHOOSER_ACTION_CREATE_FOLDER, "_Save", "_Cancel");
     auto chooser = Glib::wrap(GTK_FILE_CHOOSER(native));
     chooser->set_do_overwrite_confirmation(true);
-    auto pkg_filename = Glib::path_get_dirname(pool.get_filename(ObjectType::PACKAGE, uu));
+    auto pkg_filename = Glib::build_filename(pool.get_base_path(),
+                                             Glib::path_get_dirname(pool.get_rel_filename(ObjectType::PACKAGE, uu)));
     auto pkg_basename = Glib::path_get_basename(pkg_filename);
     auto pkg_dirname = Glib::path_get_dirname(pkg_filename);
     chooser->set_current_folder(pkg_dirname);
@@ -124,20 +120,40 @@ void PoolNotebook::handle_duplicate_package(const UUID &uu)
                 std::vector<std::string> filenames;
                 DuplicatePartWidget::duplicate_package(pool, uu, fn, Glib::path_get_basename(fn), &filenames);
                 std::string new_pkg_filename = Glib::build_filename(fn, "package.json");
-                pool_update(
-                        [this, new_pkg_filename] {
-                            appwin->spawn(PoolProjectManagerProcess::Type::IMP_PACKAGE, {new_pkg_filename});
-                        },
-                        filenames);
+                pool_update_done_cb = [this, new_pkg_filename] {
+                    appwin.spawn(PoolProjectManagerProcess::Type::IMP_PACKAGE, {new_pkg_filename});
+                };
+                pool_update(filenames);
             }
         }
         break;
     }
 }
 
+void PoolNotebook::handle_import_kicad_package()
+{
+    if (!import_kicad_package_window) {
+
+        import_kicad_package_window = ImportKiCadPackageWindow::create(appwin);
+        import_kicad_package_window->present();
+        import_kicad_package_window->signal_hide().connect([this] {
+            const auto files_saved = import_kicad_package_window->get_files_saved();
+            if (files_saved.size()) {
+                pool_update(files_saved);
+            }
+            delete import_kicad_package_window;
+            import_kicad_package_window = nullptr;
+        });
+    }
+
+    else {
+        import_kicad_package_window->present();
+    }
+}
+
 void PoolNotebook::construct_packages()
 {
-    auto br = Gtk::manage(new PoolBrowserPackage(pool));
+    auto br = Gtk::manage(new PoolBrowserPackage(pool, false, "pool_notebook"));
     br->set_show_path(true);
     br->signal_activated().connect([this, br] { handle_edit_package(br->get_selected()); });
 
@@ -159,15 +175,17 @@ void PoolNotebook::construct_packages()
     add_action_button("Duplicate", bbox, br, sigc::mem_fun(*this, &PoolNotebook::handle_duplicate_package));
     add_action_button("Create Padstack", bbox, br,
                       sigc::mem_fun(*this, &PoolNotebook::handle_create_padstack_for_package));
-    if (remote_repo.size()) {
-        add_action_button("Merge", bbox, br,
-                          [this](const UUID &uu) { remote_box->merge_item(ObjectType::PACKAGE, uu); });
-        add_action_button("Merge 3D", bbox, br, [this](const UUID &uu) {
+    add_action_button("Import KiCad package", bbox, sigc::mem_fun(*this, &PoolNotebook::handle_import_kicad_package))
+            ->get_style_context()
+            ->add_class("suggested-action");
+    add_merge_button(bbox, br);
+    if (auto bu = add_merge_button(bbox, br, [this](const UUID &uu) {
             auto pkg = pool.get_package(uu);
             for (const auto &it : pkg->models) {
                 remote_box->merge_3d_model(it.second.filename);
             }
-        });
+        })) {
+        bu->set_label("Merge 3D");
     }
 
     auto stack = Gtk::manage(new Gtk::Stack);
@@ -213,6 +231,7 @@ void PoolNotebook::construct_packages()
     });
     append_page(*paned, "Packages");
     install_search_once(paned, br);
+    create_paned_state_store(paned, "packages");
 }
 
 } // namespace horizon
