@@ -1,26 +1,20 @@
 #include "pool_notebook.hpp"
-#include "editors/editor_window.hpp"
 #include "util/util.hpp"
 #include "nlohmann/json.hpp"
 #include "pool-prj-mgr/pool-prj-mgr-app_win.hpp"
 #include "widgets/pool_browser_unit.hpp"
-#include "pool_remote_box.hpp"
 #include "widgets/unit_preview.hpp"
 #include "widgets/unit_info_box.hpp"
 
 namespace horizon {
 void PoolNotebook::handle_create_unit()
 {
-    appwin->spawn(PoolProjectManagerProcess::Type::UNIT, {""});
+    appwin.spawn(PoolProjectManagerProcess::Type::UNIT, {""});
 }
 
 void PoolNotebook::handle_edit_unit(const UUID &uu)
 {
-    if (!uu)
-        return;
-    UUID item_pool_uuid;
-    auto path = pool.get_filename(ObjectType::UNIT, uu, &item_pool_uuid);
-    appwin->spawn(PoolProjectManagerProcess::Type::UNIT, {path}, {}, pool_uuid && (item_pool_uuid != pool_uuid));
+    handle_edit_item(ObjectType::UNIT, uu);
 }
 
 void PoolNotebook::handle_create_symbol_for_unit(const UUID &uu)
@@ -44,14 +38,14 @@ void PoolNotebook::handle_create_symbol_for_unit(const UUID &uu)
     }
 
     if (gtk_native_dialog_run(GTK_NATIVE_DIALOG(native)) == GTK_RESPONSE_ACCEPT) {
-        std::string fn = EditorWindow::fix_filename(chooser->get_filename());
+        std::string fn = append_dot_json(chooser->get_filename());
         Symbol sym(horizon::UUID::random());
         auto unit = pool.get_unit(uu);
         sym.name = unit->name;
         sym.unit = unit;
         save_json_to_file(fn, sym.serialize());
-        pool_update(nullptr, {fn});
-        appwin->spawn(PoolProjectManagerProcess::Type::IMP_SYMBOL, {fn});
+        pool_update({fn});
+        appwin.spawn(PoolProjectManagerProcess::Type::IMP_SYMBOL, {fn});
     }
 }
 
@@ -76,7 +70,7 @@ void PoolNotebook::handle_create_entity_for_unit(const UUID &uu)
     }
 
     if (gtk_native_dialog_run(GTK_NATIVE_DIALOG(native)) == GTK_RESPONSE_ACCEPT) {
-        std::string fn = EditorWindow::fix_filename(chooser->get_filename());
+        std::string fn = append_dot_json(chooser->get_filename());
         Entity entity(horizon::UUID::random());
         auto unit = pool.get_unit(uu);
         entity.name = unit->name;
@@ -86,8 +80,8 @@ void PoolNotebook::handle_create_entity_for_unit(const UUID &uu)
         gate->name = "Main";
 
         save_json_to_file(fn, entity.serialize());
-        pool_update(nullptr, {fn});
-        appwin->spawn(PoolProjectManagerProcess::Type::ENTITY, {fn});
+        pool_update({fn});
+        appwin.spawn(PoolProjectManagerProcess::Type::ENTITY, {fn});
     }
 }
 
@@ -119,8 +113,7 @@ void PoolNotebook::construct_units()
     add_action_button("Duplicate", bbox, br, sigc::mem_fun(*this, &PoolNotebook::handle_duplicate_unit));
     add_action_button("Create Symbol", bbox, br, sigc::mem_fun(*this, &PoolNotebook::handle_create_symbol_for_unit));
     add_action_button("Create Entity", bbox, br, sigc::mem_fun(*this, &PoolNotebook::handle_create_entity_for_unit));
-    if (remote_repo.size())
-        add_action_button("Merge", bbox, br, [this](const UUID &uu) { remote_box->merge_item(ObjectType::UNIT, uu); });
+    add_merge_button(bbox, br);
 
     auto stack = Gtk::manage(new Gtk::Stack);
     add_preview_stack_switcher(bbox, stack);
@@ -167,6 +160,7 @@ void PoolNotebook::construct_units()
     append_page(*paned, "Units");
     br->search_once(); // we're the first page to be displayed
     install_search_once(paned, br);
+    create_paned_state_store(paned, "units");
 }
 
 } // namespace horizon
