@@ -4,7 +4,6 @@
 #include "util/autofree_ptr.hpp"
 #include "util/gtk_util.hpp"
 #include "pool_notebook.hpp"
-#include "pool_merge_dialog.hpp"
 #include <thread>
 #include "pool-update/pool-update.hpp"
 #include "common/object_descr.hpp"
@@ -14,6 +13,13 @@
 #include "github_login_window.hpp"
 #include <iomanip>
 #include "util/win32_undef.hpp"
+#include "check_column.hpp"
+#include "checks/check_item.hpp"
+#include "checks/check_util.hpp"
+#include "../pool-prj-mgr-app_win.hpp"
+#include "../pool-prj-mgr-app.hpp"
+#include "../pools_window/pools_window.hpp"
+#include "pool/pool_manager.hpp"
 
 namespace horizon {
 
@@ -23,6 +29,10 @@ public:
 
     Gtk::Entry *name_entry = nullptr;
     Gtk::Entry *email_entry = nullptr;
+
+private:
+    Gtk::Button *confirm_button;
+    void update_valid();
 };
 
 ConfirmPrDialog::ConfirmPrDialog(Gtk::Window *parent)
@@ -30,7 +40,7 @@ ConfirmPrDialog::ConfirmPrDialog(Gtk::Window *parent)
                   Gtk::DialogFlags::DIALOG_MODAL | Gtk::DialogFlags::DIALOG_USE_HEADER_BAR)
 {
     add_button("Cancel", Gtk::ResponseType::RESPONSE_CANCEL);
-    add_button("Confirm", Gtk::ResponseType::RESPONSE_OK);
+    confirm_button = add_button("Confirm", Gtk::ResponseType::RESPONSE_OK);
     set_default_response(Gtk::RESPONSE_OK);
     auto grid = Gtk::manage(new Gtk::Grid);
     grid->set_row_spacing(10);
@@ -46,6 +56,8 @@ ConfirmPrDialog::ConfirmPrDialog(Gtk::Window *parent)
 
     name_entry->signal_activate().connect([this] { email_entry->grab_focus(); });
     email_entry->signal_activate().connect([this] { response(Gtk::RESPONSE_OK); });
+    name_entry->signal_changed().connect(sigc::mem_fun(*this, &ConfirmPrDialog::update_valid));
+    email_entry->signal_changed().connect(sigc::mem_fun(*this, &ConfirmPrDialog::update_valid));
 
     auto box = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_VERTICAL, 20));
     box->property_margin() = 20;
@@ -67,14 +79,34 @@ ConfirmPrDialog::ConfirmPrDialog(Gtk::Window *parent)
 
     get_content_area()->add(*box);
     box->show_all();
+    update_valid();
+}
+
+void ConfirmPrDialog::update_valid()
+{
+    bool valid = true;
+    for (auto entry : {email_entry, name_entry}) {
+        std::string txt = entry->get_text();
+        if (needs_trim(txt)) {
+            entry_set_warning(entry, "Has trailing/leading whitespace");
+            valid = false;
+        }
+        else if (txt.size() == 0) {
+            entry_set_warning(entry, "Must not be blank");
+            valid = false;
+        }
+        else {
+            entry_set_warning(entry, "");
+        }
+    }
+    confirm_button->set_sensitive(valid);
 }
 
 static const std::string github_client_id = "094ac4cbd2e4a51820b4";
 
-PoolRemoteBox::PoolRemoteBox(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder> &x, PoolNotebook *nb)
+PoolRemoteBox::PoolRemoteBox(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder> &x, PoolNotebook &nb)
     : Gtk::Box(cobject), notebook(nb)
 {
-    x->get_widget("button_upgrade_pool", upgrade_button);
     x->get_widget("button_do_create_pr", create_pr_button);
     x->get_widget("upgrade_revealer", upgrade_revealer);
     x->get_widget("upgrade_spinner", upgrade_spinner);
@@ -86,6 +118,7 @@ PoolRemoteBox::PoolRemoteBox(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Bu
     x->get_widget("pr_body_placeholder_label", pr_body_placeholder_label);
     x->get_widget("merge_items_clear_button", merge_items_clear_button);
     x->get_widget("merge_items_remove_button", merge_items_remove_button);
+    x->get_widget("merge_items_run_checks_button", merge_items_run_checks_button);
     x->get_widget("button_refresh_pull_requests", refresh_prs_button);
     x->get_widget("remote_gh_signed_in_label", gh_signed_in_label);
     x->get_widget("remote_pr_title_entry", pr_title_entry);
@@ -143,7 +176,7 @@ PoolRemoteBox::PoolRemoteBox(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Bu
     show_only_my_prs_cb->signal_toggled().connect(sigc::mem_fun(*this, &PoolRemoteBox::update_prs));
 
     autofree_ptr<git_repository> repo(git_repository_free);
-    if (git_repository_open(&repo.ptr, notebook->remote_repo.c_str()) != 0) {
+    if (git_repository_open(&repo.ptr, notebook.remote_repo.c_str()) != 0) {
         throw std::runtime_error("error opening repo");
     }
 
@@ -166,16 +199,16 @@ PoolRemoteBox::PoolRemoteBox(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Bu
         gh_repo_link_label->set_text("couldn't find github repo!");
     }
 
-    upgrade_button->signal_clicked().connect(sigc::mem_fun(*this, &PoolRemoteBox::handle_remote_upgrade));
     create_pr_button->signal_clicked().connect(sigc::mem_fun(*this, &PoolRemoteBox::handle_create_pr));
     pr_update_button->signal_clicked().connect(sigc::mem_fun(*this, &PoolRemoteBox::handle_update_pr));
     refresh_prs_button->signal_clicked().connect(sigc::mem_fun(*this, &PoolRemoteBox::handle_refresh_prs));
 
     pr_status_dispatcher.attach(pr_spinner);
+    pr_status_dispatcher.attach(dynamic_cast<Gtk::Window *>(get_ancestor(GTK_TYPE_WINDOW)));
     pr_status_dispatcher.signal_notified().connect([this](const StatusDispatcher::Notification &n) {
         auto is_busy = n.status == StatusDispatcher::Status::BUSY;
         if (!is_busy) {
-            notebook->pool_updating = false;
+            notebook.pool_busy = false;
         }
         if (n.status == StatusDispatcher::Status::DONE) {
             update_prs();
@@ -197,16 +230,10 @@ PoolRemoteBox::PoolRemoteBox(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Bu
         logout_button->set_visible(gh_username.size() || login_succeeded == false);
         update_my_prs();
         if (!git_thread_busy) {
-            notebook->pool_updating = false;
+            notebook.pool_busy = false;
         }
-        if (git_thread_mode == GitThreadMode::UPGRADE && !git_thread_busy && !git_thread_error) {
-            auto top = dynamic_cast<Gtk::Window *>(get_ancestor(GTK_TYPE_WINDOW));
-            PoolMergeDialog dia(top, notebook->base_path, notebook->remote_repo);
-            dia.run();
-            if (dia.get_merged())
-                notebook->pool_update();
-        }
-        else if (git_thread_mode == GitThreadMode::PULL_REQUEST && !git_thread_busy && !git_thread_error) {
+
+        if (git_thread_mode == GitThreadMode::PULL_REQUEST && !git_thread_busy && !git_thread_error) {
             items_merge.clear();
             models_merge.clear();
             update_items_merge();
@@ -247,13 +274,15 @@ PoolRemoteBox::PoolRemoteBox(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Bu
                 Gtk::TreeModel::Row row = *item_store->get_iter(path);
                 const ObjectType type = row[list_columns.type];
                 if (type != ObjectType::MODEL_3D)
-                    notebook->go_to(type, row[list_columns.uuid]);
+                    notebook.go_to(type, row[list_columns.uuid]);
             });
 
     merge_items_view->append_column("Name", list_columns.name);
+    add_check_column(*merge_items_view, list_columns.check_result);
 
     merge_items_clear_button->signal_clicked().connect([this] {
         items_merge.clear();
+        models_merge.clear();
         update_items_merge();
     });
 
@@ -273,6 +302,8 @@ PoolRemoteBox::PoolRemoteBox(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Bu
         update_items_merge();
     });
 
+    merge_items_run_checks_button->signal_clicked().connect(sigc::mem_fun(*this, &PoolRemoteBox::update_items_merge));
+
     merge_items_view->get_selection()->signal_changed().connect([this] {
         auto it = merge_items_view->get_selection()->get_selected();
         merge_items_remove_button->set_sensitive(it);
@@ -291,6 +322,21 @@ PoolRemoteBox::PoolRemoteBox(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Bu
 
     pr_body_textview->get_buffer()->signal_changed().connect([this] { update_body_placeholder_label(); });
 
+    {
+        Gtk::Button *remote_check_update_button;
+        GET_WIDGET(remote_check_update_button);
+        if (PoolManager::get().get_pools().count(notebook.base_path)) {
+            remote_check_update_button->signal_clicked().connect([this] {
+                auto w = notebook.appwin.app.show_pools_window();
+                w->show_pool(notebook.base_path);
+                w->check_for_updates();
+            });
+        }
+        else {
+            remote_check_update_button->hide();
+        }
+    }
+
     set_pr_update_mode(0, "");
 }
 
@@ -331,7 +377,7 @@ static std::pair<git_oid, autofree_ptr<git_tree>> find_base(git_repository *repo
 
 void PoolRemoteBox::pr_diff_file_cb(const git_diff_delta *delta)
 {
-    SQLite::Query q(notebook->pool.db, "INSERT INTO 'git_files' VALUES (?)");
+    SQLite::Query q(notebook.pool.db, "INSERT INTO 'git_files' VALUES (?)");
     q.bind(1, std::string(delta->new_file.path));
     q.step();
 }
@@ -356,11 +402,12 @@ void PoolRemoteBox::set_pr_update_mode(unsigned int pr, const std::string branch
         pr_update_cancel_button->set_visible(false);
         create_pr_button->set_visible(true);
         pr_update_nr = 0;
+        update_prs();
         return;
     }
 
     autofree_ptr<git_repository> repo(git_repository_free);
-    if (git_repository_open(&repo.ptr, notebook->remote_repo.c_str()) != 0) {
+    if (git_repository_open(&repo.ptr, notebook.remote_repo.c_str()) != 0) {
         throw std::runtime_error("error opening repo");
     }
 
@@ -369,10 +416,10 @@ void PoolRemoteBox::set_pr_update_mode(unsigned int pr, const std::string branch
         err == GIT_ENOTFOUND) {
         // branch not found
 
-        if (notebook->pool_updating)
+        if (notebook.pool_busy)
             return;
 
-        notebook->pool_updating = true;
+        notebook.pool_busy = true;
         git_thread_busy = true;
         git_thread_error = false;
         git_thread_mode = GitThreadMode::PULL_REQUEST_UPDATE_PREPARE;
@@ -394,6 +441,8 @@ void PoolRemoteBox::set_pr_update_mode(unsigned int pr, const std::string branch
     pr_update_cancel_button->set_visible(true);
     create_pr_button->set_visible(false);
     pr_update_button->set_label("Update PR #" + std::to_string(pr));
+    pr_update_nr = pr;
+    pr_update_branch = branch_name;
     update_prs();
 
     autofree_ptr<git_commit> latest_commit(git_commit_free);
@@ -414,19 +463,19 @@ void PoolRemoteBox::set_pr_update_mode(unsigned int pr, const std::string branch
     git_diff_tree_to_tree(&diff.ptr, repo, base_tree, tree, &diff_opts);
 
 
-    notebook->pool.db.execute("CREATE TEMP TABLE 'git_files' ('git_filename' TEXT NOT NULL);");
-    notebook->pool.db.execute("BEGIN TRANSACTION");
+    notebook.pool.db.execute("CREATE TEMP TABLE 'git_files' ('git_filename' TEXT NOT NULL);");
+    notebook.pool.db.execute("BEGIN TRANSACTION");
     git_diff_foreach(diff, &PoolRemoteBox::pr_diff_file_cb_c, nullptr, nullptr, nullptr, this);
-    notebook->pool.db.execute("COMMIT");
+    notebook.pool.db.execute("COMMIT");
 
     SQLite::Query q(
-            notebook->pool.db,
+            notebook.pool.db,
             "SELECT type, uuid, name, filename FROM git_files LEFT JOIN "
             "(SELECT type, uuid, name, filename FROM all_items_view UNION ALL SELECT DISTINCT 'model_3d' AS type, "
             "'00000000-0000-0000-0000-000000000000' as uuid, '' as name, model_filename as filename FROM models) "
             "ON filename=git_filename WHERE filename IS NOT NULL");
     while (q.step()) {
-        const ObjectType type = object_type_lut.lookup(q.get<std::string>(0));
+        const auto type = q.get<ObjectType>(0);
         const UUID uu = q.get<std::string>(1);
         const auto filename = q.get<std::string>(3);
         if (type == ObjectType::MODEL_3D) {
@@ -438,7 +487,7 @@ void PoolRemoteBox::set_pr_update_mode(unsigned int pr, const std::string branch
     }
     update_items_merge();
 
-    notebook->pool.db.execute("DROP TABLE 'git_files'");
+    notebook.pool.db.execute("DROP TABLE 'git_files'");
 }
 
 class PullRequestItemBox : public Gtk::Box {
@@ -552,87 +601,35 @@ void PoolRemoteBox::merge_3d_model(const std::string &filename)
     update_items_merge();
 }
 
-bool PoolRemoteBox::exists_in_pool(Pool &pool, ObjectType ty, const UUID &uu)
-{
-    try {
-        pool.get_filename(ty, uu);
-        return true;
-    }
-    catch (...) {
-        return false;
-    }
-}
-
 ItemSet PoolRemoteBox::get_referenced(ObjectType ty, const UUID &uu)
 {
     ItemSet items;
-    items.emplace(ty, uu);
-    Pool pool_remote(notebook->remote_repo);
-    bool added = true;
-    while (added) {
-        added = false;
-        for (const auto &it : items) {
-            switch (it.first) {
-            case ObjectType::ENTITY: {
-                auto entity = notebook->pool.get_entity(it.second);
-                for (const auto &it_gate : entity->gates) {
-                    const Unit *unit = it_gate.second.unit;
-                    if (!exists_in_pool(pool_remote, ObjectType::UNIT, unit->uuid)) {
-                        if (items.emplace(ObjectType::UNIT, unit->uuid).second)
-                            added = true;
-                    }
-                }
-            } break;
-
-            case ObjectType::SYMBOL: {
-                auto symnol = notebook->pool.get_symbol(it.second);
-                const Unit *unit = symnol->unit;
-                if (!exists_in_pool(pool_remote, ObjectType::UNIT, unit->uuid)) {
-                    if (items.emplace(ObjectType::UNIT, unit->uuid).second)
-                        added = true;
-                }
-            } break;
-
-            case ObjectType::PART: {
-                auto part = notebook->pool.get_part(it.second);
-                const Entity *entity = part->entity;
-                const Package *package = part->package;
-                if (!exists_in_pool(pool_remote, ObjectType::ENTITY, entity->uuid)) {
-                    if (items.emplace(ObjectType::ENTITY, entity->uuid).second)
-                        added = true;
-                }
-                if (!exists_in_pool(pool_remote, ObjectType::PACKAGE, package->uuid)) {
-                    if (items.emplace(ObjectType::PACKAGE, package->uuid).second)
-                        added = true;
-                }
-            } break;
-
-            case ObjectType::PACKAGE: {
-                auto package = notebook->pool.get_package(it.second);
-                for (const auto &it_pad : package->pads) {
-                    const Padstack *padstack = it_pad.second.pool_padstack;
-                    if (!exists_in_pool(pool_remote, ObjectType::PADSTACK, padstack->uuid)) {
-                        if (items.emplace(ObjectType::PADSTACK, padstack->uuid).second)
-                            added = true;
-                    }
-                }
-                const Package *alt_pkg = package->alternate_for;
-                if (alt_pkg && !exists_in_pool(pool_remote, ObjectType::PACKAGE, alt_pkg->uuid)) {
-                    if (items.emplace(ObjectType::PACKAGE, alt_pkg->uuid).second)
-                        added = true;
-                }
-            } break;
-
-            default:;
-            }
-            if (added)
-                break;
-        }
+    {
+        SQLite::Query q(notebook.pool.db, "ATTACH ? AS remote");
+        q.bind(1, Glib::build_filename(notebook.remote_repo, "pool.db"));
+        q.step();
     }
+    const char *qs =
+            "WITH RECURSIVE deps(typex, uuidx) AS ( SELECT ?, ? UNION SELECT dep_type, dep_uuid FROM dependencies, "
+            "deps WHERE dependencies.type = deps.typex AND dependencies.uuid = deps.uuidx), "
+            "deps_sym(typey, uuidy) "
+            "AS (SELECT * FROM deps UNION SELECT 'symbol', symbols.uuid FROM symbols INNER JOIN deps "
+            "ON (symbols.unit = uuidx AND typex = 'unit')) "
+            "SELECT deps_sym.typey, deps_sym.uuidy FROM deps_sym "
+            "LEFT JOIN remote.all_items_view AS remote_items "
+            "ON (remote_items.type = deps_sym.typey AND remote_items.uuid = deps_sym.uuidy) "
+            "WHERE remote_items.uuid IS NULL";
+    SQLite::Query q(notebook.pool.db, qs);
+    q.bind(1, ty);
+    q.bind(2, uu);
+    while (q.step()) {
+        items.emplace(q.get<ObjectType>(0), q.get<std::string>(1));
+    }
+    notebook.pool.db.execute("DETACH remote");
     return items;
 }
 
-PoolRemoteBox *PoolRemoteBox::create(PoolNotebook *nb)
+PoolRemoteBox *PoolRemoteBox::create(PoolNotebook &nb)
 {
     PoolRemoteBox *w;
     Glib::RefPtr<Gtk::Builder> x = Gtk::Builder::create();
@@ -643,25 +640,11 @@ PoolRemoteBox *PoolRemoteBox::create(PoolNotebook *nb)
     return w;
 }
 
-void PoolRemoteBox::handle_remote_upgrade()
-{
-    if (notebook->pool_updating)
-        return;
-    notebook->pool_updating = true;
-    git_thread_busy = true;
-    git_thread_error = false;
-    git_thread_mode = GitThreadMode::UPGRADE;
-
-    std::thread thr(&PoolRemoteBox::remote_upgrade_thread, this);
-
-    thr.detach();
-}
-
 void PoolRemoteBox::handle_refresh_prs()
 {
-    if (notebook->pool_updating)
+    if (notebook.pool_busy)
         return;
-    notebook->pool_updating = true;
+    notebook.pool_busy = true;
 
     pr_status_dispatcher.reset("Starting…");
     std::thread thr(&PoolRemoteBox::refresh_prs_thread, this);
@@ -671,14 +654,14 @@ void PoolRemoteBox::handle_refresh_prs()
 
 void PoolRemoteBox::handle_create_pr()
 {
-    if (notebook->pool_updating)
+    if (notebook.pool_busy)
         return;
 
     auto top = dynamic_cast<Gtk::Window *>(get_ancestor(GTK_TYPE_WINDOW));
 
     { // confirm user name
         autofree_ptr<git_repository> repo(git_repository_free);
-        if (git_repository_open(&repo.ptr, notebook->remote_repo.c_str()) != 0) {
+        if (git_repository_open(&repo.ptr, notebook.remote_repo.c_str()) != 0) {
             throw std::runtime_error("error opening repo");
         }
 
@@ -709,7 +692,7 @@ void PoolRemoteBox::handle_create_pr()
         git_config_set_string(repo_config, "user.email", dia.email_entry->get_text().c_str());
     }
 
-    notebook->pool_updating = true;
+    notebook.pool_busy = true;
     git_thread_busy = true;
     git_thread_error = false;
     git_thread_mode = GitThreadMode::PULL_REQUEST;
@@ -726,13 +709,13 @@ void PoolRemoteBox::handle_create_pr()
 
 void PoolRemoteBox::handle_update_pr()
 {
-    if (notebook->pool_updating)
+    if (notebook.pool_busy)
         return;
 
     if (!pr_update_nr)
         return;
 
-    notebook->pool_updating = true;
+    notebook.pool_busy = true;
     git_thread_busy = true;
     git_thread_error = false;
     git_thread_mode = GitThreadMode::PULL_REQUEST_UPDATE;
@@ -753,16 +736,19 @@ void PoolRemoteBox::update_items_merge()
     auto can_merge = items_merge.size() > 0 || models_merge.size() > 0;
     create_pr_button->set_sensitive(can_merge);
     merge_items_clear_button->set_sensitive(can_merge);
+    merge_items_run_checks_button->set_sensitive(can_merge);
     merge_items_placeholder_label->set_visible(items_merge.size() == 0 && models_merge.size() == 0);
     for (const auto &it : items_merge) {
-        SQLite::Query q(notebook->pool.db, "SELECT name FROM all_items_view WHERE uuid = ? AND type = ?");
+        SQLite::Query q(notebook.pool.db, "SELECT name FROM all_items_view WHERE uuid = ? AND type = ?");
         q.bind(1, it.second);
-        q.bind(2, object_type_lut.lookup_reverse(it.first));
+        q.bind(2, it.first);
         if (q.step()) {
             Gtk::TreeModel::Row row = *item_store->append();
             row[list_columns.name] = q.get<std::string>(0);
             row[list_columns.type] = it.first;
             row[list_columns.uuid] = it.second;
+            auto result = check_item(notebook.pool, it.first, it.second);
+            row[list_columns.check_result] = result;
         }
     }
 
@@ -772,6 +758,16 @@ void PoolRemoteBox::update_items_merge()
         row[list_columns.type] = ObjectType::MODEL_3D;
         row[list_columns.filename] = it;
     }
+
+    if (notebook.page_num(*this) != -1) {
+        const auto n = items_merge.size() + models_merge.size();
+        if (n > 0) {
+            notebook.set_tab_label_text(*this, "Remote (" + std::to_string(n) + ")");
+        }
+        else {
+            notebook.set_tab_label_text(*this, "Remote");
+        }
+    }
 }
 
 void PoolRemoteBox::update_body_placeholder_label()
@@ -804,125 +800,9 @@ void PoolRemoteBox::refresh_prs_thread()
     }
 }
 
-void PoolRemoteBox::remote_upgrade_thread()
-{
-    try {
-        {
-            std::lock_guard<std::mutex> lock(git_thread_mutex);
-            git_thread_status = "Opening repository…";
-        }
-        git_thread_dispatcher.emit();
-
-        autofree_ptr<git_repository> repo(git_repository_free);
-        if (git_repository_open(&repo.ptr, notebook->remote_repo.c_str()) != 0) {
-            throw std::runtime_error("error opening repo");
-        }
-
-        autofree_ptr<git_remote> remote(git_remote_free);
-        if (git_remote_lookup(&remote.ptr, repo, "origin") != 0) {
-            throw std::runtime_error("error looking up remote");
-        }
-
-        {
-            std::lock_guard<std::mutex> lock(git_thread_mutex);
-            git_thread_status = "Fetching…";
-        }
-        git_thread_dispatcher.emit();
-
-        git_fetch_options fetch_opts = GIT_FETCH_OPTIONS_INIT;
-        if (git_remote_fetch(remote, NULL, &fetch_opts, NULL) != 0) {
-            throw std::runtime_error("error fetching");
-        }
-
-        autofree_ptr<git_annotated_commit> latest_commit(git_annotated_commit_free);
-        if (git_annotated_commit_from_revspec(&latest_commit.ptr, repo, "origin/master") != 0) {
-            throw std::runtime_error("error getting latest commit ");
-        }
-
-        auto oid = git_annotated_commit_id(latest_commit);
-
-        git_checkout_options checkout_opts = GIT_CHECKOUT_OPTIONS_INIT;
-        checkout_opts.checkout_strategy = GIT_CHECKOUT_SAFE;
-
-        const git_annotated_commit *com = latest_commit.ptr;
-        git_merge_analysis_t merge_analysis;
-        git_merge_preference_t merge_prefs = GIT_MERGE_PREFERENCE_FASTFORWARD_ONLY;
-        if (git_merge_analysis(&merge_analysis, &merge_prefs, repo, &com, 1) != 0) {
-            throw std::runtime_error("error getting merge analysis");
-        }
-
-        if (!(merge_analysis & GIT_MERGE_ANALYSIS_UP_TO_DATE)) {
-            if (!(merge_analysis & GIT_MERGE_ANALYSIS_FASTFORWARD)) {
-                throw std::runtime_error("can't fast-forward");
-            }
-
-            autofree_ptr<git_object> obj(git_object_free);
-            if (git_object_lookup(&obj.ptr, repo, oid, GIT_OBJ_COMMIT) != 0) {
-                throw std::runtime_error("error lookup");
-            }
-
-            if (git_checkout_tree(repo, obj, &checkout_opts) != 0) {
-                throw std::runtime_error("error checkout");
-            }
-
-            autofree_ptr<git_reference> ref_head(git_reference_free);
-            if (git_repository_head(&ref_head.ptr, repo) != 0) {
-                throw std::runtime_error("error head");
-            }
-
-            autofree_ptr<git_reference> ref_new(git_reference_free);
-            if (git_reference_set_target(&ref_new.ptr, ref_head, oid, "test") != 0) {
-                throw std::runtime_error("error target");
-            }
-        }
-        else {
-            std::cout << "up to date" << std::endl;
-        }
-
-        {
-            std::lock_guard<std::mutex> lock(git_thread_mutex);
-            git_thread_status = "Updating remote pool…";
-        }
-        git_thread_dispatcher.emit();
-        pool_update(notebook->remote_repo);
-
-        {
-            std::lock_guard<std::mutex> lock(git_thread_mutex);
-            git_thread_status = "Updating local pool…";
-        }
-        git_thread_dispatcher.emit();
-        unsigned int n_errors = 0;
-        pool_update(notebook->base_path,
-                    [&n_errors](PoolUpdateStatus status, std::string filename, std::string message) {
-                        if (status == PoolUpdateStatus::FILE_ERROR) {
-                            n_errors++;
-                        }
-                    });
-        if (n_errors) {
-            throw std::runtime_error("local pool update encountered errors");
-        }
-
-        {
-            std::lock_guard<std::mutex> lock(git_thread_mutex);
-            git_thread_busy = false;
-            git_thread_status = "Done";
-        }
-        git_thread_dispatcher.emit();
-    }
-    catch (const std::exception &e) {
-        {
-            std::lock_guard<std::mutex> lock(git_thread_mutex);
-            git_thread_busy = false;
-            git_thread_error = true;
-            git_thread_status = "Error: " + std::string(e.what());
-        }
-        git_thread_dispatcher.emit();
-    }
-}
-
 std::string PoolRemoteBox::get_token_filename() const
 {
-    return Glib::build_filename(notebook->remote_repo, ".auth.json");
+    return Glib::build_filename(notebook.remote_repo, ".auth.json");
 }
 
 bool PoolRemoteBox::update_login()
@@ -938,9 +818,9 @@ bool PoolRemoteBox::update_login()
         logout_button->hide();
     }
     else {
-        if (notebook->pool_updating)
+        if (notebook.pool_busy)
             return false;
-        notebook->pool_updating = true;
+        notebook.pool_busy = true;
         git_thread_busy = true;
         git_thread_error = false;
         git_thread_mode = GitThreadMode::LOGIN;
@@ -1048,11 +928,11 @@ git_oid PoolRemoteBox::items_to_tree(git_repository *repo)
     std::set<std::string> files_merge;
 
     for (const auto &it : items_merge) {
-        SQLite::Query q(notebook->pool.db,
+        SQLite::Query q(notebook.pool.db,
                         "SELECT filename FROM all_items_view WHERE "
                         "uuid = ? AND type = ?");
         q.bind(1, it.second);
-        q.bind(2, object_type_lut.lookup_reverse(it.first));
+        q.bind(2, it.first);
         if (q.step()) {
             std::string filename = q.get<std::string>(0);
             files_merge.insert(filename);
@@ -1062,8 +942,8 @@ git_oid PoolRemoteBox::items_to_tree(git_repository *repo)
 
     for (std::string filename : files_merge) {
         std::cout << "merge " << filename << std::endl;
-        std::string filename_src = Glib::build_filename(notebook->base_path, filename);
-        std::string filename_dest = Glib::build_filename(notebook->remote_repo, filename);
+        std::string filename_src = Glib::build_filename(notebook.base_path, filename);
+        std::string filename_dest = Glib::build_filename(notebook.remote_repo, filename);
         std::string dirname_dest = Glib::path_get_dirname(filename_dest);
         if (!Glib::file_test(dirname_dest, Glib::FILE_TEST_IS_DIR))
             Gio::File::create_for_path(dirname_dest)->make_directory_with_parents();
@@ -1213,7 +1093,7 @@ void PoolRemoteBox::create_pr_thread()
         git_thread_dispatcher.emit();
 
         autofree_ptr<git_repository> repo(git_repository_free);
-        if (git_repository_open(&repo.ptr, notebook->remote_repo.c_str()) != 0) {
+        if (git_repository_open(&repo.ptr, notebook.remote_repo.c_str()) != 0) {
             throw std::runtime_error("error opening repo");
         }
 
@@ -1263,7 +1143,9 @@ void PoolRemoteBox::create_pr_thread()
 
 
             autofree_ptr<git_signature> signature(git_signature_free);
-            git_signature_default(&signature.ptr, repo);
+            if (git_signature_default(&signature.ptr, repo) != 0) {
+                throw std::runtime_error("error getting default signature");
+            }
 
             git_oid new_commit_oid;
             if (git_commit_create(&new_commit_oid, repo, "HEAD", signature, signature, "UTF-8", pr_title.c_str(), tree,
@@ -1359,7 +1241,7 @@ void PoolRemoteBox::update_pr_thread()
         git_thread_dispatcher.emit();
 
         autofree_ptr<git_repository> repo(git_repository_free);
-        if (git_repository_open(&repo.ptr, notebook->remote_repo.c_str()) != 0) {
+        if (git_repository_open(&repo.ptr, notebook.remote_repo.c_str()) != 0) {
             throw std::runtime_error("error opening repo");
         }
 
@@ -1400,7 +1282,9 @@ void PoolRemoteBox::update_pr_thread()
             }
 
             autofree_ptr<git_signature> signature(git_signature_free);
-            git_signature_default(&signature.ptr, repo);
+            if (git_signature_default(&signature.ptr, repo) != 0) {
+                throw std::runtime_error("error getting default signature");
+            }
 
             const git_commit *parent_commit_p = latest_commit.ptr;
 
@@ -1504,7 +1388,7 @@ void PoolRemoteBox::update_prepare_pr_thread()
         git_thread_dispatcher.emit();
 
         autofree_ptr<git_repository> repo(git_repository_free);
-        if (git_repository_open(&repo.ptr, notebook->remote_repo.c_str()) != 0) {
+        if (git_repository_open(&repo.ptr, notebook.remote_repo.c_str()) != 0) {
             throw std::runtime_error("error opening repo");
         }
 
