@@ -12,20 +12,25 @@ using json = nlohmann::json;
 
 class PoolSettingsBox : public Gtk::Box {
 public:
-    PoolSettingsBox(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder> &x, class PoolNotebook *nb,
-                    const std::string &bp);
-    static PoolSettingsBox *create(class PoolNotebook *nb, const std::string &bp);
+    PoolSettingsBox(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder> &x, class IPool &pool);
+    static PoolSettingsBox *create(class IPool &pool);
     bool get_needs_save() const;
     void save();
     void pool_updated();
 
+    typedef sigc::signal<void, std::string> type_signal_open_pool;
+    type_signal_open_pool signal_open_pool()
+    {
+        return s_signal_open_pool;
+    }
+
 private:
-    class PoolNotebook *notebook = nullptr;
-    std::string pool_base_path;
+    IPool &pool;
     Gtk::Entry *entry_name = nullptr;
     Gtk::Button *save_button = nullptr;
     Gtk::ListBox *pools_available_listbox = nullptr;
     Gtk::ListBox *pools_included_listbox = nullptr;
+    Gtk::ListBox *pools_actually_included_listbox = nullptr;
     Gtk::Button *pool_inc_button = nullptr;
     Gtk::Button *pool_excl_button = nullptr;
     Gtk::Button *pool_up_button = nullptr;
@@ -35,10 +40,14 @@ private:
     std::vector<UUID> pools_included;
 
     void update_pools();
+    void update_actual();
 
     void inc_excl_pool(bool inc);
+    void pool_up_down(bool up);
 
     bool needs_save = false;
     void set_needs_save();
+
+    type_signal_open_pool s_signal_open_pool;
 };
 } // namespace horizon
