@@ -16,6 +16,8 @@
 #include "logger/logger.hpp"
 #include "widgets/log_view.hpp"
 #include "util/zmq_helper.hpp"
+#include "pools_window/pools_window.hpp"
+#include "util/stock_info_provider.hpp"
 
 #ifdef G_OS_WIN32
 #include <winsock2.h>
@@ -29,10 +31,7 @@ PoolProjectManagerApplication::PoolProjectManagerApplication()
       sock_broadcast(zctx, ZMQ_PUB)
 {
     sock_broadcast.bind("tcp://127.0.0.1:*");
-    char ep[1024];
-    size_t sz = sizeof(ep);
-    sock_broadcast.getsockopt(ZMQ_LAST_ENDPOINT, ep, &sz);
-    sock_broadcast_ep = ep;
+    sock_broadcast_ep = zmq_helper::get_last_endpoint(sock_broadcast);
 }
 
 const std::string &PoolProjectManagerApplication::get_ep_broadcast() const
@@ -58,7 +57,7 @@ Glib::RefPtr<PoolProjectManagerApplication> PoolProjectManagerApplication::creat
 
 PoolProjectManagerAppWindow *PoolProjectManagerApplication::create_appwindow()
 {
-    auto appwindow = PoolProjectManagerAppWindow::create(this);
+    auto appwindow = PoolProjectManagerAppWindow::create(*this);
 
     // Make sure that the application runs for as long this window is still
     // open.
@@ -117,6 +116,7 @@ void PoolProjectManagerApplication::on_startup()
     curl_global_init(CURL_GLOBAL_ALL);
 
     add_action("preferences", [this] { show_preferences_window(); });
+    add_action("pools", [this] { show_pools_window(); });
     add_action("logger", [this] { show_log_window(); });
     add_action("quit", sigc::mem_fun(*this, &PoolProjectManagerApplication::on_action_quit));
     add_action("new_window", sigc::mem_fun(*this, &PoolProjectManagerApplication::on_action_new_window));
@@ -176,6 +176,20 @@ PreferencesWindow *PoolProjectManagerApplication::show_preferences_window(guint3
     return preferences_window;
 }
 
+PoolsWindow *PoolProjectManagerApplication::show_pools_window(guint32 timestamp)
+{
+    if (!pools_window) {
+        pools_window = PoolsWindow::create(*this);
+
+        pools_window->signal_hide().connect([this] {
+            delete pools_window;
+            pools_window = nullptr;
+        });
+    }
+    pools_window->present(timestamp);
+    return pools_window;
+}
+
 LogWindow *PoolProjectManagerApplication::show_log_window(guint32 timestamp)
 {
     log_window->present(timestamp);
@@ -199,6 +213,8 @@ void PoolProjectManagerApplication::on_shutdown()
     }
     j["pool_doc_info_bar_dismissed"] = pool_doc_info_bar_dismissed;
     save_json_to_file(config_filename, j);
+
+    StockInfoProvider::cleanup();
 }
 
 void PoolProjectManagerApplication::on_open(const Gio::Application::type_vec_files &files,
@@ -224,7 +240,16 @@ void PoolProjectManagerApplication::on_open(const Gio::Application::type_vec_fil
 void PoolProjectManagerApplication::open_pool(const std::string &pool_json, ObjectType type, const UUID &uu,
                                               guint32 timestamp)
 {
-    auto appwindow = create_appwindow();
+    PoolProjectManagerAppWindow *appwindow = nullptr;
+    for (auto ws : get_windows()) {
+        auto wi = dynamic_cast<PoolProjectManagerAppWindow *>(ws);
+        if (wi && wi->get_filename() == pool_json) {
+            appwindow = wi;
+            break;
+        }
+    }
+    if (!appwindow)
+        appwindow = create_appwindow();
     appwindow->open_pool(pool_json, type, uu);
     appwindow->present(timestamp);
 }
@@ -279,7 +304,7 @@ void PoolProjectManagerApplication::on_action_quit()
             for (const auto &it : files_from_dia) {
                 auto win = files_windows.at(it.first);
                 win->prepare_close();
-                if (ConfirmCloseDialog::RESPONSE_SAVE) {
+                if (r == ConfirmCloseDialog::RESPONSE_SAVE) {
                     for (const auto &it2 : it.second) {
                         win->process_save(it2);
                     }
