@@ -36,9 +36,16 @@ public:
                    guint32 timestamp = 0);
 
     class PreferencesWindow *show_preferences_window(guint32 timestamp = 0);
+    class PoolsWindow *show_pools_window(guint32 timestamp = 0);
     class LogWindow *show_log_window(guint32 timestamp = 0);
 
-    typedef sigc::signal<void, UUID> type_signal_pool_updated;
+    typedef sigc::signal<void, std::vector<std::string>> type_signal_pool_items_edited;
+    type_signal_pool_items_edited signal_pool_items_edited()
+    {
+        return s_signal_pool_items_edited;
+    }
+
+    typedef sigc::signal<void, std::string> type_signal_pool_updated;
     type_signal_pool_updated signal_pool_updated()
     {
         return s_signal_pool_updated;
@@ -63,10 +70,12 @@ private:
     void load_from_config(const std::string &config_filename);
     Preferences preferences;
     class PreferencesWindow *preferences_window = nullptr;
+    class PoolsWindow *pools_window = nullptr;
 
     LogDispatcher log_dispatcher;
     class LogWindow *log_window = nullptr;
 
+    type_signal_pool_items_edited s_signal_pool_items_edited;
     type_signal_pool_updated s_signal_pool_updated;
 
 
