@@ -7,27 +7,26 @@
 #include "nlohmann/json_fwd.hpp"
 #include "util/editor_process.hpp"
 #include "util/window_state_store.hpp"
-#include "util/status_dispatcher.hpp"
 #include "pool-prj-mgr-process.hpp"
-#include "project/project.hpp"
 #include "prj-mgr/prj-mgr_views.hpp"
 #include "pool-mgr/view_create_pool.hpp"
 #include "common/common.hpp"
-#include <git2.h>
 
 namespace horizon {
 using json = nlohmann::json;
 
+enum class PoolUpdateStatus;
+
 class PoolProjectManagerAppWindow : public Gtk::ApplicationWindow {
     friend class PoolProjectManagerViewProject;
     friend class PoolProjectManagerViewCreateProject;
 
 public:
     PoolProjectManagerAppWindow(BaseObjectType *cobject, const Glib::RefPtr<Gtk::Builder> &refBuilder,
-                                class PoolProjectManagerApplication *app);
+                                class PoolProjectManagerApplication &app);
     ~PoolProjectManagerAppWindow();
 
-    static PoolProjectManagerAppWindow *create(class PoolProjectManagerApplication *app);
+    static PoolProjectManagerAppWindow *create(class PoolProjectManagerApplication &app);
 
     void open_file_view(const Glib::RefPtr<Gio::File> &file);
     void prepare_close();
@@ -36,11 +35,10 @@ public:
     void wait_for_all_processes();
     std::string get_filename() const;
 
+    enum class SpawnFlags { NONE = 0, READ_ONLY = (1 << 0), TEMP = (1 << 1) };
+
     PoolProjectManagerProcess *spawn(PoolProjectManagerProcess::Type type, const std::vector<std::string> &args,
-                                     const std::vector<std::string> &env = {}, bool read_only = false,
-                                     bool is_temp = false);
-    PoolProjectManagerProcess *spawn_for_project(PoolProjectManagerProcess::Type type,
-                                                 const std::vector<std::string> &args);
+                                     SpawnFlags flags = SpawnFlags::NONE);
 
     std::map<UUID, PoolProjectManagerProcess *> get_processes();
 
@@ -59,8 +57,6 @@ public:
         return s_signal_process_saved;
     }
 
-    void reload();
-
     class ClosePolicy {
     public:
         bool can_close = true;
@@ -73,16 +69,20 @@ public:
     std::string get_proc_filename(const UUID &uu);
     void process_save(const UUID &uu);
     void process_close(const UUID &uu);
-    void cleanup_pool_cache();
+    bool cleanup_pool_cache(Gtk::Window *parent);
 
-    enum class ViewMode { OPEN, POOL, DOWNLOAD, PROJECT, CREATE_PROJECT, CREATE_POOL };
+    enum class ViewMode { OPEN, POOL, PROJECT, CREATE_PROJECT, CREATE_POOL };
     ViewMode get_view_mode() const;
 
     UUID get_pool_uuid() const;
     void pool_notebook_go_to(ObjectType type, const UUID &uu);
     void open_pool(const std::string &pool_json, ObjectType type = ObjectType::INVALID, const UUID &uu = UUID());
-    void handle_download(bool back_to_start = false);
     void update_pool_cache_status_now();
+    const std::string &get_project_title() const;
+
+    void pool_update(const std::vector<std::string> &filenames = {});
+
+    PoolProjectManagerApplication &app;
 
 private:
     Glib::RefPtr<Gtk::Builder> builder;
@@ -90,24 +90,12 @@ private:
     Gtk::Button *button_open = nullptr;
     Gtk::Button *button_close = nullptr;
     Gtk::Button *button_update = nullptr;
-    Gtk::Button *button_download = nullptr;
-    Gtk::Button *button_do_download = nullptr;
     Gtk::Button *button_cancel = nullptr;
     Gtk::MenuButton *button_new = nullptr;
     Gtk::Button *button_create = nullptr;
     Gtk::Button *button_save = nullptr;
-    Gtk::Spinner *spinner_update = nullptr;
-    Gtk::Revealer *download_revealer = nullptr;
-    Gtk::Label *download_label = nullptr;
-    Gtk::Spinner *download_spinner = nullptr;
-    Gtk::ProgressBar *download_progress = nullptr;
     Gtk::MenuButton *hamburger_menu_button = nullptr;
 
-    Gtk::Entry *download_gh_username_entry = nullptr;
-    Gtk::Entry *download_gh_repo_entry = nullptr;
-    Gtk::Entry *download_dest_dir_entry = nullptr;
-    Gtk::Button *download_dest_dir_button = nullptr;
-
     Gtk::HeaderBar *header = nullptr;
     Gtk::ListBox *recent_pools_listbox = nullptr;
     Gtk::ListBox *recent_projects_listbox = nullptr;
@@ -137,13 +125,13 @@ private:
     Gtk::InfoBar *info_bar_version = nullptr;
     Gtk::Label *version_label = nullptr;
 
-    std::unique_ptr<Project> project = nullptr;
+    Gtk::InfoBar *info_bar_gitignore = nullptr;
+
+    std::unique_ptr<class Project> project;
     std::string project_filename;
     bool project_needs_save = false;
     void save_project();
     class PartBrowserWindow *part_browser_window = nullptr;
-    class PoolCacheWindow *pool_cache_window = nullptr;
-    std::unique_ptr<class PoolCacheMonitor> pool_cache_monitor;
     void cleanup();
 
     ViewMode view_mode = ViewMode::OPEN;
@@ -155,7 +143,6 @@ private:
     void handle_close();
     void handle_recent();
     void handle_update();
-    void handle_do_download();
     void handle_new_project();
     void handle_new_pool();
     void handle_create();
@@ -165,10 +152,6 @@ private:
 
     bool on_delete_event(GdkEventAny *ev) override;
 
-    void download_thread(std::string gh_username, std::string gh_repo, std::string dest_dir);
-
-    StatusDispatcher download_status_dispatcher;
-
     WindowStateStore state_store;
 
     std::map<UUID, PoolProjectManagerProcess> processes;
@@ -190,29 +173,45 @@ private:
     void handle_place_part(const UUID &uu);
     void handle_assign_part(const UUID &uu);
 
-    PoolProjectManagerApplication *app;
     zmq::socket_t sock_mgr;
     std::string sock_mgr_ep;
 
     bool check_pools();
     bool check_schema_update(const std::string &base_path);
-
-    bool download_back_to_start = false;
+    void check_pool_update(const std::string &base_path);
 
     bool check_autosave(PoolProjectManagerProcess::Type type, const std::vector<std::string> &filenames);
 
-    static int git_transfer_cb(const git_transfer_progress *stats, void *payload);
-    static void git_checkout_progress_cb(const char *path, size_t completed_steps, size_t total_steps, void *payload);
-    bool downloading = false;
-    bool download_cancel = false;
-
     void set_version_info(const std::string &s);
     bool project_read_only = false;
 
-public:
-    zmq::context_t &zctx;
+    bool migrate_project(const std::string &path);
+    std::string project_title;
+
+    Glib::Dispatcher pool_update_dispatcher;
+    bool in_pool_update_handler = false;
+    std::mutex pool_update_status_queue_mutex;
+    std::list<std::tuple<PoolUpdateStatus, std::string, std::string>> pool_update_status_queue;
+    std::list<std::tuple<PoolUpdateStatus, std::string, std::string>> pool_update_error_queue;
+    bool pool_updating = false;
+    void pool_updated(bool success);
+    std::string pool_update_last_file;
+    std::string pool_update_last_info;
+    unsigned int pool_update_n_files = 0;
+    unsigned int pool_update_n_files_last = 0;
+    std::vector<std::string> pool_update_filenames;
+
+    void pool_update_thread();
+    void handle_pool_update_progress();
     void set_pool_updating(bool v, bool success);
     void set_pool_update_status_text(const std::string &txt);
     void set_pool_update_progress(float progress);
+
+    std::string get_pool_base_path() const;
+
+    std::string get_gitignore_filename() const;
+
+public:
+    zmq::context_t &zctx;
 };
 }; // namespace horizon
