@@ -5,6 +5,8 @@
 #ifdef G_OS_WIN32
 #include <windows.h>
 #endif
+#include "util/automatic_prefs.hpp"
+#include "util/stock_info_provider.hpp"
 
 int main(int argc, char *argv[])
 {
@@ -17,6 +19,8 @@ int main(int argc, char *argv[])
     horizon::setup_locale();
     horizon::create_config_dir();
     horizon::PoolManager::init();
+    horizon::AutomaticPreferences::get();
+    horizon::StockInfoProvider::init_db();
     horizon::install_signal_exception_handler();
 
     // Start the application, showing the initial window,
