@@ -6,14 +6,17 @@ namespace horizon {
 
 class PreferencesRow : public Gtk::Box {
 public:
-    PreferencesRow(const std::string &title, const std::string &subtitle);
+    PreferencesRow(const std::string &title, const std::string &subtitle, Preferences &prefs);
     virtual void activate()
     {
     }
+
+protected:
+    Preferences &preferences;
 };
 
-PreferencesRow::PreferencesRow(const std::string &title, const std::string &subtitle)
-    : Gtk::Box(Gtk::ORIENTATION_HORIZONTAL, 48)
+PreferencesRow::PreferencesRow(const std::string &title, const std::string &subtitle, Preferences &prefs)
+    : Gtk::Box(Gtk::ORIENTATION_HORIZONTAL, 48), preferences(prefs)
 {
     set_valign(Gtk::ALIGN_CENTER);
     property_margin() = 10;
@@ -44,14 +47,12 @@ public:
     void activate() override;
 
 private:
-    Preferences &preferences;
-    bool &value;
     Gtk::Switch *sw = nullptr;
 };
 
 PreferencesRowBool::PreferencesRowBool(const std::string &title, const std::string &subtitle, Preferences &prefs,
                                        bool &v)
-    : PreferencesRow(title, subtitle), preferences(prefs), value(v)
+    : PreferencesRow(title, subtitle, prefs)
 {
     sw = Gtk::manage(new Gtk::Switch);
     sw->set_valign(Gtk::ALIGN_CENTER);
@@ -66,6 +67,22 @@ void PreferencesRowBool::activate()
     sw->set_active(!sw->get_active());
 }
 
+class PreferencesRowBoolButton : public PreferencesRow {
+public:
+    PreferencesRowBoolButton(const std::string &title, const std::string &subtitle, const std::string &label_true,
+                             const std::string &label_false, Preferences &prefs, bool &v);
+};
+
+PreferencesRowBoolButton::PreferencesRowBoolButton(const std::string &title, const std::string &subtitle,
+                                                   const std::string &label_true, const std::string &label_false,
+                                                   Preferences &prefs, bool &v)
+    : PreferencesRow(title, subtitle, prefs)
+{
+    auto box = make_boolean_ganged_switch(v, label_false, label_true,
+                                          [this](bool x) { preferences.signal_changed().emit(); });
+    pack_start(*box, false, false, 0);
+}
+
 class PreferencesRowNumeric : public PreferencesRow {
 public:
     PreferencesRowNumeric(const std::string &title, const std::string &subtitle, Preferences &prefs, float &v);
@@ -74,14 +91,13 @@ public:
     void bind();
 
 private:
-    Preferences &preferences;
     float &value;
     Gtk::SpinButton *sp = nullptr;
 };
 
 PreferencesRowNumeric::PreferencesRowNumeric(const std::string &title, const std::string &subtitle, Preferences &prefs,
                                              float &v)
-    : PreferencesRow(title, subtitle), preferences(prefs), value(v)
+    : PreferencesRow(title, subtitle, prefs), value(v)
 {
     sp = Gtk::manage(new Gtk::SpinButton);
     sp->set_valign(Gtk::ALIGN_CENTER);
@@ -132,7 +148,7 @@ PreferencesGroup::PreferencesGroup(const std::string &title) : Gtk::Box(Gtk::ORI
     sc->add(*listbox);
     sc->show_all();
     sc->set_min_content_width(500);
-    listbox->signal_row_activated().connect([this](Gtk::ListBoxRow *lrow) {
+    listbox->signal_row_activated().connect([](Gtk::ListBoxRow *lrow) {
         if (auto row = dynamic_cast<PreferencesRow *>(lrow->get_child())) {
             row->activate();
         }
@@ -253,17 +269,36 @@ MiscPreferencesEditor::MiscPreferencesEditor(Preferences &prefs) : preferences(p
                                                         preferences, preferences.zoom.touchpad_pan));
             gr->add_row(*r);
         }
+        {
+            auto r = Gtk::manage(new PreferencesRowBoolButton("Keyboard zoom center", "Where to zoom in using keyboard",
+                                                              "Cursor", "Screen center", preferences,
+                                                              preferences.zoom.keyboard_zoom_to_cursor));
+            gr->add_row(*r);
+        }
     }
     {
         auto gr = Gtk::manage(new PreferencesGroup("Mouse"));
         box->pack_start(*gr, false, false, 0);
         gr->show();
+        {
+            auto r = Gtk::manage(new PreferencesRowBool("Drag to move",
+                                                        "Drag selected items to invoke the \"Move\" tool", preferences,
+                                                        preferences.mouse.drag_to_move));
+            gr->add_row(*r);
+        }
         {
             auto r = Gtk::manage(new PreferencesRowBool("Switch layers with navigation buttons",
                                                         "Use back/forward buttons on mouse to move one layer down/up",
                                                         preferences, preferences.mouse.switch_layers));
             gr->add_row(*r);
         }
+        {
+            auto r = Gtk::manage(
+                    new PreferencesRowBool("Switch sheets with navigation buttons",
+                                           "Use back/forward buttons on mouse to select next/previous sheet",
+                                           preferences, preferences.mouse.switch_sheets));
+            gr->add_row(*r);
+        }
         {
             auto r = Gtk::manage(new PreferencesRowBool(
                     "Keep slope when dragging polygon edge",
