@@ -58,9 +58,23 @@ public:
     UUID get_uuid() const;
 
     std::map<std::string, std::string> parametric;
+    class Column {
+    public:
+        Column(const std::string &d, const std::string &v) : display_name(d), value(v)
+        {
+        }
+        const std::string display_name;
+        const std::string value;
+    };
+    std::map<std::string, Column> parametric_formatted;
 
     std::map<UUID, PadMapItem> pad_map;
 
+    enum class FlagState { SET, CLEAR, INHERIT };
+    enum class Flag { EXCLUDE_BOM, EXCLUDE_PNP, BASE_PART };
+    std::map<Flag, FlagState> flags;
+    bool get_flag(Flag fl) const;
+
     FileVersion version;
 
     json serialize() const;
