@@ -11,6 +11,7 @@
 #include <set>
 #include <vector>
 #include "ipool.hpp"
+#include "pool_info.hpp"
 
 #include "util/sqlite.hpp"
 
@@ -40,6 +41,7 @@ public:
     std::string get_model_filename(const UUID &pkg_uuid, const UUID &model_uuid) override;
 
     virtual std::string get_filename(ObjectType type, const UUID &uu, UUID *pool_uuid_out = nullptr);
+    std::string get_rel_filename(ObjectType type, const UUID &uu);
     const std::string &get_base_path() const override;
     bool check_filename(ObjectType type, const std::string &filename, std::string *error_msg = nullptr) const;
 
@@ -48,6 +50,16 @@ public:
         return db;
     }
 
+    class PoolParametric *get_parametric() override
+    {
+        return nullptr;
+    }
+
+    const PoolInfo &get_pool_info() const override
+    {
+        return pool_info;
+    }
+
     /**
      * The database connection.
      * You may use it to perform more advanced queries on the pool.
@@ -64,8 +76,12 @@ public:
     virtual ~Pool();
     static const UUID tmp_pool_uuid;
 
+    std::map<std::string, UUID> get_actually_included_pools(bool include_self) override;
+
 protected:
-    std::string base_path;
+    const std::string base_path;
+    const PoolInfo pool_info;
+
     std::string get_flat_filename(ObjectType type, const UUID &uu) const;
 
     std::map<UUID, Unit> units;
