@@ -1,5 +1,6 @@
 #include "pool/package.hpp"
 #include "util/util.hpp"
+#include "util/geom_util.hpp"
 #include "nlohmann/json.hpp"
 #include "pool/pool.hpp"
 #include "board/board_layers.hpp"
@@ -17,11 +18,11 @@
 #include "util/version.hpp"
 #include "board/board.hpp"
 #include "export_3d_image/export_3d_image.hpp"
-#include "pool/pool_cached.hpp"
 #include "checks/check_entity.hpp"
 #include "checks/check_part.hpp"
 #include "checks/check_unit.hpp"
 #include "checks/check_util.hpp"
+#include "checks/check_item.hpp"
 
 using namespace horizon;
 
@@ -55,13 +56,6 @@ static std::string delta_to_string(git_delta_t delta)
     }
 }
 
-static int count_manufactuer(Pool &pool, const std::string &mfr)
-{
-    SQLite::Query q(pool.db, "SELECT COUNT(*) FROM parts WHERE manufacturer = ?");
-    q.bind(1, mfr);
-    q.step();
-    return q.get<int>(0);
-}
 
 static std::string surround_if(const char *prefix, const char *suffix, const std::string &s, bool cond = true)
 {
@@ -91,7 +85,52 @@ private:
 
 static PinDirectionMap pin_direction_map;
 
-static void print_rules_check_result(std::ostream &ofs, const RulesCheckResult &r, const std::string &name = "Checks")
+class Once {
+public:
+    bool operator()()
+    {
+        if (first) {
+            first = false;
+            return true;
+        }
+        return false;
+    }
+
+private:
+    bool first = true;
+};
+
+class Reviewer {
+public:
+    int main(int c_argc, char *c_argv[]);
+
+private:
+    void print_rules_check_result(const RulesCheckResult &r, const std::string &name = "Checks");
+    int count_manufactuer(const std::string &mfr);
+    int parse_options(int c_argc, char *c_argv[]);
+    void review_symbol(const UUID &uu, const std::string &title_prefix);
+
+    std::ofstream ofs;
+    std::optional<Pool> pool;
+
+    std::string images_dir;
+    std::string images_prefix;
+    std::string labels_filename;
+    std::string pool_base_path;
+    std::string output_filename;
+    bool do_pool_update = false;
+};
+
+int Reviewer::count_manufactuer(const std::string &mfr)
+{
+    SQLite::Query q(pool->db, "SELECT COUNT(*) FROM parts WHERE manufacturer = ?");
+    q.bind(1, mfr);
+    q.step();
+    return q.get<int>(0);
+}
+
+
+void Reviewer::print_rules_check_result(const RulesCheckResult &r, const std::string &name)
 {
     if (r.level != RulesCheckErrorLevel::PASS) {
         ofs << name << " didn't pass\n";
@@ -122,90 +161,105 @@ static void print_rules_check_result(std::ostream &ofs, const RulesCheckResult &
     ofs << "\n";
 }
 
-static void accumulate_level(RulesCheckErrorLevel &r, RulesCheckErrorLevel l)
+void Reviewer::review_symbol(const UUID &sym_uu, const std::string &title_prefix)
 {
-    r = static_cast<RulesCheckErrorLevel>(std::max(static_cast<int>(r), static_cast<int>(l)));
-}
-
-static RulesCheckResult check_item(Pool &pool, ObjectType type, const UUID &uu)
-{
-    switch (type) {
-    case ObjectType::UNIT:
-        return check_unit(*pool.get_unit(uu));
-
-    case ObjectType::ENTITY:
-        return check_entity(*pool.get_entity(uu));
-
-    case ObjectType::PART:
-        return check_part(*pool.get_part(uu));
-
-    case ObjectType::SYMBOL: {
-        const auto &sym = *pool.get_symbol(uu);
-        return sym.rules.check(RuleID::SYMBOL_CHECKS, sym);
+    const auto &pool_sym = *pool->get_symbol(sym_uu);
+    Symbol sym = pool_sym;
+    sym.expand();
+    sym.apply_placement(Placement());
+    ofs << title_prefix << " " << sym.name << "\n";
+    if (sym.can_expand) {
+        ofs << "Is expandable\n\n";
+    }
+    {
+        auto r = sym.rules.check(RuleID::SYMBOL_CHECKS, sym);
+        print_rules_check_result(r);
+        ofs << "\n";
+    }
+    for (auto &[uu, txt] : sym.texts) {
+        if (txt.text == "$VALUE") {
+            txt.text += "\nGroup\nTag";
+        }
     }
 
-    case ObjectType::PACKAGE: {
-        auto pkg = *pool.get_package(uu);
-        pkg.expand();
-        pkg.apply_parameter_set({});
-        RulesCheckResult result;
-
-        accumulate_level(result.level, pkg.rules.check(RuleID::PACKAGE_CHECKS, pkg).level);
-
-        auto &rule_cl = dynamic_cast<RuleClearancePackage &>(*pkg.rules.get_rule_nc(RuleID::CLEARANCE_PACKAGE));
-        rule_cl.clearance_silkscreen_cu = 0.2_mm;
-        rule_cl.clearance_silkscreen_pkg = 0.2_mm;
-        const auto r = pkg.rules.check(RuleID::CLEARANCE_PACKAGE, pkg);
-        accumulate_level(result.level, r.level);
-
-        return result;
+    if (sym.text_placements.size() == 0) {
+        CanvasCairo2 ca;
+        ca.load(sym);
+        const std::string img_filename = "sym_" + static_cast<std::string>(sym.uuid) + ".png";
+        ca.get_image_surface(1, 1.25_mm)->write_to_png(Glib::build_filename(images_dir, img_filename));
+        ofs << "![Symbol](" << images_prefix << img_filename << ")\n";
     }
-
-    default:
-        return {};
+    else {
+        ofs << "| Angle | Normal | Mirrored |\n";
+        ofs << "| --- | --- | --- |\n";
+        for (const auto angle : {0, 90, 180, 270}) {
+            ofs << "| " << angle << "° ";
+            for (const auto mirror : {false, true}) {
+                Placement pl;
+                pl.set_angle_deg(angle);
+                pl.mirror = mirror;
+                sym.apply_placement(pl);
+                CanvasCairo2 ca;
+                ca.load(sym, pl);
+                const std::string img_filename = "sym_" + static_cast<std::string>(sym.uuid) + "_"
+                                                 + (mirror ? "m" : "n") + std::to_string(angle) + ".png";
+                ca.get_image_surface(1, 1.25_mm)->write_to_png(Glib::build_filename(images_dir, img_filename));
+                ofs << "| ![Symbol](" << images_prefix << img_filename << ") ";
+            }
+            ofs << "\n";
+        }
+    }
+    sym.apply_placement(Placement());
+    if (sym.can_expand) {
+        ofs << "<details>\n<summary>Expanded by 5</summary>\n\n";
+        sym.apply_expand(pool_sym, 5);
+        CanvasCairo2 ca;
+        ca.load(sym);
+        const std::string img_filename = "sym_" + static_cast<std::string>(sym.uuid) + "_expanded.png";
+        ca.get_image_surface(1, 1.25_mm)->write_to_png(Glib::build_filename(images_dir, img_filename));
+        ofs << "![Symbol](" << images_prefix << img_filename << ")\n";
+        ofs << "</details>\n\n";
     }
 }
 
-int main(int c_argc, char *c_argv[])
+int Reviewer::parse_options(int c_argc, char *c_argv[])
 {
-    Gio::init();
-    PoolManager::init();
-    git_libgit2_init();
-
     Glib::OptionContext options;
     options.set_summary("horizon pr review");
     options.set_help_enabled();
 
     Glib::OptionGroup group("pr-review", "pr-review");
 
-    std::string output_filename;
     Glib::OptionEntry entry;
     entry.set_long_name("output");
     entry.set_short_name('o');
     entry.set_description("output filename");
     group.add_entry_filename(entry, output_filename);
 
-    bool do_pool_update = false;
     Glib::OptionEntry entry_pool_update;
     entry_pool_update.set_long_name("pool-update");
     entry_pool_update.set_short_name('u');
     entry_pool_update.set_description("update pool before generating review");
     group.add_entry(entry_pool_update, do_pool_update);
 
-    std::string images_dir;
     Glib::OptionEntry entry_images_dir;
     entry_images_dir.set_long_name("img-dir");
     entry_images_dir.set_short_name('i');
     entry_images_dir.set_description("images directory");
     group.add_entry_filename(entry_images_dir, images_dir);
 
-    std::string images_prefix;
     Glib::OptionEntry entry_images_prefix;
     entry_images_prefix.set_long_name("img-prefix");
     entry_images_prefix.set_short_name('p');
     entry_images_prefix.set_description("images prefix");
     group.add_entry_filename(entry_images_prefix, images_prefix);
 
+    Glib::OptionEntry entry_labels_filename;
+    entry_labels_filename.set_long_name("labels-file");
+    entry_labels_filename.set_short_name('l');
+    entry_labels_filename.set_description("labels filename");
+    group.add_entry_filename(entry_labels_filename, labels_filename);
+
 
     std::vector<std::string> filenames;
     Glib::OptionEntry entry_f;
@@ -231,13 +285,21 @@ int main(int c_argc, char *c_argv[])
         std::cerr << "image directory not specified" << std::endl;
         return 1;
     }
+    pool_base_path = filenames.at(0);
+    return 0;
+}
 
-    auto ofs = make_ofstream(output_filename);
+int Reviewer::main(int c_argc, char *c_argv[])
+{
+    if (auto r = parse_options(c_argc, c_argv))
+        return r;
+
+
+    ofs = make_ofstream(output_filename);
 
     ofs << "This review is brought to you by the Horizon EDA Poolbot commit [" << Version::commit_hash
         << "](https://github.com/horizon-eda/horizon/commit/" << Version::commit_hash << ").\n\n";
 
-    auto pool_base_path = filenames.at(0);
     if (do_pool_update) {
         std::list<std::pair<std::string, std::string>> errors;
         pool_update(
@@ -261,7 +323,7 @@ int main(int c_argc, char *c_argv[])
     }
 
 
-    Pool pool(pool_base_path);
+    pool.emplace(pool_base_path);
 
 
     autofree_ptr<git_repository> repo(git_repository_free);
@@ -286,15 +348,15 @@ int main(int c_argc, char *c_argv[])
         throw std::runtime_error("error finding master tree");
     }
 
-    pool.db.execute("CREATE TEMP TABLE 'git_files' ('git_filename' TEXT NOT NULL, 'status' INT NOT NULL);");
-    pool.db.execute("BEGIN");
+    pool->db.execute("CREATE TEMP TABLE 'git_files' ('git_filename' TEXT NOT NULL, 'status' INT NOT NULL);");
+    pool->db.execute("BEGIN");
     {
         autofree_ptr<git_diff> diff(git_diff_free);
         git_diff_tree_to_workdir_with_index(&diff.ptr, repo, tree_master, nullptr);
-        git_diff_foreach(diff, &diff_file_cb_c, nullptr, nullptr, nullptr, &pool.db);
+        git_diff_foreach(diff, &diff_file_cb_c, nullptr, nullptr, nullptr, &pool->db);
     }
-    pool.db.execute("COMMIT");
-    pool.db.execute(
+    pool->db.execute("COMMIT");
+    pool->db.execute(
             "CREATE TEMP VIEW git_files_view AS "
             "SELECT type, uuid, name, filename, status FROM git_files INNER JOIN "
             "(SELECT type, uuid, name, filename FROM all_items_view UNION ALL SELECT DISTINCT 'model_3d' AS type, "
@@ -302,19 +364,21 @@ int main(int c_argc, char *c_argv[])
             "ON filename=git_filename");
 
     {
+        RulesCheckErrorLevel overall_result = RulesCheckErrorLevel::PASS;
         ofs << "# Items in this PR\n";
         ofs << "| State | Type | Name | Checks | Filename |\n";
         ofs << "| --- | --- | --- | --- | --- |\n";
-        SQLite::Query q(pool.db, "SELECT type, uuid, name, filename, status FROM git_files_view");
+        SQLite::Query q(pool->db, "SELECT type, uuid, name, filename, status FROM git_files_view");
         while (q.step()) {
-            auto type = object_type_lut.lookup(q.get<std::string>(0));
+            const auto type = q.get<ObjectType>(0);
             const auto name = q.get<std::string>(2);
             ofs << "|" << delta_to_string(static_cast<git_delta_t>(q.get<int>(4))) << " | "
                 << object_descriptions.at(type).name << " | " << name;
             if (needs_trim(name))
                 ofs << " " << whitespace_warning;
             ofs << " | ";
-            const auto r = check_item(pool, type, UUID(q.get<std::string>(1)));
+            const auto r = check_item(*pool, type, UUID(q.get<std::string>(1)));
+            accumulate_level(overall_result, r.level);
             switch (r.level) {
             case RulesCheckErrorLevel::WARN:
                 ofs << ":warning: ";
@@ -328,23 +392,40 @@ int main(int c_argc, char *c_argv[])
             default:;
             }
             ofs << rules_check_error_level_to_string(r.level);
-
             ofs << " | " << q.get<std::string>(3) << "\n";
         }
         ofs << "\n";
+        if (labels_filename.size()) {
+            std::string label;
+            switch (overall_result) {
+            case RulesCheckErrorLevel::PASS:
+                label = "pass";
+                break;
+            case RulesCheckErrorLevel::WARN:
+                label = "warn";
+                break;
+            case RulesCheckErrorLevel::FAIL:
+                label = "fail";
+                break;
+            default:;
+            }
+            if (label.size()) {
+                const json j = {"bot: " + label};
+                save_json_to_file(labels_filename, j);
+            }
+        }
     }
 
     {
-        SQLite::Query q(pool.db,
+        SQLite::Query q(pool->db,
                         "SELECT git_filename FROM git_files LEFT JOIN "
                         "(SELECT filename FROM all_items_view "
                         "UNION ALL SELECT DISTINCT model_filename as filename FROM models) "
                         "ON filename=git_filename WHERE filename is NULL");
-        bool first = true;
+        Once once;
         while (q.step()) {
-            if (first) {
+            if (once()) {
                 ofs << "# Non-items\n";
-                first = false;
             }
             ofs << " - " << q.get<std::string>(0) << "\n";
         }
@@ -352,15 +433,14 @@ int main(int c_argc, char *c_argv[])
     }
 
     {
-        SQLite::Query q(pool.db, "SELECT status, git_filename FROM git_files WHERE status != ?");
+        SQLite::Query q(pool->db, "SELECT status, git_filename FROM git_files WHERE status != ?");
         q.bind(1, static_cast<int>(GIT_DELTA_ADDED));
-        bool first = true;
+        Once once;
         while (q.step()) {
-            if (first) {
+            if (once()) {
                 ofs << "# Non-new files\n";
                 ofs << "| Status | File |\n";
                 ofs << "| --- | --- |\n";
-                first = false;
             }
             ofs << "| " << delta_to_string(static_cast<git_delta_t>(q.get<int>(0))) << " | " << q.get<std::string>(1)
                 << "|\n";
@@ -368,14 +448,14 @@ int main(int c_argc, char *c_argv[])
         ofs << "\n\n";
     }
 
-    pool.db.execute(
+    pool->db.execute(
             "CREATE TEMP VIEW top_parts AS "
             "SELECT git_files_view.uuid AS part_uuid FROM git_files_view "
             "LEFT JOIN parts ON git_files_view.uuid = parts.uuid "
             "LEFT JOIN git_files_view AS gfv ON gfv.uuid = parts.base "
             "WHERE (gfv.uuid IS NULL OR parts.base = '00000000-0000-0000-0000-000000000000') "
             "AND git_files_view.type = 'part'");
-    pool.db.execute(
+    pool->db.execute(
             "CREATE TEMP VIEW parts_tree AS "
             "WITH RECURSIVE where_used(typex, uuidx, level, path) AS ( SELECT 'part', part_uuid, 0, "
             "part_uuid from top_parts UNION "
@@ -391,7 +471,7 @@ int main(int c_argc, char *c_argv[])
             "LEFT JOIN all_items_view "
             "ON where_used.typex = all_items_view.type "
             "AND where_used.uuidx = all_items_view.uuid");
-    pool.db.execute(
+    pool->db.execute(
             "CREATE TEMP VIEW derived_parts_tree AS "
             "WITH RECURSIVE where_used(uuidx, level, root) AS ( SELECT part_uuid, 0, part_uuid "
             "FROM top_parts UNION "
@@ -405,7 +485,7 @@ int main(int c_argc, char *c_argv[])
             "FROM where_used "
             "LEFT JOIN parts ON where_used.uuidx = parts.uuid "
             "ORDER BY root, level");
-    pool.db.execute(
+    pool->db.execute(
             "CREATE TEMP VIEW all_parts_tree AS "
             "SELECT * FROM ("
             "SELECT * FROM parts_tree "
@@ -419,9 +499,9 @@ int main(int c_argc, char *c_argv[])
     {
         ofs << "# Parts overview (excluding derived)\n";
         ofs << "Bold items are from this PR\n";
-        SQLite::Query q(pool.db, "SELECT * FROM all_parts_tree");
+        SQLite::Query q(pool->db, "SELECT * FROM all_parts_tree");
         while (q.step()) {
-            const auto type = object_type_lut.lookup(q.get<std::string>(0));
+            const auto type = q.get<ObjectType>(0);
             const auto name = q.get<std::string>(1);
             const auto level = q.get<int>(2);
             const auto from_pr = q.get<int>(3);
@@ -432,8 +512,7 @@ int main(int c_argc, char *c_argv[])
         }
     }
     {
-        bool first = true;
-        SQLite::Query q(pool.db,
+        SQLite::Query q(pool->db,
                         "SELECT git_files_view.type, git_files_view.name FROM git_files_view "
                         "LEFT JOIN all_parts_tree ON git_files_view.uuid = all_parts_tree.uuid "
                         "AND git_files_view.type = all_parts_tree.type "
@@ -441,12 +520,12 @@ int main(int c_argc, char *c_argv[])
                         "AND git_files_view.type = 'part' "
                         "WHERE all_parts_tree.uuid IS NULL AND derived_parts_tree.uuid IS NULL "
                         "AND git_files_view.type != 'model_3d'");
+        Once once;
         while (q.step()) {
-            if (first)
+            if (once())
                 ofs << "# Items not associated with any part\n";
-            first = false;
-            auto type = object_type_lut.lookup(q.get<std::string>(0));
 
+            const auto type = q.get<ObjectType>(0);
             ofs << " - " << object_descriptions.at(type).name << " " << q.get<std::string>(1) << "\n";
         }
     }
@@ -455,7 +534,7 @@ int main(int c_argc, char *c_argv[])
     {
         int n_derived = 0;
         {
-            SQLite::Query q(pool.db,
+            SQLite::Query q(pool->db,
                             "SELECT COUNT(*) FROM git_files_view "
                             "LEFT JOIN parts ON git_files_view.uuid = parts.uuid AND git_files_view.type = 'part' "
                             "WHERE parts.base != '00000000-0000-0000-0000-000000000000'");
@@ -468,7 +547,7 @@ int main(int c_argc, char *c_argv[])
             {
                 ofs << "# Derived parts\n";
                 ofs << "Bold items are from this PR\n";
-                SQLite::Query q(pool.db, "SELECT * FROM derived_parts_tree");
+                SQLite::Query q(pool->db, "SELECT * FROM derived_parts_tree");
                 while (q.step()) {
                     const auto name = q.get<std::string>(0);
                     const auto level = q.get<int>(1);
@@ -484,9 +563,9 @@ int main(int c_argc, char *c_argv[])
                 ofs << "Values in italic are inherited\n\n";
                 ofs << "| MPN | Value | Manufacturer | Datasheet | Description | Tags |\n";
                 ofs << "| --- | ----- | ------------ | --------- | ----------- | ---- |\n";
-                SQLite::Query q(pool.db, "SELECT uuid FROM derived_parts_tree");
+                SQLite::Query q(pool->db, "SELECT uuid FROM derived_parts_tree");
                 while (q.step()) {
-                    const auto &part = *pool.get_part(q.get<std::string>(0));
+                    const auto &part = *pool->get_part(q.get<std::string>(0));
 
                     auto get_attr = [&part](Part::Attribute attr) {
                         return surround_if("*", "*", part.get_attribute(attr), part.attributes.at(attr).first);
@@ -498,7 +577,7 @@ int main(int c_argc, char *c_argv[])
                     ofs << "| " << get_attr(Part::Attribute::DATASHEET);
                     ofs << "| " << get_attr(Part::Attribute::DESCRIPTION);
                     {
-                        SQLite::Query qtags(pool.db, "SELECT tags FROM tags_view WHERE type = 'part' AND uuid = ?");
+                        SQLite::Query qtags(pool->db, "SELECT tags FROM tags_view WHERE type = 'part' AND uuid = ?");
                         qtags.bind(1, part.uuid);
                         if (qtags.step()) {
                             ofs << "| " << surround_if("*", "*", qtags.get<std::string>(0), part.inherit_tags);
@@ -512,15 +591,17 @@ int main(int c_argc, char *c_argv[])
     }
 
     ofs << "# Details\n";
-    ofs << "## Parts\n";
     {
-        SQLite::Query q(pool.db, "SELECT uuid FROM derived_parts_tree");
+        SQLite::Query q(pool->db, "SELECT uuid FROM derived_parts_tree");
+        Once once;
         while (q.step()) {
-            const auto &part = *pool.get_part(q.get<std::string>(0));
+            if (once())
+                ofs << "## Parts\n";
+            const auto &part = *pool->get_part(q.get<std::string>(0));
             ofs << "### " << part.get_MPN() << "\n";
             if (part.base)
                 ofs << "Inerhits from " << part.base->get_MPN() << "\n\n";
-            print_rules_check_result(ofs, check_part(part));
+            print_rules_check_result(check_part(part));
 
             ofs << "| Attribute | Value |\n";
             ofs << "| --- | --- |\n";
@@ -537,7 +618,7 @@ int main(int c_argc, char *c_argv[])
                 if (needs_trim(val))
                     ofs << " " << whitespace_warning;
                 if (attr == Part::Attribute::MANUFACTURER) {
-                    ofs << " (" << count_manufactuer(pool, val) << " other parts)";
+                    ofs << " (" << count_manufactuer(val) << " other parts)";
                 }
                 if (part.attributes.at(attr).first) {
                     ofs << " (inherited)";
@@ -545,11 +626,14 @@ int main(int c_argc, char *c_argv[])
                 ofs << "\n";
             }
             {
-                SQLite::Query qtags(pool.db, "SELECT tags FROM tags_view WHERE type = 'part' AND uuid = ?");
+                SQLite::Query qtags(pool->db, "SELECT tags FROM tags_view WHERE type = 'part' AND uuid = ?");
                 qtags.bind(1, part.uuid);
                 if (qtags.step()) {
-                    ofs << "|Tags | " << qtags.get<std::string>(0) << "\n";
+                    ofs << "|Tags | " << qtags.get<std::string>(0);
                 }
+                if (part.inherit_tags)
+                    ofs << " (inherited)";
+                ofs << "\n";
             }
             ofs << "\n\n";
 
@@ -558,7 +642,27 @@ int main(int c_argc, char *c_argv[])
                 for (const auto &[uu, MPN] : part.orderable_MPNs) {
                     ofs << " - " << MPN << "\n";
                 }
-                ofs << "\n";
+                ofs << "\n\n";
+            }
+
+            if (std::count_if(part.flags.begin(), part.flags.end(),
+                              [](const auto &x) { return x.second != Part::FlagState::CLEAR; })) {
+                static const std::map<Part::Flag, std::string> flag_names = {
+                        {Part::Flag::BASE_PART, "Base part"},
+                        {Part::Flag::EXCLUDE_BOM, "Exclude from BOM"},
+                        {Part::Flag::EXCLUDE_PNP, "Exclude from Pick&Place"},
+                };
+                ofs << "Flags\n";
+                for (const auto &[fl, name] : flag_names) {
+                    if (part.get_flag(fl)) {
+                        ofs << " - " << name;
+                        if (part.flags.at(fl) == Part::FlagState::INHERIT) {
+                            ofs << " (inherited)";
+                        }
+                        ofs << "\n";
+                    }
+                }
+                ofs << "\n\n";
             }
 
             std::set<std::pair<UUID, UUID>> all_pins;
@@ -601,22 +705,25 @@ int main(int c_argc, char *c_argv[])
             }
         }
     }
-    ofs << "## Entities\n";
+
     {
-        SQLite::Query q(pool.db, "SELECT uuid from git_files_view where type = 'entity'");
+        SQLite::Query q(pool->db, "SELECT uuid from git_files_view where type = 'entity'");
+        Once once;
         while (q.step()) {
-            const auto &entity = *pool.get_entity(q.get<std::string>(0));
+            if (once())
+                ofs << "## Entities\n";
+            const auto &entity = *pool->get_entity(q.get<std::string>(0));
             ofs << "### " << entity.name << "\n";
 
-            print_rules_check_result(ofs, check_entity(entity));
+            print_rules_check_result(check_entity(entity));
 
             ofs << "| Attribute | Value |\n";
             ofs << "| --- | --- |\n";
-            ofs << "|Manufacturer | " << entity.manufacturer << " (" << count_manufactuer(pool, entity.manufacturer)
+            ofs << "|Manufacturer | " << entity.manufacturer << " (" << count_manufactuer(entity.manufacturer)
                 << " other parts)\n";
             ofs << "|Prefix | " << entity.prefix << "\n";
             {
-                SQLite::Query qtags(pool.db, "SELECT tags FROM tags_view WHERE type = 'entity' AND uuid = ?");
+                SQLite::Query qtags(pool->db, "SELECT tags FROM tags_view WHERE type = 'entity' AND uuid = ?");
                 qtags.bind(1, entity.uuid);
                 if (qtags.step()) {
                     ofs << "|Tags | " << qtags.get<std::string>(0) << "\n";
@@ -643,18 +750,20 @@ int main(int c_argc, char *c_argv[])
             }
         }
     }
-    ofs << "## Units\n";
     {
-        SQLite::Query q(pool.db, "SELECT DISTINCT uuid from git_files_view where type = 'unit'");
+        SQLite::Query q(pool->db, "SELECT DISTINCT uuid from git_files_view where type = 'unit'");
+        Once once;
         while (q.step()) {
-            const auto &unit = *pool.get_unit(q.get<std::string>(0));
+            if (once())
+                ofs << "## Units\n";
+            const auto &unit = *pool->get_unit(q.get<std::string>(0));
             ofs << "### " << unit.name << "\n";
 
-            print_rules_check_result(ofs, check_unit(unit));
+            print_rules_check_result(check_unit(unit));
 
             ofs << "| Attribute | Value |\n";
             ofs << "| --- | --- |\n";
-            ofs << "|Manufacturer | " << unit.manufacturer << " (" << count_manufactuer(pool, unit.manufacturer)
+            ofs << "|Manufacturer | " << unit.manufacturer << " (" << count_manufactuer(unit.manufacturer)
                 << " other parts)\n";
 
             ofs << "\n\n";
@@ -688,69 +797,11 @@ int main(int c_argc, char *c_argv[])
 
             {
                 bool has_sym = false;
-                SQLite::Query q_symbol(pool.db, "SELECT uuid FROM symbols WHERE unit = ?");
+                SQLite::Query q_symbol(pool->db, "SELECT uuid FROM symbols WHERE unit = ?");
                 q_symbol.bind(1, unit.uuid);
                 while (q_symbol.step()) {
                     has_sym = true;
-                    const auto &pool_sym = *pool.get_symbol(q_symbol.get<std::string>(0));
-                    Symbol sym = pool_sym;
-                    sym.expand();
-                    sym.apply_placement(Placement());
-                    ofs << "#### Symbol: " << sym.name << "\n";
-                    if (sym.can_expand) {
-                        ofs << "Is expandable\n\n";
-                    }
-                    {
-                        auto r = sym.rules.check(RuleID::SYMBOL_CHECKS, sym);
-                        print_rules_check_result(ofs, r);
-                        ofs << "\n";
-                    }
-                    for (auto &[uu, txt] : sym.texts) {
-                        if (txt.text == "$VALUE") {
-                            txt.text += "\nGroup\nTag";
-                        }
-                    }
-
-                    if (sym.text_placements.size() == 0) {
-                        CanvasCairo2 ca;
-                        ca.load(sym);
-                        const std::string img_filename = "sym_" + static_cast<std::string>(sym.uuid) + ".png";
-                        ca.get_image_surface(1, 1.25_mm)->write_to_png(Glib::build_filename(images_dir, img_filename));
-                        ofs << "![Symbol](" << images_prefix << img_filename << ")\n";
-                    }
-                    else {
-                        ofs << "| Angle | Normal | Mirrored |\n";
-                        ofs << "| --- | --- | --- |\n";
-                        for (const auto angle : {0, 90, 180, 270}) {
-                            ofs << "| " << angle << "° ";
-                            for (const auto mirror : {false, true}) {
-                                Placement pl;
-                                pl.set_angle_deg(angle);
-                                pl.mirror = mirror;
-                                sym.apply_placement(pl);
-                                CanvasCairo2 ca;
-                                ca.load(sym, pl);
-                                const std::string img_filename = "sym_" + static_cast<std::string>(sym.uuid) + "_"
-                                                                 + (mirror ? "m" : "n") + std::to_string(angle)
-                                                                 + ".png";
-                                ca.get_image_surface(1, 1.25_mm)
-                                        ->write_to_png(Glib::build_filename(images_dir, img_filename));
-                                ofs << "| ![Symbol](" << images_prefix << img_filename << ") ";
-                            }
-                            ofs << "\n";
-                        }
-                    }
-                    sym.apply_placement(Placement());
-                    if (sym.can_expand) {
-                        ofs << "<details>\n<summary>Expanded by 5</summary>\n\n";
-                        sym.apply_expand(pool_sym, 5);
-                        CanvasCairo2 ca;
-                        ca.load(sym);
-                        const std::string img_filename = "sym_" + static_cast<std::string>(sym.uuid) + "_expanded.png";
-                        ca.get_image_surface(1, 1.25_mm)->write_to_png(Glib::build_filename(images_dir, img_filename));
-                        ofs << "![Symbol](" << images_prefix << img_filename << ")\n";
-                        ofs << "</details>\n\n";
-                    }
+                    review_symbol(q_symbol.get<std::string>(0), "#### Symbol:");
                 }
 
                 if (!has_sym) {
@@ -761,21 +812,38 @@ int main(int c_argc, char *c_argv[])
     }
 
     {
-        ofs << "## Packages\n";
-        SQLite::Query q(pool.db, "SELECT DISTINCT uuid from git_files_view where type = 'package'");
+        SQLite::Query q(pool->db,
+                        "SELECT git_files_view.uuid FROM git_files_view "
+                        "LEFT JOIN symbols ON (symbols.uuid = git_files_view.uuid AND git_files_view.type = 'symbol') "
+                        "LEFT JOIN git_files_view AS gfv2 ON (symbols.unit = gfv2.uuid AND gfv2.type = 'unit') "
+                        "WHERE git_files_view.type = 'symbol' AND gfv2.uuid IS NULL");
+        Once once;
         while (q.step()) {
-            Package pkg = *pool.get_package(q.get<std::string>(0));
+            if (once()) {
+                ofs << "## Symbols\n";
+            }
+            review_symbol(q.get<std::string>(0), "###");
+        }
+    }
+
+    {
+        SQLite::Query q(pool->db, "SELECT DISTINCT uuid from git_files_view where type = 'package'");
+        Once once;
+        while (q.step()) {
+            if (once())
+                ofs << "## Packages\n";
+            Package pkg = *pool->get_package(q.get<std::string>(0));
             pkg.expand();
             ofs << "### " << pkg.name << "\n";
             ofs << "| Attribute | Value |\n";
             ofs << "| --- | --- |\n";
-            ofs << "|Manufacturer | " << pkg.manufacturer << " (" << count_manufactuer(pool, pkg.manufacturer)
+            ofs << "|Manufacturer | " << pkg.manufacturer << " (" << count_manufactuer(pkg.manufacturer)
                 << " other parts)\n";
             if (pkg.alternate_for) {
                 ofs << "|Alt. for | " << pkg.alternate_for->name << "\n";
             }
             {
-                SQLite::Query qtags(pool.db, "SELECT tags FROM tags_view WHERE type = 'package' AND uuid = ?");
+                SQLite::Query qtags(pool->db, "SELECT tags FROM tags_view WHERE type = 'package' AND uuid = ?");
                 qtags.bind(1, pkg.uuid);
                 if (qtags.step()) {
                     ofs << "|Tags | " << qtags.get<std::string>(0) << "\n";
@@ -788,7 +856,7 @@ int main(int c_argc, char *c_argv[])
             }
             {
                 auto r = pkg.rules.check(RuleID::PACKAGE_CHECKS, pkg);
-                print_rules_check_result(ofs, r, "Package checks");
+                print_rules_check_result(r, "Package checks");
                 ofs << "\n";
             }
             {
@@ -796,7 +864,7 @@ int main(int c_argc, char *c_argv[])
                 rule_cl.clearance_silkscreen_cu = 0.2_mm;
                 rule_cl.clearance_silkscreen_pkg = 0.2_mm;
                 auto r = pkg.rules.check(RuleID::CLEARANCE_PACKAGE, pkg);
-                print_rules_check_result(ofs, r, "Clearance checks");
+                print_rules_check_result(r, "Clearance checks");
                 ofs << "\n";
             }
             for (auto &[uu, txt] : pkg.texts) {
@@ -865,10 +933,10 @@ int main(int c_argc, char *c_argv[])
                 fake_package.package = pkg;
                 fake_package.pool_package = &pkg;
 
-                Image3DExporter ex(fake_board, pool, 1024, 1024);
+                Image3DExporter ex(fake_board, *pool, 1024, 1024);
                 ex.view_all();
-                ex.show_models = false;
-                ex.projection = Canvas3DBase::Projection::ORTHO;
+                ex.set_show_models(false);
+                ex.set_projection(Canvas3DBase::Projection::ORTHO);
 
                 ofs << "#### Without model \n";
                 ofs << "| Top | Bottom |\n";
@@ -883,14 +951,14 @@ int main(int c_argc, char *c_argv[])
                 {
                     const std::string img_filename =
                             "pkg_" + static_cast<std::string>(pkg.uuid) + "_3d_bottom_no_model.png";
-                    ex.cam_elevation = -89.99;
-                    ex.cam_azimuth = 90;
+                    ex.set_cam_elevation(-89.99);
+                    ex.set_cam_azimuth(90);
                     ex.render_to_surface()->write_to_png(Glib::build_filename(images_dir, img_filename));
                     ofs << "| ![3D](" << images_prefix << img_filename << ") ";
                 }
 
                 ofs << "\n\n";
-                ex.show_models = true;
+                ex.set_show_models(true);
                 ex.view_all();
                 for (const auto &[model_uu, model] : pkg.models) {
                     fake_package.model = model_uu;
@@ -909,8 +977,8 @@ int main(int c_argc, char *c_argv[])
                     {
                         const std::string img_filename = "pkg_" + static_cast<std::string>(pkg.uuid) + "_3d_bottom_"
                                                          + static_cast<std::string>(model_uu) + ".png";
-                        ex.cam_elevation = -89.99;
-                        ex.cam_azimuth = 90;
+                        ex.set_cam_elevation(-89.99);
+                        ex.set_cam_azimuth(90);
                         ex.render_to_surface()->write_to_png(Glib::build_filename(images_dir, img_filename));
                         ofs << "| ![3D](" << images_prefix << img_filename << ") ";
                     }
@@ -920,10 +988,10 @@ int main(int c_argc, char *c_argv[])
                     ofs << "| South  | East | North | West |\n";
                     ofs << "| --- | --- | --- | --- |\n";
                     ex.view_all();
-                    ex.cam_elevation = 45;
-                    ex.projection = Canvas3DBase::Projection::PERSP;
+                    ex.set_cam_elevation(45);
+                    ex.set_projection(Canvas3DBase::Projection::PERSP);
                     for (const int az : {270, 0, 90, 180}) {
-                        ex.cam_azimuth = az;
+                        ex.set_cam_azimuth(az);
                         const std::string img_filename = "pkg_" + static_cast<std::string>(pkg.uuid) + "_3d_"
                                                          + std::to_string(az) + "_" + static_cast<std::string>(model_uu)
                                                          + ".png";
@@ -973,6 +1041,19 @@ int main(int c_argc, char *c_argv[])
             }
         }
     }
+    return 0;
+}
+
+
+int main(int c_argc, char *c_argv[])
+{
+    Gio::init();
+    PoolManager::init();
+    git_libgit2_init();
+
+    Reviewer rev;
+    return rev.main(c_argc, c_argv);
+
 
     return 0;
 }
