@@ -9,6 +9,7 @@
 #include "logger/logger.hpp"
 #include "imp/in_tool_action_catalog.hpp"
 #include "core/tool_id.hpp"
+#include "imp/actions.hpp"
 
 namespace horizon {
 
@@ -29,6 +30,11 @@ static const LutEnumStr<Appearance::CursorSize> cursor_size_lut = {
         {"full", Appearance::CursorSize::FULL},
 };
 
+static const LutEnumStr<Preferences::StockInfoProviderSel> stock_info_provider_lut = {
+        {"none", Preferences::StockInfoProviderSel::NONE},
+        {"partinfo", Preferences::StockInfoProviderSel::PARTINFO},
+        {"digikey", Preferences::StockInfoProviderSel::DIGIKEY}};
+
 #ifdef G_OS_WIN32
 static const bool capture_output_default = true;
 #else
@@ -94,6 +100,7 @@ static const LutEnumStr<ColorP> colorp_lut = {
         COLORP_LUT_ITEM(SHADOW),
         COLORP_LUT_ITEM(CONNECTION_LINE),
         COLORP_LUT_ITEM(NOPOPULATE_X),
+        COLORP_LUT_ITEM(PROJECTION),
 };
 
 json CanvasPreferences::serialize() const
@@ -208,6 +215,7 @@ json ZoomPreferences::serialize() const
     j["smooth_zoom_3d"] = smooth_zoom_3d;
     j["touchpad_pan"] = touchpad_pan;
     j["zoom_factor"] = zoom_factor;
+    j["keyboard_zoom_to_cursor"] = keyboard_zoom_to_cursor;
     return j;
 }
 
@@ -217,6 +225,7 @@ void ZoomPreferences::load_from_json(const json &j)
     smooth_zoom_3d = j.value("smooth_zoom_3d", false);
     touchpad_pan = j.value("touchpad_pan", false);
     zoom_factor = j.value("zoom_factor", 50);
+    keyboard_zoom_to_cursor = j.value("keyboard_zoom_to_cursor", false);
 }
 
 json KeySequencesPreferences::serialize() const
@@ -347,7 +356,6 @@ void InToolKeySequencesPreferences::append_from_json(const json &j)
 
 void PartInfoPreferences::load_from_json(const json &j)
 {
-    enable = j.at("enable");
     url = j.at("url");
     preferred_distributor = j.at("preferred_distributor");
     ignore_moq_gt_1 = j.at("ignore_moq_gt_1");
@@ -357,7 +365,7 @@ void PartInfoPreferences::load_from_json(const json &j)
 json PartInfoPreferences::serialize() const
 {
     json j;
-    j["enable"] = enable;
+    j["enable"] = false;
     j["url"] = url;
     j["preferred_distributor"] = preferred_distributor;
     j["ignore_moq_gt_1"] = ignore_moq_gt_1;
@@ -365,9 +373,25 @@ json PartInfoPreferences::serialize() const
     return j;
 }
 
-bool PartInfoPreferences::is_enabled() const
+void DigiKeyApiPreferences::load_from_json(const json &j)
 {
-    return enable && preferred_distributor.size();
+    client_id = j.at("client_id");
+    client_secret = j.at("client_secret");
+    currency = j.value("currency", "EUR");
+    site = j.value("site", "DE");
+    max_price_breaks = j.value("max_price_breaks", 3);
+}
+
+json DigiKeyApiPreferences::serialize() const
+{
+    json j;
+    j["enable"] = false;
+    j["client_id"] = client_id;
+    j["client_secret"] = client_secret;
+    j["site"] = site;
+    j["currency"] = currency;
+    j["max_price_breaks"] = max_price_breaks;
+    return j;
 }
 
 json ActionBarPreferences::serialize() const
@@ -390,14 +414,18 @@ json MousePreferences::serialize() const
 {
     json j;
     j["switch_layers"] = switch_layers;
+    j["switch_sheets"] = switch_sheets;
     j["drag_polygon_edges"] = drag_polygon_edges;
+    j["drag_to_move"] = drag_to_move;
     return j;
 }
 
 void MousePreferences::load_from_json(const json &j)
 {
     switch_layers = j.value("switch_layers", true);
+    switch_sheets = j.value("switch_sheets", true);
     drag_polygon_edges = j.value("drag_polygon_edges", true);
+    drag_to_move = j.value("drag_to_move", true);
 }
 
 json Preferences::serialize() const
@@ -411,7 +439,9 @@ json Preferences::serialize() const
     j["board"] = board.serialize();
     j["zoom"] = zoom.serialize();
     j["capture_output"] = capture_output;
+    j["stock_info_provider"] = stock_info_provider_lut.lookup_reverse(stock_info_provider);
     j["partinfo"] = partinfo.serialize();
+    j["digikey_api"] = digikey_api.serialize();
     j["action_bar"] = action_bar.serialize();
     j["mouse"] = mouse.serialize();
     j["show_pull_request_tools"] = show_pull_request_tools;
@@ -452,8 +482,11 @@ void Preferences::load_from_json(const json &j)
     capture_output = j.value("capture_output", capture_output_default);
     show_pull_request_tools = j.value("show_pull_request_tools", false);
     hud_debug = j.value("hud_debug", false);
+    stock_info_provider = stock_info_provider_lut.lookup(j.value("stock_info_provider", "none"));
     if (j.count("partinfo"))
         partinfo.load_from_json(j.at("partinfo"));
+    if (j.count("digikey_api"))
+        digikey_api.load_from_json(j.at("digikey_api"));
 }
 
 void Preferences::load()
