@@ -8,8 +8,6 @@
 namespace horizon {
 using json = nlohmann::json;
 
-enum class ActionID;
-enum class ToolID;
 enum class InToolActionID;
 
 class CanvasPreferences {
@@ -67,6 +65,7 @@ public:
     bool smooth_zoom_3d = false;
     bool touchpad_pan = false;
     float zoom_factor = 50;
+    bool keyboard_zoom_to_cursor = false;
 
     void load_from_json(const json &j);
     json serialize() const;
@@ -74,13 +73,23 @@ public:
 
 class PartInfoPreferences {
 public:
-    bool enable = false;
     std::string url = "https://dev-partinfo.kitspace.org/graphql";
     std::string preferred_distributor;
     bool ignore_moq_gt_1 = true;
     unsigned int max_price_breaks = 3;
     unsigned int cache_days = 5;
-    bool is_enabled() const;
+
+    void load_from_json(const json &j);
+    json serialize() const;
+};
+
+class DigiKeyApiPreferences {
+public:
+    std::string client_id;
+    std::string client_secret;
+    std::string site = "DE";
+    std::string currency = "EUR";
+    unsigned int max_price_breaks = 3;
 
     void load_from_json(const json &j);
     json serialize() const;
@@ -99,7 +108,9 @@ public:
 class MousePreferences {
 public:
     bool switch_layers = true;
+    bool switch_sheets = true;
     bool drag_polygon_edges = true;
+    bool drag_to_move = true;
 
     void load_from_json(const json &j);
     json serialize() const;
@@ -123,7 +134,12 @@ public:
     KeySequencesPreferences key_sequences;
     ZoomPreferences zoom;
     bool capture_output = false;
+
+    enum class StockInfoProviderSel { NONE, PARTINFO, DIGIKEY };
+    StockInfoProviderSel stock_info_provider = StockInfoProviderSel::NONE;
+
     PartInfoPreferences partinfo;
+    DigiKeyApiPreferences digikey_api;
     ActionBarPreferences action_bar;
     InToolKeySequencesPreferences in_tool_key_sequences;
     MousePreferences mouse;
