@@ -10,10 +10,6 @@ ProjectWrapper::ProjectWrapper(const std::string &path) : project(horizon::Proje
 static PyObject *PyProject_open_top_schematic(PyObject *pself)
 {
     auto self = reinterpret_cast<PyProject *>(pself);
-    if (horizon::PoolManager::get().get_by_uuid(self->project->project.pool_uuid) == nullptr) {
-        PyErr_SetString(PyExc_FileNotFoundError, "pool not found");
-        return NULL;
-    }
     SchematicWrapper *schematic = nullptr;
     try {
         auto top_block = self->project->project.get_top_block();
@@ -36,10 +32,6 @@ static PyObject *PyProject_open_top_schematic(PyObject *pself)
 static PyObject *PyProject_open_board(PyObject *pself)
 {
     auto self = reinterpret_cast<PyProject *>(pself);
-    if (horizon::PoolManager::get().get_by_uuid(self->project->project.pool_uuid) == nullptr) {
-        PyErr_SetString(PyExc_FileNotFoundError, "pool not found");
-        return NULL;
-    }
     class BoardWrapper *board = nullptr;
     try {
         board = create_board_wrapper(self->project->project);
