@@ -1,7 +1,6 @@
 #include "pns_horizon_iface.hpp"
 #include "board/board.hpp"
 #include "board/board_layers.hpp"
-#include "board/via_padstack_provider.hpp"
 #include "canvas/canvas_gl.hpp"
 #include "clipper/clipper.hpp"
 #include "geometry/shape_simple.h"
@@ -10,8 +9,9 @@
 #include "router/pns_solid.h"
 #include "router/pns_topology.h"
 #include "router/pns_via.h"
-#include "util/util.hpp"
+#include "util/geom_util.hpp"
 #include "logger/logger.hpp"
+#include "pool/ipool.hpp"
 
 namespace PNS {
 
@@ -166,7 +166,10 @@ int PNS_HORIZON_RULE_RESOLVER::Clearance(const PNS::ITEM *aA, const PNS::ITEM *a
         auto layer = layers.Start();
 
         auto clearance = m_rules->get_clearance_copper_other(net, PNS_HORIZON_IFACE::layer_from_router(layer));
-        return clearance->get_clearance(pt, horizon::PatchType::BOARD_EDGE) + clearance->routing_offset;
+        int64_t routing_offset = clearance->routing_offset;
+        if (m_iface->get_override_routing_offset() >= 0)
+            routing_offset = m_iface->get_override_routing_offset();
+        return clearance->get_clearance(pt, horizon::PatchType::BOARD_EDGE) + routing_offset;
     }
 
     if ((parent_a && parent_a->pad && parent_a->pad->padstack.type == horizon::Padstack::Type::MECHANICAL)
@@ -188,7 +191,10 @@ int PNS_HORIZON_RULE_RESOLVER::Clearance(const PNS::ITEM *aA, const PNS::ITEM *a
         auto layer = layers.Start();
 
         auto clearance = m_rules->get_clearance_copper_other(net, PNS_HORIZON_IFACE::layer_from_router(layer));
-        return clearance->get_clearance(pt, horizon::PatchType::HOLE_NPTH) + clearance->routing_offset;
+        int64_t routing_offset = clearance->routing_offset;
+        if (m_iface->get_override_routing_offset() >= 0)
+            routing_offset = m_iface->get_override_routing_offset();
+        return clearance->get_clearance(pt, horizon::PatchType::HOLE_NPTH) + routing_offset;
     }
 
     if ((parent_a && parent_a->keepout) || (parent_b && parent_b->keepout)) { // one is keepout
@@ -196,12 +202,14 @@ int PNS_HORIZON_RULE_RESOLVER::Clearance(const PNS::ITEM *aA, const PNS::ITEM *a
         auto net = net_a ? net_a : net_b; // only one has net
         horizon::KeepoutContour keepout_contour;
         keepout_contour.keepout = a_is_keepout ? parent_a->keepout : parent_b->keepout;
-        ;
         keepout_contour.pkg = a_is_keepout ? parent_a->package : parent_b->package;
         for (const auto &rule : m_rules_keepout) {
             if (rule->enabled && rule->match.match(net) && rule->match_keepout.match(&keepout_contour)) {
                 auto cl = rule->get_clearance(a_is_keepout ? pt_b : pt_a);
-                return rule->routing_offset + cl;
+                int64_t routing_offset = rule->routing_offset;
+                if (m_iface->get_override_routing_offset() >= 0)
+                    routing_offset = m_iface->get_override_routing_offset();
+                return routing_offset + cl;
             }
         }
         return 0;
@@ -399,9 +407,9 @@ void PNS_HORIZON_IFACE::SetRules(const horizon::BoardRules *ru)
     rules = ru;
 }
 
-void PNS_HORIZON_IFACE::SetViaPadstackProvider(horizon::ViaPadstackProvider *v)
+void PNS_HORIZON_IFACE::SetPool(horizon::IPool *p)
 {
-    vpp = v;
+    pool = p;
 }
 
 int PNS_HORIZON_IFACE::get_net_code(const horizon::UUID &uu)
@@ -978,7 +986,7 @@ void PNS_HORIZON_IFACE::AddItem(PNS::ITEM *aItem)
         PNS::VIA *pvia = static_cast<PNS::VIA *>(aItem);
         auto uu = horizon::UUID::random();
         auto net = get_net_for_code(pvia->Net());
-        auto padstack = vpp->get_padstack(rules->get_via_padstack_uuid(net));
+        auto padstack = pool->get_padstack(rules->get_via_padstack_uuid(net));
         if (padstack) {
             auto ps = rules->get_via_parameter_set(net);
             auto via = &board->vias
@@ -1018,6 +1026,9 @@ void PNS_HORIZON_IFACE::Commit()
               || ju->connected_tracks.size())) {
             for (auto &it : ju->connected_connection_lines)
                 board->connection_lines.erase(it);
+            for (auto &[net, airwires] : board->airwires) {
+                airwires.remove_if([ju](const auto &aw) { return aw.from.junc == ju || aw.to.junc == ju; });
+            }
             board->junctions.erase(ju->uuid);
         }
     }
