@@ -8,6 +8,7 @@
 #include "logger/logger.hpp"
 #include "util/picture_load.hpp"
 #include "pool/ipool.hpp"
+#include "util/str_util.hpp"
 
 namespace horizon {
 
@@ -47,7 +48,7 @@ json Schematic::Annotation::serialize() const
     return j;
 }
 
-static const unsigned int app_version = 0;
+static const unsigned int app_version = 1;
 
 unsigned int Schematic::get_app_version()
 {
@@ -391,8 +392,7 @@ bool Schematic::place_bipole_on_line(Sheet *sheet, SchematicSymbol *sym)
             auto pin1_pos = sym->placement.transform(pin1.position);
             auto pin2_pos = sym->placement.transform(pin2.position);
             LineNet *line = nullptr;
-            for (auto it : sheet->net_lines) {
-                auto &li = it.second;
+            for (auto &[li_uu, li] : sheet->net_lines) {
                 if ((li.coord_on_line(pin1_pos) || li.from.get_position() == pin1_pos
                      || li.to.get_position() == pin1_pos)
                     && (li.coord_on_line(pin2_pos) || li.from.get_position() == pin2_pos
@@ -404,59 +404,33 @@ bool Schematic::place_bipole_on_line(Sheet *sheet, SchematicSymbol *sym)
             if (line) {
                 auto from = line->from;
                 auto to = line->to;
-                delete_net_line(sheet, line);
-                line = nullptr;
-                expand(true);
-                from.update_refs(*sheet);
-                to.update_refs(*sheet);
-
-                UUID from_net_segment = from.get_net_segment();
-                UUID to_net_segment = to.get_net_segment();
-                auto ns_info = sheet->analyze_net_segments(false);
-                Net *net_from = nullptr;
-                Net *net_to = nullptr;
-                if (ns_info.count(from_net_segment))
-                    net_from = ns_info.at(from_net_segment).net;
-                else if (from.is_bus_ripper())
-                    net_from = from.bus_ripper->bus_member->net;
-
-                if (ns_info.count(to_net_segment))
-                    net_to = ns_info.at(to_net_segment).net;
-                else if (to.is_bus_ripper())
-                    net_to = to.bus_ripper->bus_member->net;
-
-                // normal pin1-from pin2-to
-                // swapped pin1-to pin2-from
-                auto dst_normal = (pin1_pos - from.get_position()).mag_sq() + (pin2_pos - to.get_position()).mag_sq();
-                auto dst_swapped = (pin2_pos - from.get_position()).mag_sq() + (pin1_pos - to.get_position()).mag_sq();
+                const auto dst_normal =
+                        (pin1_pos - from.get_position()).mag_sq() + (pin2_pos - to.get_position()).mag_sq();
+                const auto dst_swapped =
+                        (pin2_pos - from.get_position()).mag_sq() + (pin1_pos - to.get_position()).mag_sq();
                 SymbolPin *pin_from = &pin1;
                 SymbolPin *pin_to = &pin2;
                 if (dst_swapped < dst_normal) {
                     std::swap(pin_from, pin_to);
                 }
-                auto connect_pin = [this, sheet, sym](SymbolPin &pin, const LineNet::Connection &conn, Net *net) {
-                    bool new_net = !net;
-                    if (!net)
-                        net = block->insert_net();
-
+                auto connect_pin = [sheet, sym, line](SymbolPin &pin, const LineNet::Connection &conn) {
                     auto uu = UUID::random();
                     auto &new_line = sheet->net_lines.emplace(uu, uu).first->second;
                     new_line.from = conn;
+                    if (conn.is_pin())
+                        conn.pin->connected_net_lines.emplace(uu, &new_line);
                     new_line.to.connect(sym, &pin);
-                    new_line.net = net;
+                    new_line.net = line->net;
                     sym->component->connections.emplace(std::piecewise_construct,
                                                         std::forward_as_tuple(sym->gate->uuid, pin.uuid),
-                                                        std::forward_as_tuple(net));
-                    if (conn.is_pin() && new_net) {
-                        conn.symbol->component->connections.emplace(
-                                std::piecewise_construct,
-                                std::forward_as_tuple(conn.symbol->gate->uuid, conn.pin->uuid),
-                                std::forward_as_tuple(net));
-                    }
+                                                        std::forward_as_tuple(line->net));
                 };
-                connect_pin(*pin_from, from, net_from);
-                connect_pin(*pin_to, to, net_to);
+                connect_pin(*pin_from, from);
+                connect_pin(*pin_to, to);
+                delete_net_line(sheet, line);
+                line = nullptr;
                 placed = true;
+                expand(true);
             }
         }
     }
@@ -761,10 +735,12 @@ void Schematic::expand(bool careful)
         }
     }
     std::map<std::string, std::set<const Net *>> net_names;
-    for (const auto &it : block->nets) {
-        if (it.second.is_named()) {
-            net_names[it.second.name];
-            net_names[it.second.name].insert(&it.second);
+    for (const auto &[uu, net] : block->nets) {
+        if (net.is_named()) {
+            std::string net_name = net.name;
+            trim(net_name);
+            const std::string net_name_casefold = Glib::ustring(net_name).casefold();
+            net_names[net_name_casefold].insert(&net);
         }
     }
     std::set<const Net *> nets_duplicate;
@@ -1006,11 +982,13 @@ void Schematic::swap_gates(const UUID &comp_uu, const UUID &g1_uu, const UUID &g
     auto entity = block->components.at(comp_uu).entity;
     for (auto &it_sheet : sheets) {
         for (auto &it_sym : it_sheet.second.symbols) {
-            if (it_sym.second.gate->uuid == g1_uu) {
-                it_sym.second.gate = &entity->gates.at(g2_uu);
-            }
-            else if (it_sym.second.gate->uuid == g2_uu) {
-                it_sym.second.gate = &entity->gates.at(g1_uu);
+            if (it_sym.second.component->uuid == comp_uu) {
+                if (it_sym.second.gate->uuid == g1_uu) {
+                    it_sym.second.gate = &entity->gates.at(g2_uu);
+                }
+                else if (it_sym.second.gate->uuid == g2_uu) {
+                    it_sym.second.gate = &entity->gates.at(g1_uu);
+                }
             }
         }
     }
