@@ -7,8 +7,6 @@
 #include "block/block.hpp"
 #include "util/uuid_ptr.hpp"
 #include "util/placement.hpp"
-#include "util/uuid_provider.hpp"
-#include "pool/pool.hpp"
 #include <vector>
 #include <map>
 #include <fstream>
@@ -16,9 +14,9 @@
 namespace horizon {
 using json = nlohmann::json;
 
-class SchematicSymbol : public UUIDProvider {
+class SchematicSymbol {
 public:
-    SchematicSymbol(const UUID &uu, const json &, Pool &pool, Block *block = nullptr);
+    SchematicSymbol(const UUID &uu, const json &, class IPool &pool, Block *block = nullptr);
     SchematicSymbol(const UUID &uu, const Symbol *sym);
     UUID uuid;
     const Symbol *pool_symbol;
@@ -37,8 +35,11 @@ public:
     void apply_pin_names();
 
     std::string replace_text(const std::string &t, bool *replaced, const class Schematic &sch) const;
+    std::string get_custom_value() const;
 
-    UUID get_uuid() const override;
+    std::string custom_value;
+
+    UUID get_uuid() const;
     json serialize() const;
 };
 } // namespace horizon
