@@ -13,6 +13,7 @@
 #include "alphanum/alphanum.hpp"
 #include <gdk/gdkkeysyms.h>
 #include "imp/in_tool_action.hpp"
+#include "str_util.hpp"
 
 namespace horizon {
 
@@ -70,26 +71,6 @@ json json_from_resource(const std::string &rsrc)
     return json::parse((const char *)json_bytes->get_data(size));
 }
 
-int orientation_to_angle(Orientation o)
-{
-    int angle = 0;
-    switch (o) {
-    case Orientation::RIGHT:
-        angle = 0;
-        break;
-    case Orientation::LEFT:
-        angle = 32768;
-        break;
-    case Orientation::UP:
-        angle = 16384;
-        break;
-    case Orientation::DOWN:
-        angle = 49152;
-        break;
-    }
-    return angle;
-}
-
 #ifdef G_OS_WIN32
 std::string get_exe_dir()
 {
@@ -163,81 +144,6 @@ void allow_set_foreground_window(int pid)
 #endif
 }
 
-
-std::string coord_to_string(const Coordf &pos, bool delta)
-{
-    std::ostringstream ss;
-    ss.imbue(get_locale());
-    if (delta) {
-        ss << "Δ";
-    }
-    ss << "X:";
-    if (pos.x >= 0) {
-        ss << "+";
-    }
-    else {
-        ss << "−"; // this is U+2212 MINUS SIGN, has same width as +
-    }
-    ss << std::fixed << std::setprecision(3) << std::setw(7) << std::setfill('0') << std::internal
-       << std::abs(pos.x / 1e6) << " mm "; // NBSP
-    if (delta) {
-        ss << "Δ";
-    }
-    ss << "Y:";
-    if (pos.y >= 0) {
-        ss << "+";
-    }
-    else {
-        ss << "−";
-    }
-    ss << std::setw(7) << std::abs(pos.y / 1e6) << " mm"; // NBSP
-    return ss.str();
-}
-
-std::string dim_to_string(int64_t x, bool with_sign)
-{
-    std::ostringstream ss;
-    ss.imbue(get_locale());
-    if (with_sign) {
-        if (x >= 0) {
-            ss << "+";
-        }
-        else {
-            ss << "−"; // this is U+2212 MINUS SIGN, has same width as +
-        }
-    }
-    ss << std::fixed << std::setprecision(3) << std::setw(7) << std::setfill('0') << std::internal << std::abs(x / 1e6)
-       << " mm"; // NBSP
-    return ss.str();
-}
-
-std::string angle_to_string(int x, bool pos_only)
-{
-    while (x < 0) {
-        x += 65536;
-    }
-    x %= 65536;
-    if (!pos_only && x > 32768)
-        x -= 65536;
-
-    std::ostringstream ss;
-    ss.imbue(get_locale());
-    if (x >= 0) {
-        ss << "+";
-    }
-    else {
-        ss << "−"; // this is U+2212 MINUS SIGN, has same width as +
-    }
-    ss << std::fixed << std::setprecision(3) << std::setw(7) << std::setfill('0') << std::internal
-       << std::abs((x / 65536.0) * 360) << " °"; // NBSP
-    return ss.str();
-}
-
-int64_t round_multiple(int64_t x, int64_t mul)
-{
-    return ((x + sgn(x) * mul / 2) / mul) * mul;
-}
-
 bool endswith(const std::string &haystack, const std::string &needle)
 {
     auto pos = haystack.rfind(needle);
@@ -504,50 +410,93 @@ void rmdir_recursive(const std::string &dir_name)
         throw std::runtime_error("g_rmdir");
 }
 
-std::string replace_placeholders(const std::string &s, std::function<std::string(const std::string &)> fn,
-                                 bool keep_empty)
+static bool isvar(char c)
 {
-    std::string r;
-    r.reserve(s.size());
+    return std::isalnum(c) || c == '_' || c == ':';
+}
+
+std::string interpolate_text(const std::string &str,
+                             std::function<std::optional<std::string>(const std::string &s)> interpolator)
+{
+
+    enum class State { NORMAL, DOLLAR, VAR, VAR_BRACED };
+    State state = State::NORMAL;
+    std::string out;
     std::string var_current;
-    bool var_mode = false;
-    for (const auto &it : s) {
-        if (var_mode) {
-            if (std::isspace(it)) {
-                auto repl = fn(var_current);
-                if (repl.size()) {
-                    r += repl;
-                }
-                else if (keep_empty) {
-                    r += "$" + var_current;
-                }
-                r += it;
+    size_t i = 0;
+    while (true) {
+        const char c = str.c_str()[i++];
+        switch (state) {
+        case State::NORMAL:
+            if (c == '$') {
+                state = State::DOLLAR;
+            }
+            else if (c) {
+                out.append(1, c);
+            }
+            break;
+
+        case State::DOLLAR:
+            if (isvar(c)) {
                 var_current.clear();
-                var_mode = false;
+                var_current.append(1, c);
+                state = State::VAR;
+            }
+            else if (c == '{') {
+                var_current.clear();
+                state = State::VAR_BRACED;
             }
             else {
-                var_current += it;
+                out.append("$");
+                if (c)
+                    out.append(1, c);
+                state = State::NORMAL;
             }
-        }
-        else {
-            if (it == '$') {
-                var_mode = true;
+            break;
+
+        case State::VAR:
+            if (isvar(c)) {
+                var_current.append(1, c);
             }
             else {
-                r += it;
+                if (auto subst = interpolator(var_current)) {
+                    out.append(*subst);
+                }
+                else {
+                    out.append("$" + var_current);
+                }
+
+                if (c == '$') {
+                    state = State::DOLLAR;
+                }
+                else if (c) {
+                    out.append(1, c);
+                    state = State::NORMAL;
+                }
             }
+            break;
+
+        case State::VAR_BRACED:
+            if (c == '}') {
+                if (auto subst = interpolator(var_current)) {
+                    out.append(*subst);
+                }
+                else {
+                    out.append("${" + var_current + "}");
+                }
+                state = State::NORMAL;
+            }
+            else {
+                var_current.append(1, c);
+            }
+            break;
         }
+
+        if (!c)
+            break;
     }
-    if (var_mode) {
-        auto repl = fn(var_current);
-        if (repl.size()) {
-            r += repl;
-        }
-        else if (keep_empty) {
-            r += "$" + var_current;
-        }
-    }
-    return r;
+
+    return out;
 }
 
 std::pair<Coordi, bool> dir_from_action(InToolActionID a)
@@ -579,16 +528,6 @@ std::pair<Coordi, bool> dir_from_action(InToolActionID a)
     }
 }
 
-Coordd project_onto_perp_bisector(const Coordd &a, const Coordd &b, const Coordd &p)
-{
-    const auto c = (a + b) / 2;
-    const auto d = b - a;
-    if (d.mag_sq() == 0)
-        return p;
-    const auto u = (d.dot(c) - d.dot(p)) / d.mag_sq();
-    return p + d * u;
-}
-
 void check_object_type(const json &j, ObjectType type)
 {
     if (j.at("type") != object_type_lut.lookup_reverse(type)) {
@@ -596,4 +535,23 @@ void check_object_type(const json &j, ObjectType type)
     }
 }
 
+void ensure_parent_dir(const std::string &path)
+{
+    auto parent = Glib::path_get_dirname(path);
+    if (!Glib::file_test(parent, Glib::FILE_TEST_IS_DIR)) {
+        Gio::File::create_for_path(parent)->make_directory_with_parents();
+    }
+}
+
+
+std::string append_dot_json(const std::string &s)
+{
+    std::string r = s;
+    trim(r);
+    if (!endswith(r, ".json")) {
+        return r + ".json";
+    }
+    return r;
+}
+
 } // namespace horizon
