@@ -6,6 +6,7 @@
 #include <functional>
 #include <locale>
 #include <fstream>
+#include <optional>
 
 namespace horizon {
 using json = nlohmann::json;
@@ -16,17 +17,11 @@ std::ofstream make_ofstream(const std::string &filename_utf8, std::ios_base::ope
 
 void save_json_to_file(const std::string &filename, const json &j);
 json load_json_from_file(const std::string &filename);
-int orientation_to_angle(Orientation o);
 std::string get_exe_dir();
 void allow_set_foreground_window(int pid);
-std::string coord_to_string(const Coordf &c, bool delta = false);
-std::string dim_to_string(int64_t x, bool with_sign = true);
-std::string angle_to_string(int angle, bool pos_only = true);
 void setup_locale();
 const std::locale &get_locale();
 
-int64_t round_multiple(int64_t x, int64_t mul);
-
 template <typename T, typename U> std::vector<T> dynamic_cast_vector(const std::vector<U> &cin)
 {
     std::vector<T> out;
@@ -43,11 +38,6 @@ template <typename Map, typename F> static void map_erase_if(Map &m, F pred)
 
 bool endswith(const std::string &haystack, const std::string &needle);
 
-template <typename T> int sgn(T val)
-{
-    return (T(0) < val) - (val < T(0));
-}
-
 int strcmp_natural(const std::string &a, const std::string &b);
 int strcmp_natural(const char *a, const char *b);
 void create_config_dir();
@@ -70,21 +60,11 @@ std::string format_digits(unsigned int m, unsigned int digits_max);
 double parse_si(const std::string &inps);
 
 void rmdir_recursive(const std::string &dir_name);
-std::string replace_placeholders(const std::string &s, std::function<std::string(const std::string &)> fn,
-                                 bool keep_empty);
+std::string interpolate_text(const std::string &str,
+                             std::function<std::optional<std::string>(const std::string &s)> interpolator);
 
 std::pair<Coordi, bool> dir_from_action(InToolActionID a);
 
-template <typename T> std::pair<Coord<T>, Coord<T>> pad_bbox(std::pair<Coord<T>, Coord<T>> bb, T pad)
-{
-    bb.first.x -= pad;
-    bb.first.y -= pad;
-
-    bb.second.x += pad;
-    bb.second.y += pad;
-    return bb;
-}
-Coordd project_onto_perp_bisector(const Coordd &a, const Coordd &b, const Coordd &p);
 
 template <typename T> constexpr bool any_of(T value, std::initializer_list<T> choices)
 {
@@ -92,5 +72,8 @@ template <typename T> constexpr bool any_of(T value, std::initializer_list<T> ch
 }
 
 void check_object_type(const json &j, ObjectType type);
+void ensure_parent_dir(const std::string &path);
+
+std::string append_dot_json(const std::string &s);
 
 } // namespace horizon
