@@ -6,47 +6,104 @@
 #include "cell_renderer_color_box.hpp"
 #include "pool/pool_manager.hpp"
 #include "tag_entry.hpp"
+#include "util/sqlite.hpp"
+#include "pool_selector.hpp"
 
 namespace horizon {
-PoolBrowser::PoolBrowser(IPool &p) : Gtk::Box(Gtk::ORIENTATION_VERTICAL, 0), pool(p)
+PoolBrowser::PoolBrowser(IPool &p, const std::string &prefix)
+    : Gtk::Box(Gtk::ORIENTATION_VERTICAL, 0), pool(p), store_prefix(prefix)
 {
-    const auto &pools = PoolManager::get().get_pools();
-    if (pools.count(pool.get_base_path())) {
-        const auto &mpool = pools.at(pool.get_base_path());
-        pools_included = mpool.pools_included.size();
-        pool_uuid = mpool.uuid;
-    }
+    const auto &pool_info = pool.get_pool_info();
+    pools_included = pool_info.pools_included.size();
+    pool_uuid = pool_info.uuid;
 }
 
-Gtk::Entry *PoolBrowser::create_search_entry(const std::string &label)
+Gtk::Entry *PoolBrowser::create_search_entry(const std::string &label, Gtk::Widget *extra_widget)
 {
     auto entry = Gtk::manage(new Gtk::SearchEntry());
-    add_search_widget(label, *entry);
+    add_search_widget(label, *entry, extra_widget);
     entry->signal_search_changed().connect(sigc::mem_fun(*this, &PoolBrowser::search));
     search_entries.insert(entry);
     return entry;
 }
 
-TagEntry *PoolBrowser::create_tag_entry(const std::string &label)
+TagEntry *PoolBrowser::create_tag_entry(const std::string &label, Gtk::Widget *extra_widget)
 {
     auto entry = Gtk::manage(new TagEntry(pool, get_type()));
-    add_search_widget(label, *entry);
+    add_search_widget(label, *entry, extra_widget);
     entry->signal_changed().connect(sigc::mem_fun(*this, &PoolBrowser::search));
     tag_entries.insert(entry);
     return entry;
 }
 
-void PoolBrowser::add_search_widget(const std::string &label, Gtk::Widget &w)
+void PoolBrowser::add_search_widget(const std::string &label, Gtk::Widget &w, Gtk::Widget *extra_widget)
 {
     auto la = Gtk::manage(new Gtk::Label(label));
     la->get_style_context()->add_class("dim-label");
     la->set_halign(Gtk::ALIGN_END);
     grid->attach(*la, 0, grid_top, 1, 1);
     w.set_hexpand(true);
-    grid->attach(w, 1, grid_top, 1, 1);
+    if (extra_widget == nullptr) {
+        grid->attach(w, 1, grid_top, 1, 1);
+        w.show();
+    }
+    else {
+        auto box = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_HORIZONTAL, 10));
+        box->pack_start(w, true, true, 0);
+        box->pack_start(*extra_widget, false, false, 0);
+        box->show_all();
+        grid->attach(*box, 1, grid_top, 1, 1);
+    }
     grid_top++;
     la->show();
-    w.show();
+}
+
+Gtk::Widget *PoolBrowser::create_pool_selector()
+{
+    if (pools_included) {
+        if (pool_selector)
+            throw std::runtime_error("can't create pool selector twice");
+        pool_selector = Gtk::manage(new PoolSelector(pool));
+        pool_selector->signal_changed().connect([this] {
+            if (!pools_reloading)
+                search();
+        });
+        return pool_selector;
+    }
+    else {
+        return nullptr;
+    }
+}
+
+void PoolBrowser::reload_pools()
+{
+    if (!pool_selector)
+        return;
+    pools_reloading = true;
+    pool_selector->reload();
+    pools_reloading = false;
+}
+
+std::string PoolBrowser::get_pool_selector_query() const
+{
+    if (!pool_selector)
+        return "";
+
+    if (pool_selector->get_selected_pool() == UUID())
+        return "";
+
+    return " AND (" + IPool::type_names.at(get_type()) + ".pool_uuid = $pool_uuid) ";
+}
+
+void PoolBrowser::bind_pool_selector_query(SQLite::Query &q) const
+{
+    if (!pool_selector)
+        return;
+
+    if (pool_selector->get_selected_pool() == UUID())
+        return;
+
+    q.bind("$pool_uuid", pool_selector->get_selected_pool());
 }
 
 void PoolBrowser::install_column_tooltip(Gtk::TreeViewColumn &tvc, const Gtk::TreeModelColumnBase &col)
@@ -167,11 +224,12 @@ void PoolBrowser::construct(Gtk::Widget *search_box)
 
     {
         status_box = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_HORIZONTAL, 8));
-        status_box->get_style_context()->add_class("dim-label");
-        status_box->property_margin() = 2;
+        status_box->get_style_context()->add_class("pool-browser-status-box");
         status_label = Gtk::manage(new Gtk::Label("Status"));
         label_set_tnum(status_label);
         status_label->set_xalign(0);
+        status_label->get_style_context()->add_class("dim-label");
+        status_label->property_margin() = 2;
         status_label->show();
         status_box->pack_start(*status_label, false, false, 0);
         pack_start(*status_box, false, false, 0);
@@ -199,15 +257,28 @@ void PoolBrowser::construct(Gtk::Widget *search_box)
         path_column->set_visible(false);
     }
     treeview->signal_button_press_event().connect_notify([this](GdkEventButton *ev) {
-        Gtk::TreeModel::Path path;
         if (ev->button == 3 && context_menu.get_children().size()) {
+            Gtk::TreeModel::Path path;
+            if (treeview->get_bin_window()->gobj() == ev->window && treeview->get_path_at_pos(ev->x, ev->y, path)) {
+                if (auto it = store->get_iter(path)) {
+                    Gtk::TreeModel::Row row = *it;
+                    auto sel = uuid_from_row(row);
+                    for (auto &[widget, cb] : menu_item_sensitive_cbs) {
+                        widget->set_visible(!sel || cb(sel));
+                    }
 #if GTK_CHECK_VERSION(3, 22, 0)
-            context_menu.popup_at_pointer((GdkEvent *)ev);
+                    context_menu.popup_at_pointer((GdkEvent *)ev);
 #else
-            context_menu.popup(ev->button, gtk_get_current_event_time());
+                    context_menu.popup(ev->button, gtk_get_current_event_time());
 #endif
+                }
+            }
         }
     });
+
+    if (store_prefix.size()) {
+        state_store.emplace(treeview, store_prefix);
+    }
 }
 
 void PoolBrowser::set_busy(bool busy)
@@ -285,6 +356,12 @@ void PoolBrowser::clear_search()
     }
 }
 
+void PoolBrowser::focus_search()
+{
+    if (focus_widget)
+        focus_widget->grab_focus();
+}
+
 void PoolBrowser::go_to(const UUID &uu)
 {
     clear_search();
@@ -293,22 +370,37 @@ void PoolBrowser::go_to(const UUID &uu)
     scroll_to_selection();
 }
 
-void PoolBrowser::add_context_menu_item(const std::string &label, sigc::slot1<void, UUID> cb)
+void PoolBrowser::add_context_menu_item(const std::string &label, std::function<void(UUID)> cb,
+                                        std::function<bool(UUID)> cb_sensitive)
 {
     auto la = Gtk::manage(new Gtk::MenuItem(label));
     la->show();
     context_menu.append(*la);
     la->signal_activate().connect([this, cb] { cb(get_selected()); });
+    if (cb_sensitive)
+        menu_item_sensitive_cbs.emplace_back(la, cb_sensitive);
 }
 
-PoolBrowser::PoolItemSource PoolBrowser::pool_item_source_from_db(const UUID &uu, bool overridden)
+PoolBrowser::PoolItemSource PoolBrowser::pool_item_source_from_db(const SQLite::Query &q, int idx_uu,
+                                                                  int idx_last_uu) const
 {
-    if (overridden && uu == pool_uuid)
-        return PoolItemSource::OVERRIDING;
-    else if (uu == pool_uuid)
+    return pool_item_source_from_db(q.get<std::string>(idx_uu), q.get<std::string>(idx_last_uu));
+}
+
+
+PoolBrowser::PoolItemSource PoolBrowser::pool_item_source_from_db(const UUID &uu, const UUID &last_uu) const
+{
+    if (uu == pool_uuid && last_uu == UUID())
         return PoolItemSource::LOCAL;
-    else
+    else if (uu == pool_uuid && last_uu != UUID() && uu == PoolInfo::project_pool_uuid)
+        return PoolItemSource::CACHED;
+    else if (uu == pool_uuid && last_uu != UUID())
+        return PoolItemSource::OVERRIDDEN_LOCAL;
+    else if (uu != pool_uuid && last_uu == UUID())
         return PoolItemSource::INCLUDED;
+    else if (uu != pool_uuid && last_uu != UUID())
+        return PoolItemSource::OVERRIDDEN;
+    throw std::logic_error("unhandled case");
 }
 
 void PoolBrowser::install_pool_item_source_tooltip()
@@ -346,9 +438,15 @@ void PoolBrowser::install_pool_item_source_tooltip()
                         case PoolItemSource::INCLUDED:
                             tooltip->set_text("Item is from included pool");
                             break;
-                        case PoolItemSource::OVERRIDING:
+                        case PoolItemSource::OVERRIDDEN_LOCAL:
                             tooltip->set_text("Item is from this pool overriding an item from an included pool");
                             break;
+                        case PoolItemSource::CACHED:
+                            tooltip->set_text("Item is cached from an included pool");
+                            break;
+                        case PoolItemSource::OVERRIDDEN:
+                            tooltip->set_text("Item is from another pool overriding an item from an included pool");
+                            break;
                         }
                         //
                         treeview->set_tooltip_cell(tooltip, &path, column, cell);
@@ -378,6 +476,14 @@ CellRendererColorBox *PoolBrowser::create_pool_item_source_cr(Gtk::TreeViewColum
             co = Color::new_from_int(138, 226, 52);
             break;
 
+        case PoolItemSource::CACHED:
+            co = Color::new_from_int(0x72, 0x9f, 0xcf);
+            break;
+
+        case PoolItemSource::OVERRIDDEN_LOCAL:
+            co = Color::new_from_int(0xad, 0x7f, 0xa8);
+            break;
+
         case PoolItemSource::INCLUDED:
             co = Color::new_from_int(252, 175, 62);
             break;
@@ -424,4 +530,35 @@ void PoolBrowser::finish_search()
     status_label->set_text(format_digits(n_items, 5) + " " + (n_items == 1 ? "Result" : "Results"));
 }
 
+std::string PoolBrowser::get_tags_query(const std::set<std::string> &tags) const
+{
+    if (tags.size() == 0)
+        return "";
+    std::string query;
+    query += "INNER JOIN (SELECT uuid FROM tags WHERE tags.tag IN (";
+    int i = 0;
+    for (const auto &it : tags) {
+        (void)it;
+        query += "$tag" + std::to_string(i) + ", ";
+        i++;
+    }
+    query += "'') AND tags.type = $tag_type "
+             "GROUP by tags.uuid HAVING count(*) >= $ntags) as x_tags ON x_tags.uuid = "
+             + IPool::type_names.at(get_type()) + ".uuid ";
+    return query;
+}
+
+void PoolBrowser::bind_tags_query(SQLite::Query &q, const std::set<std::string> &tags) const
+{
+    int i = 0;
+    for (const auto &it : tags) {
+        q.bind(("$tag" + std::to_string(i)).c_str(), it);
+        i++;
+    }
+    if (tags.size()) {
+        q.bind("$ntags", tags.size());
+        q.bind("$tag_type", get_type());
+    }
+}
+
 } // namespace horizon
