@@ -5,11 +5,13 @@
 #include <set>
 
 namespace horizon {
-PoolBrowserEntity::PoolBrowserEntity(IPool &p) : PoolBrowser(p)
+PoolBrowserEntity::PoolBrowserEntity(IPool &p, const std::string &instance)
+    : PoolBrowser(p, TreeViewStateStore::get_prefix(instance, "pool_browser_entity"))
 {
     construct();
     name_entry = create_search_entry("Name");
-    tag_entry = create_tag_entry("Tags");
+    tag_entry = create_tag_entry("Tags", create_pool_selector());
+    focus_widget = name_entry;
     install_pool_item_source_tooltip();
 }
 
@@ -20,12 +22,23 @@ Glib::RefPtr<Gtk::ListStore> PoolBrowserEntity::create_list_store()
 
 void PoolBrowserEntity::create_columns()
 {
-    append_column_with_item_source_cr("Entity", list_columns.entity_name);
-    treeview->append_column("Manufacturer", list_columns.entity_manufacturer);
+    {
+        auto col = append_column_with_item_source_cr("Entity", list_columns.entity_name, Pango::ELLIPSIZE_END);
+        col->set_resizable(true);
+        col->set_min_width(150);
+    }
+    {
+        auto col = append_column("Manufacturer", list_columns.entity_manufacturer, Pango::ELLIPSIZE_END);
+        col->set_resizable(true);
+        col->set_min_width(50);
+        install_column_tooltip(*col, list_columns.entity_manufacturer);
+    }
     treeview->append_column("Prefix", list_columns.prefix);
     treeview->append_column("Gates", list_columns.n_gates);
     {
-        auto col = append_column("Tags", list_columns.tags);
+        auto col = append_column("Tags", list_columns.tags, Pango::ELLIPSIZE_END);
+        col->set_resizable(true);
+        col->set_min_width(50);
         install_column_tooltip(*col, list_columns.tags);
     }
     path_column = append_column("Path", list_columns.path, Pango::ELLIPSIZE_START);
@@ -42,49 +55,22 @@ void PoolBrowserEntity::search()
 {
     prepare_search();
 
-    std::string name_search = name_entry->get_text();
+    const std::string name_search = name_entry->get_text();
 
-    auto tags = tag_entry->get_tags();
-    std::string query;
-    if (tags.size() == 0) {
-        query = "SELECT entities.uuid, entities.name, entities.prefix, "
-                "entities.n_gates, tags_view.tags, "
-                "entities.manufacturer, entities.filename, entities.pool_uuid, entities.overridden FROM entities "
-                "LEFT JOIN tags_view ON tags_view.uuid = entities.uuid AND tags_view.type = 'entity' "
-                "WHERE entities.name LIKE $name "
-                + sort_controller->get_order_by();
-    }
-    else {
-        std::ostringstream qs;
-        qs << "SELECT entities.uuid, entities.name, entities.prefix, "
-              "entities.n_gates, tags_view.tags, entities.manufacturer, "
-              "entities.filename, entities.pool_uuid, entities.overridden FROM entities "
-              "LEFT JOIN tags_view ON tags_view.uuid = entities.uuid AND tags_view.type = 'entity' "
-              "INNER JOIN (SELECT uuid FROM tags WHERE tags.tag IN (";
-
-        int i = 0;
-        for (const auto &it : tags) {
-            (void)sizeof it;
-            qs << "$tag" << i << ", ";
-            i++;
-        }
-        qs << "'') AND tags.type = 'entity' "
-              "GROUP by tags.uuid HAVING count(*) >= $ntags) as x ON x.uuid = entities.uuid "
-              "WHERE entities.name LIKE $name ";
-
-        qs << sort_controller->get_order_by();
-        query = qs.str();
-    }
-    std::cout << query << std::endl;
+    const auto tags = tag_entry->get_tags();
+    std::string query =
+            "SELECT entities.uuid, entities.name, entities.prefix, "
+            "entities.n_gates, tags_view.tags, entities.manufacturer, "
+            "entities.filename, entities.pool_uuid, entities.last_pool_uuid FROM entities "
+            "LEFT JOIN tags_view ON tags_view.uuid = entities.uuid AND tags_view.type = 'entity' ";
+    query += get_tags_query(tags);
+    query += "WHERE entities.name LIKE $name ";
+    query += get_pool_selector_query();
+    query += sort_controller->get_order_by();
     SQLite::Query q(pool.get_db(), query);
     q.bind("$name", "%" + name_search + "%");
-    int i = 0;
-    for (const auto &it : tags) {
-        q.bind(("$tag" + std::to_string(i)).c_str(), it);
-        i++;
-    }
-    if (tags.size())
-        q.bind("$ntags", tags.size());
+    bind_tags_query(q, tags);
+    bind_pool_selector_query(q);
 
     Gtk::TreeModel::Row row;
     if (show_none) {
@@ -102,7 +88,7 @@ void PoolBrowserEntity::search()
             row[list_columns.tags] = q.get<std::string>(4);
             row[list_columns.entity_manufacturer] = q.get<std::string>(5);
             row[list_columns.path] = q.get<std::string>(6);
-            row[list_columns.source] = pool_item_source_from_db(q.get<std::string>(7), q.get<int>(8));
+            row[list_columns.source] = pool_item_source_from_db(q, 7, 8);
         }
         set_busy(false);
     }
