@@ -6,7 +6,8 @@ namespace horizon {
 PoolBrowserFrame::PoolBrowserFrame(IPool &p) : PoolBrowser(p)
 {
     construct();
-    name_entry = create_search_entry("Name");
+    name_entry = create_search_entry("Name", create_pool_selector());
+    focus_widget = name_entry;
     install_pool_item_source_tooltip();
 }
 
@@ -34,11 +35,13 @@ void PoolBrowserFrame::search()
     std::string name_search = name_entry->get_text();
 
     std::string query =
-            "SELECT frames.uuid, frames.name, frames.filename, frames.pool_uuid, frames.overridden FROM frames WHERE "
+            "SELECT frames.uuid, frames.name, frames.filename, frames.pool_uuid, frames.last_pool_uuid FROM frames "
+            "WHERE "
             "frames.name LIKE ?"
-            + sort_controller->get_order_by();
+            + get_pool_selector_query() + sort_controller->get_order_by();
     SQLite::Query q(pool.get_db(), query);
     q.bind(1, "%" + name_search + "%");
+    bind_pool_selector_query(q);
 
     Gtk::TreeModel::Row row;
     if (show_none) {
@@ -52,7 +55,7 @@ void PoolBrowserFrame::search()
             row[list_columns.uuid] = q.get<std::string>(0);
             row[list_columns.name] = q.get<std::string>(1);
             row[list_columns.path] = q.get<std::string>(2);
-            row[list_columns.source] = pool_item_source_from_db(q.get<std::string>(3), q.get<int>(4));
+            row[list_columns.source] = pool_item_source_from_db(q, 3, 4);
         }
         set_busy(false);
     }
