@@ -7,7 +7,8 @@ namespace horizon {
 PoolBrowserPadstack::PoolBrowserPadstack(IPool &p) : PoolBrowser(p)
 {
     construct();
-    name_entry = create_search_entry("Name");
+    name_entry = create_search_entry("Name", create_pool_selector());
+    focus_widget = name_entry;
     install_pool_item_source_tooltip();
 }
 
@@ -36,14 +37,14 @@ void PoolBrowserPadstack::set_package_uuid(const UUID &uu)
     search();
 }
 
-void PoolBrowserPadstack::set_include_padstack_type(Padstack::Type ty, bool v)
+const std::set<Padstack::Type> &PoolBrowserPadstack::get_padstacks_included() const
 {
-    if (padstacks_included.count(ty)) {
-        if (!v)
-            padstacks_included.erase(ty);
-    }
-    if (v)
-        padstacks_included.insert(ty);
+    return padstacks_included;
+}
+
+void PoolBrowserPadstack::set_padstacks_included(const std::set<Padstack::Type> &types)
+{
+    padstacks_included = types;
     search();
 }
 
@@ -53,14 +54,16 @@ void PoolBrowserPadstack::search()
 
     std::string name_search = name_entry->get_text();
 
-    SQLite::Query q(pool.get_db(),
-                    "SELECT padstacks.uuid, padstacks.name, padstacks.type, "
-                    "packages.name, padstacks.filename, padstacks.pool_uuid, padstacks.overridden FROM padstacks LEFT "
-                    "JOIN packages ON padstacks.package = packages.uuid WHERE "
-                    "(packages.uuid=? OR ? OR padstacks.package = "
-                    "'00000000-0000-0000-0000-000000000000')  AND "
-                    "padstacks.name LIKE ? AND padstacks.type IN (?, ?, ?, ?, "
-                    "?, ?) " + sort_controller->get_order_by());
+    SQLite::Query q(
+            pool.get_db(),
+            "SELECT padstacks.uuid, padstacks.name, padstacks.type, "
+            "packages.name, padstacks.filename, padstacks.pool_uuid, padstacks.last_pool_uuid FROM padstacks LEFT "
+            "JOIN packages ON padstacks.package = packages.uuid WHERE "
+            "(packages.uuid=? OR ? OR padstacks.package = "
+            "'00000000-0000-0000-0000-000000000000')  AND "
+            "padstacks.name LIKE ? AND padstacks.type IN (?, ?, ?, ?, "
+            "?, ?) " + get_pool_selector_query()
+                    + sort_controller->get_order_by());
     q.bind(1, package_uuid);
     q.bind(2, package_uuid == UUID());
     q.bind(3, "%" + name_search + "%");
@@ -94,6 +97,7 @@ void PoolBrowserPadstack::search()
     else
         q.bind(9, std::string(""));
 
+    bind_pool_selector_query(q);
 
     Gtk::TreeModel::Row row;
     if (show_none) {
@@ -109,7 +113,7 @@ void PoolBrowserPadstack::search()
             row[list_columns.padstack_type] = q.get<std::string>(2);
             row[list_columns.package_name] = q.get<std::string>(3);
             row[list_columns.path] = q.get<std::string>(4);
-            row[list_columns.source] = pool_item_source_from_db(q.get<std::string>(5), q.get<int>(6));
+            row[list_columns.source] = pool_item_source_from_db(q, 5, 6);
         }
         set_busy(false);
     }
