@@ -2,6 +2,7 @@
 #include "pool/ipool.hpp"
 #include "pool/part.hpp"
 #include "util/util.hpp"
+#include "util/geom_util.hpp"
 
 namespace horizon {
 
@@ -13,7 +14,7 @@ static Gtk::TreeViewColumn *create_tvc(const PoolParametric::Column &col,
         auto tvc = Gtk::manage(new Gtk::TreeViewColumn(col.display_name));
         auto cr_val = Gtk::manage(new Gtk::CellRendererText());
         auto cr_unit = Gtk::manage(new Gtk::CellRendererText());
-        tvc->set_cell_data_func(*cr_val, [&col, &tree_col](Gtk::CellRenderer *tcr, const Gtk::TreeModel::iterator &it) {
+        tvc->set_cell_data_func(*cr_val, [&tree_col](Gtk::CellRenderer *tcr, const Gtk::TreeModel::iterator &it) {
             Gtk::TreeModel::Row row = *it;
             auto mcr = dynamic_cast<Gtk::CellRendererText *>(tcr);
             std::string v = row[tree_col];
@@ -23,17 +24,16 @@ static Gtk::TreeViewColumn *create_tvc(const PoolParametric::Column &col,
             else
                 mcr->property_text() = v.substr(0, pos);
         });
-        tvc->set_cell_data_func(*cr_unit,
-                                [&col, &tree_col](Gtk::CellRenderer *tcr, const Gtk::TreeModel::iterator &it) {
-                                    Gtk::TreeModel::Row row = *it;
-                                    auto mcr = dynamic_cast<Gtk::CellRendererText *>(tcr);
-                                    std::string v = row[tree_col];
-                                    auto pos = v.find(' ');
-                                    if (pos == std::string::npos)
-                                        mcr->property_text() = "";
-                                    else
-                                        mcr->property_text() = v.substr(pos + 1);
-                                });
+        tvc->set_cell_data_func(*cr_unit, [&tree_col](Gtk::CellRenderer *tcr, const Gtk::TreeModel::iterator &it) {
+            Gtk::TreeModel::Row row = *it;
+            auto mcr = dynamic_cast<Gtk::CellRendererText *>(tcr);
+            std::string v = row[tree_col];
+            auto pos = v.find(' ');
+            if (pos == std::string::npos)
+                mcr->property_text() = "";
+            else
+                mcr->property_text() = v.substr(pos + 1);
+        });
         cr_val->property_xalign() = 1;
         cr_unit->property_xalign() = 1;
         auto attributes_list = pango_attr_list_new();
@@ -205,8 +205,10 @@ private:
     type_signal_activated s_signal_activated;
 };
 
-PoolBrowserParametric::PoolBrowserParametric(IPool &p, PoolParametric &pp, const std::string &table_name)
-    : PoolBrowserStockinfo(p), pool_parametric(pp), table(pp.get_tables().at(table_name)), list_columns(table)
+PoolBrowserParametric::PoolBrowserParametric(IPool &p, PoolParametric &pp, const std::string &table_name,
+                                             const std::string &instance)
+    : PoolBrowserStockinfo(p, TreeViewStateStore::get_prefix(instance, "pool_browser_parametric_" + table_name)),
+      pool_parametric(pp), table(pp.get_tables().at(table_name)), list_columns(table)
 {
     search_box = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_VERTICAL, 10));
     search_box->property_margin() = 10;
@@ -265,6 +267,10 @@ PoolBrowserParametric::PoolBrowserParametric(IPool &p, PoolParametric &pp, const
     filters_applied_box->show();
     hbox->pack_start(*filters_applied_box, true, true, 0);
 
+    if (auto selector = create_pool_selector()) {
+        hbox->pack_start(*selector, false, false, 0);
+        selector->show();
+    }
 
     hbox->show();
     search_box->pack_start(*hbox, false, false, 0);
@@ -286,7 +292,6 @@ void PoolBrowserParametric::create_columns()
     {
         auto col = append_column_with_item_source_cr("MPN", list_columns.MPN, Pango::ELLIPSIZE_END);
         col->set_resizable(true);
-        col->set_expand(true);
         col->set_min_width(100);
     }
     {
@@ -368,7 +373,7 @@ void PoolBrowserParametric::search()
               "all_derived.uuidy = parts.base) ";
     }
     qs += "SELECT p.*, parts.MPN, parts.manufacturer, packages.name, parts.filename, parts.pool_uuid, "
-          "parts.overridden "
+          "parts.last_pool_uuid "
           "FROM "
           + table.name
           + " AS p LEFT JOIN pool.parts USING (uuid) LEFT JOIN pool.packages ON parts.package = packages.uuid ";
@@ -389,11 +394,12 @@ void PoolBrowserParametric::search()
             }
         }
     }
+    qs += get_pool_selector_query();
     qs += sort_controller->get_order_by();
-    std::cout << qs << std::endl;
     SQLite::Query q(pool_parametric.db, qs);
     bind_set(q, "_manufacturer", manufacturers);
     bind_set(q, "_package", packages);
+    bind_pool_selector_query(q);
     if (similar_part_uuid) {
         q.bind("$similar_part", similar_part_uuid);
     }
@@ -423,7 +429,7 @@ void PoolBrowserParametric::search()
             std::string manufacturer = q.get<std::string>(ofs + 1);
             std::string package = q.get<std::string>(ofs + 2);
             row[list_columns.path] = q.get<std::string>(ofs + 3);
-            row[list_columns.source] = pool_item_source_from_db(q.get<std::string>(ofs + 4), q.get<int>(ofs + 5));
+            row[list_columns.source] = pool_item_source_from_db(q, ofs + 4, ofs + 5);
             row[list_columns.manufacturer] = manufacturer;
             row[list_columns.package] = package;
             values_remaining["_manufacturer"].emplace(manufacturer);
