@@ -6,13 +6,34 @@
 #include <set>
 
 namespace horizon {
-PoolBrowserPart::PoolBrowserPart(IPool &p, const UUID &uu) : PoolBrowserStockinfo(p), entity_uuid(uu)
+PoolBrowserPart::PoolBrowserPart(IPool &p, const UUID &uu, const std::string &instance)
+    : PoolBrowserStockinfo(p, TreeViewStateStore::get_prefix(instance, "pool_browser_part")), entity_uuid(uu)
 {
     construct();
     MPN_entry = create_search_entry("MPN");
+    focus_widget = MPN_entry;
     manufacturer_entry = create_search_entry("Manufacturer");
     desc_entry = create_search_entry("Description");
-    tag_entry = create_tag_entry("Tags");
+
+    auto extra_box = Gtk::manage(new Gtk::Box(Gtk::ORIENTATION_HORIZONTAL, 10));
+
+    base_parts_cb = Gtk::manage(new Gtk::CheckButton("Base parts"));
+    base_parts_cb->set_tooltip_text("Include base parts");
+    base_parts_cb->set_active(true);
+    base_parts_cb->show();
+    base_parts_cb->signal_toggled().connect([this] {
+        if (searched_once)
+            search();
+    });
+    extra_box->pack_start(*base_parts_cb, false, false, 0);
+
+    if (auto sel = create_pool_selector()) {
+        extra_box->pack_start(*sel, false, false, 0);
+        sel->show();
+    }
+    extra_box->show();
+
+    tag_entry = create_tag_entry("Tags", extra_box);
 
     install_pool_item_source_tooltip();
 }
@@ -29,6 +50,11 @@ void PoolBrowserPart::set_entity_uuid(const UUID &uu)
     search();
 }
 
+void PoolBrowserPart::set_include_base_parts(bool v)
+{
+    base_parts_cb->set_active(v);
+}
+
 Glib::RefPtr<Gtk::ListStore> PoolBrowserPart::create_list_store()
 {
     return Gtk::ListStore::create(list_columns);
@@ -39,13 +65,12 @@ void PoolBrowserPart::create_columns()
     {
         auto col = append_column_with_item_source_cr("MPN", list_columns.MPN, Pango::ELLIPSIZE_END);
         col->set_resizable(true);
-        col->set_expand(true);
-        col->set_min_width(200);
+        col->set_min_width(50);
     }
     {
         auto col = append_column("Manufacturer", list_columns.manufacturer, Pango::ELLIPSIZE_END);
         col->set_resizable(true);
-        col->set_min_width(200);
+        col->set_min_width(50);
     }
     {
         auto col = append_column("Description", list_columns.description, Pango::ELLIPSIZE_END);
@@ -56,12 +81,12 @@ void PoolBrowserPart::create_columns()
     {
         auto col = append_column("Package", list_columns.package, Pango::ELLIPSIZE_END);
         col->set_resizable(true);
-        col->set_min_width(100);
+        col->set_min_width(50);
     }
     {
         auto col = append_column("Tags", list_columns.tags, Pango::ELLIPSIZE_END);
         col->set_resizable(true);
-        col->set_min_width(100);
+        col->set_min_width(50);
         install_column_tooltip(*col, list_columns.tags);
     }
     path_column = append_column("Path", list_columns.path, Pango::ELLIPSIZE_START);
@@ -80,55 +105,36 @@ void PoolBrowserPart::search()
     prepare_search();
     iter_cache.clear();
     Gtk::TreeModel::Row row;
-    std::string MPN_search = MPN_entry->get_text();
-    std::string manufacturer_search = manufacturer_entry->get_text();
-    std::string desc_search = desc_entry->get_text();
+    const std::string MPN_search = MPN_entry->get_text();
+    const std::string manufacturer_search = manufacturer_entry->get_text();
+    const std::string desc_search = desc_entry->get_text();
 
-    auto tags = tag_entry->get_tags();
-    std::stringstream query;
-    if (tags.size() == 0) {
-        query << "SELECT parts.uuid, parts.MPN, parts.manufacturer, "
-                 "packages.name, tags_view.tags, parts.filename, "
-                 "parts.description, parts.pool_uuid, parts.overridden "
-                 "FROM parts LEFT JOIN tags_view ON "
-                 "tags_view.uuid = parts.uuid AND tags_view.type = 'part' "
-                 "LEFT JOIN packages ON packages.uuid = parts.package ";
-    }
-    else {
-        query << "SELECT parts.uuid, parts.MPN, parts.manufacturer, "
-                 "packages.name, tags_view.tags, parts.filename, "
-                 "parts.description, parts.pool_uuid, parts.overridden FROM parts "
-                 "LEFT JOIN tags_view ON tags_view.uuid = parts.uuid AND tags_view.type = 'part' "
-                 "LEFT JOIN packages ON packages.uuid = parts.package "
-                 "INNER JOIN (SELECT uuid FROM tags WHERE tags.tag IN (";
-        int i = 0;
-        for (const auto &it : tags) {
-            (void)sizeof it;
-            query << "$tag" << i << ", ";
-            i++;
-        }
-        query << "'') AND tags.type = 'part' "
-                 "GROUP by tags.uuid HAVING count(*) >= $ntags) as x ON x.uuid = parts.uuid ";
-    }
-    query << "WHERE parts.MPN LIKE $mpn "
+    const auto tags = tag_entry->get_tags();
+    std::string query =
+            "SELECT parts.uuid, parts.MPN, parts.manufacturer, "
+            "packages.name, tags_view.tags, parts.filename, "
+            "parts.description, parts.pool_uuid, parts.last_pool_uuid "
+            "FROM parts LEFT JOIN tags_view ON "
+            "tags_view.uuid = parts.uuid AND tags_view.type = 'part' "
+            "LEFT JOIN packages ON packages.uuid = parts.package ";
+    query += get_tags_query(tags);
+    query += "WHERE parts.MPN LIKE $mpn "
              "AND parts.manufacturer LIKE $manufacturer "
              "AND parts.description LIKE $desc "
              "AND (parts.entity=$entity or $entity_all) ";
-    query << sort_controller->get_order_by();
-    std::cout << query.str() << std::endl;
-    SQLite::Query q(pool.get_db(), query.str());
+    if (base_parts_cb->get_active() == false) {
+        query += "AND flag_base_part = 0 ";
+    }
+    query += get_pool_selector_query();
+    query += sort_controller->get_order_by();
+    SQLite::Query q(pool.get_db(), query);
     q.bind("$mpn", "%" + MPN_search + "%");
     q.bind("$desc", "%" + desc_search + "%");
     q.bind("$manufacturer", "%" + manufacturer_search + "%");
     q.bind("$entity", entity_uuid);
     q.bind("$entity_all", entity_uuid == UUID());
-    int i = 0;
-    for (const auto &it : tags) {
-        q.bind(("$tag" + std::to_string(i)).c_str(), it);
-        i++;
-    }
-    if (tags.size())
-        q.bind("$ntags", tags.size());
+    bind_tags_query(q, tags);
+    bind_pool_selector_query(q);
 
     if (show_none) {
         row = *(store->append());
@@ -152,7 +158,7 @@ void PoolBrowserPart::search()
             row[list_columns.tags] = q.get<std::string>(4);
             row[list_columns.path] = q.get<std::string>(5);
             row[list_columns.description] = q.get<std::string>(6);
-            row[list_columns.source] = pool_item_source_from_db(q.get<std::string>(7), q.get<int>(8));
+            row[list_columns.source] = pool_item_source_from_db(q, 7, 8);
         }
         set_busy(false);
     }
