@@ -3,10 +3,12 @@
 #include "util/sqlite.hpp"
 
 namespace horizon {
-PoolBrowserSymbol::PoolBrowserSymbol(IPool &p, const UUID &uu) : PoolBrowser(p), unit_uuid(uu)
+PoolBrowserSymbol::PoolBrowserSymbol(IPool &p, const UUID &uu, const std::string &instance)
+    : PoolBrowser(p, TreeViewStateStore::get_prefix(instance, "pool_browser_package")), unit_uuid(uu)
 {
     construct();
-    name_entry = create_search_entry("Name");
+    name_entry = create_search_entry("Name", create_pool_selector());
+    focus_widget = name_entry;
     install_pool_item_source_tooltip();
 }
 
@@ -17,9 +19,23 @@ Glib::RefPtr<Gtk::ListStore> PoolBrowserSymbol::create_list_store()
 
 void PoolBrowserSymbol::create_columns()
 {
-    append_column_with_item_source_cr("Symbol", list_columns.name);
-    treeview->append_column("Unit", list_columns.unit_name);
-    treeview->append_column("Manufacturer", list_columns.unit_manufacturer);
+    {
+        auto col = append_column_with_item_source_cr("Symbol", list_columns.name, Pango::ELLIPSIZE_END);
+        col->set_resizable(true);
+        col->set_min_width(150);
+    }
+    {
+        auto col = append_column("Unit", list_columns.unit_name, Pango::ELLIPSIZE_END);
+        col->set_resizable(true);
+        col->set_min_width(50);
+    }
+    {
+        auto col = append_column("Manufacturer", list_columns.unit_manufacturer, Pango::ELLIPSIZE_END);
+        col->set_resizable(true);
+        col->set_min_width(50);
+    }
+    path_column = append_column("Path", list_columns.path, Pango::ELLIPSIZE_START);
+    install_column_tooltip(*path_column, list_columns.path);
 }
 
 void PoolBrowserSymbol::add_sort_controller_columns()
@@ -27,8 +43,6 @@ void PoolBrowserSymbol::add_sort_controller_columns()
     sort_controller->add_column(0, "symbols.name");
     sort_controller->add_column(1, "units.name");
     sort_controller->add_column(2, "units.manufacturer");
-    path_column = append_column("Path", list_columns.path, Pango::ELLIPSIZE_START);
-    install_column_tooltip(*path_column, list_columns.path);
 }
 
 void PoolBrowserSymbol::set_unit_uuid(const UUID &uu)
@@ -45,14 +59,15 @@ void PoolBrowserSymbol::search()
 
     std::string query =
             "SELECT symbols.uuid, symbols.name, units.name, "
-            "units.manufacturer, symbols.filename, symbols.pool_uuid, symbols.overridden FROM symbols,units WHERE "
+            "units.manufacturer, symbols.filename, symbols.pool_uuid, symbols.last_pool_uuid FROM symbols,units WHERE "
             "symbols.unit = units.uuid AND (units.uuid=? OR ?) AND "
             "symbols.name LIKE ?"
-            + sort_controller->get_order_by();
+            + get_pool_selector_query() + sort_controller->get_order_by();
     SQLite::Query q(pool.get_db(), query);
     q.bind(1, unit_uuid);
     q.bind(2, unit_uuid == UUID());
     q.bind(3, "%" + name_search + "%");
+    bind_pool_selector_query(q);
 
     Gtk::TreeModel::Row row;
     if (show_none) {
@@ -69,7 +84,7 @@ void PoolBrowserSymbol::search()
             row[list_columns.unit_name] = q.get<std::string>(2);
             row[list_columns.unit_manufacturer] = q.get<std::string>(3);
             row[list_columns.path] = q.get<std::string>(4);
-            row[list_columns.source] = pool_item_source_from_db(q.get<std::string>(5), q.get<int>(6));
+            row[list_columns.source] = pool_item_source_from_db(q, 5, 6);
         }
         set_busy(false);
     }
