@@ -1,6 +1,6 @@
-major = 1
-minor = 4
+major = 2
+minor = 1
 micro = 0
-name = "Étoile"
+name = "Green Flash"
 
 string = ".".join(str(x) for x in (major, minor, micro))
